/**
 * @copyright Mamba ( http://corp.mamba.ru/ )
 */

mambo.use('{mambo.ui}Input', function () {
	"use strict";

	mambo.ui = mambo.ui || {};

	var instances = [],
		_uid = 0;

	function create(params) {

		var $layer = $(params.layer);

		if (!$layer.length) return;

		var $layerTextarea = $('textarea', $layer),
			$layerSubmit = $('[type="submit"]', $layer),
			$layerForm = $('form', $layer),
			$layerButtonOk = $('.js-button-ok', $layer),
			$textarea = $(params.textarea),
			$submit = $(params.submit);
        
		var linkSelector = params.linkSelector,
			valueSelector = params.valueSelector,
			defaultValue,
			layerTextareaValueMaxLength = Number($layerTextarea.attr('maxlength')) || 0;

		Mb.UIFramework.ModalManager.getInstance().watch({
			cssClickSelector: linkSelector,
			modalParams: {
				content: {
					htmlElement: $layer
				},
				removeResetCssClass: true
			}
		}, function (modal) {
            $(modal.scope).addClass('modal-rejected');
            
            $layerButtonOk.on('click', function (e) {
				e.preventDefault();
                modal.dispose();
			});
            
			$layerSubmit.on('disable', function () {
				$(this).attr('disabled', true).addClass('disabled');
			}).on('enable', function () {
				$(this).removeAttr('disabled').removeClass('disabled');
			}).on('click', function () {
				if ($textarea.length && $submit.length) {
					$textarea.val($layerTextarea.val());
					$submit.trigger('click');
				}
			});
            
            if (params.submitHandler && $layerForm.length) {
                $layerForm.on('submit', params.submitHandler);
                $layerForm.on('submit', function() {
                    modal.dispose();
                });
            }
            
            if (params.$form) {
                params.$form.on('submit', function() {
                    modal.dispose();
                });
            }
		});


		function binds() {

			if (!($layerTextarea.length && $layerSubmit.length)) return;

			_onInput($layerTextarea, function () {
				var origVal = $(this).attr('data-orig-value');
				var val = $(this).val();

				if (layerTextareaValueMaxLength && (val.length > layerTextareaValueMaxLength)) {
					$(this).val(val.substr(0, layerTextareaValueMaxLength));
					if ($.browser.msie && parseInt($.browser.version) <= 8) {
						setTimeout(function() {
							$layerTextarea.scrollTop(999);
						}.bind(this), 10);
					}
				}

				if (origVal && val === origVal) {
					$layerSubmit.trigger('disable');
				} else {
					$layerSubmit.trigger('enable');
				}
			});

			if (linkSelector) {
				$(document).on('click', linkSelector, function (e) {
					e.preventDefault();

					if (valueSelector) {
						defaultValue = $(valueSelector).val() || 
                            ($(valueSelector).html() && $(valueSelector).html().replace(/<br[\s/]*>/ig, '\n'));
					} else if ($textarea.length) {
						defaultValue = $textarea.val();
					}

					$layerTextarea.attr('data-orig-value', defaultValue);

					$layerTextarea.val('').focus();
				});
			}
		}

		binds();

		return {};
	}

	function _onInput($field, callback) {

		if ($.browser.msie && parseInt($.browser.version) <= 9) {
			(function inputEvent() {
				var inputedValue = $field.val();
				function fire() {
					if (inputedValue !== $field.val()) {
						callback.call($field);
						inputedValue = $field.val();
					}
				}
				$field.on('paste keyup', function (e) {
					fire();
				});
			}());
		} else {
			$field.on('input', function (e) {
				callback.call($field);
			});
		}
	}

	mambo.ui.BlockRejected = function (params) {

		var instance,
			uid = _uid++;

		params = $.extend({}, params);

		instance = create(params);

		instances[uid] = instance;

		return instance;
	};

	mambo.ready('{mambo.ui}BlockRejected');

}, true);