/**
 * Uploader
 *
 * @version 2.0
 * @copyright Mamba ( http://corp.mamba.ru/ )
 */

/* jshint undef:true, unused:true, forin:true, latedef:true, newcap:true, noarg:true, nonew:true, lastsemic:true */
/* global window:false, document:false, XMLHttpRequest:false, mambo:false, stats:false */

mambo.use(function ()
{
	var uploader = {},
		_uid = 0,
		instancesHtml5 = [],
		instancesIframe = [],
		instancesUrlProvider = [];

	/*
	 * Static properties
	 */

	uploader.checkFileApiSupport = function ()
	{
		return Boolean( (window.XMLHttpRequest || window.XDomainRequest) && (window.FormData || window.FileReader && (window.XMLHttpRequest && XMLHttpRequest.sendAsBinary || window.ArrayBuffer && window.Uint8Array && (window.MozBlobBuilder || window.WebKitBlobBuilder || window.BlobBuilder))) );
	};

	uploader.checkWindowURLSupportInBrowser = function ()
	{
		window.URL = window.URL || window.webkitURL;// standardization to same view
		return (window.URL && window.URL.revokeObjectURL && window.URL.createObjectURL);
	};

	uploader.checkFileReaderSupportInBrowser = function ()
	{
		return (window.FileReader && (window.btoa && window.atob));
	};

	uploader.checkCanvasSupportInBrowser = function ()
	{
		var testCanvas = document.createElement("canvas");
		return (testCanvas.getContext && testCanvas.getContext("2d"));
	};

	uploader.checkImagePreprocessingPossibilitySupport = function ()
	{
		return ( uploader.checkCanvasSupportInBrowser() && ( uploader.checkWindowURLSupportInBrowser() || uploader.checkFileReaderSupportInBrowser() ) );
	};

	uploader.FakeFile = function () { };

	uploader.FakeResponse = function ()
	{
		var obj = {
			"s": 1,
			"e": 0,
			"d": {},
			"r": 0,
			"XForms": 0
		};

		return obj;
	}


	/*
	 * Upload by html5
	 */

	uploader.Html5 = function (btn, opt) // {{{
	{
		var url = opt.url,
			$trigger = opt.triggerEl,
			maxFileSize = opt.maxFileSize || 100000000,
			leftFiles = opt.leftFiles,
			lang = opt.lang,
			obj = {},
			imagePreprocesingFlag = (opt.preprocessing && uploader.checkImagePreprocessingPossibilitySupport()),
			windowURLSupportInBrowserFlag = uploader.checkWindowURLSupportInBrowser();

		function setUrl(_url) {
			url = _url || opt.url;
			return url;
		}

		function filesUpload(files) {

			files = checkFiles(files, maxFileSize, lang);
			if (!files.length) {
				return;
			}
			if (leftFiles && files.length > leftFiles) {
				alert(lang.maxFiles);
				return;
			}
			$trigger.trigger({
				type: 'onStartXHRFileUpload',
				files: files
			});
			if ($trigger.data('preUploadConditionsErrors')) {// check errors in preload conditions flag
				$trigger.data('preUploadConditionsErrors', false);// reset errors in preload conditions flag
				return;
			}
			var XHR = window.XMLHttpRequest || window.XDomainRequest,
				uploadFiles = 0,
				remainFiles = files.length;

			var checkRemainFiles = function (e, file) {
				if (!remainFiles) {
					$trigger.trigger({
						type: 'onFinishXHRFileUpload',
						e: e,
						file: file,
						uploadFiles: uploadFiles
					});
				}
			};

			// TODO
			var xhrToAbort = [];

			$trigger.on('onAbortXHRFileUpload', function () {
				for (var i=0, loop_length = xhrToAbort.length; i < loop_length; i++) {
					xhrToAbort[i].abort();
				}
				xhrToAbort.length = 0;
				files.length = 0;
			});

			$trigger.on('onStartXHR', function (data) {
				xhrToAbort.push(data.xhr);
			});

			function filesToUploadQueue(queueFiles, $queueTrigger) {
				var ii;
				if (!opt.parallel) {/* progressive files upload */
					if (queueFiles.length) {
						ii = 0;
						$queueTrigger.on('onCompleteXHR', function (data) {
							ii++;
							if (queueFiles[ii]) {
								fileUpload(queueFiles[ii]);
							}
						});
						fileUpload(queueFiles[ii]);
					}
				} else {/* parallel files upload */
					var len = queueFiles.length;
					for (ii = 0; ii < len; ii++) {
						fileUpload(queueFiles[ii]);
					}
				}
			}

			function fileStartLoad(file, xhr) {
				$trigger.trigger({
					type: 'onStartXHR',
					file: file,
					xhr: xhr
				});
			}

			function fileLoad(e, file) {
				uploadFiles++;
				remainFiles--;
				$trigger.trigger({
					type: 'onLoadXHR',
					e: e,
					file: file
				});
				checkRemainFiles(e, file);
			}

			function fileError(e, file) {
				remainFiles--;
				$trigger.trigger({
					type: 'onErrorXHR',
					e: e,
					file: file
				});
				checkRemainFiles(e, file);
			}

			function fileProgress(e, file) {
				$trigger.trigger({
					type: 'onProgressXHR',
					e: e,
					file: file
				});
			}

			function fileComplete(e, file) {/*error or success*/
				$trigger.trigger({
					type: 'onCompleteXHR',
					e: e,
					file: file,
					files: files
				});
			}

			function fileUpload(file) {
				if (imagePreprocesingFlag) {
					preprocessFile({
						file: file,
						success: function (postProcessedFile) {
							processingFile(postProcessedFile);
						},
						error: function (brokenFile) {
							$trigger.trigger({
								type: 'onErrorXHR',
								e: file,
								file: file
							});
							processingFile(file);
						}
					});
				} else {
					processingFile(file);
				}
			}

			function sendAtobData(file, atobData) {
				try {
					var xhr = new XHR();
					xhr.onload = function (e) {
						fileLoad(e, file);
						fileComplete(e, file);
					};
					xhr.onerror = function (e) {
						fileError(e, file);
						fileComplete(e, file);
					};
					xhr.upload.onprogress = function (e) {
						fileProgress(e, file);
					};
					xhr.open('POST', url, true);
					var boundary = '---------' + mambo.utils.math.getRandomInt(1111111111, 9999999999);
					/*ToDo options._file_name*/
					var name = mambo.utils.translit(file.name).replace(/\W/g, '');

					xhr.setRequestHeader("Content-Type", "multipart/form-data, boundary=" + boundary);

					var body = [];
					body.push("--" + boundary);
					body.push("Content-Disposition: form-data; name='" + name + "'; filename='" + name);
					body.push("Content-Type: application/octet-stream");
					body.push("");
					body.push(atobData);
					body.push("--" + boundary);

					xhr.sendAsBinary(body.join('\r\n'));
					fileStartLoad(file, xhr);
				} catch (err) {
					if (window.stats) {
						stats.jsErrorPush(err);
					}
					sendFormData(file);
				}
			}

			function sendFormData(file) {
				if (window.FormData) {
					var formData = new FormData();
					formData.append("images[]", file);
					var xhr = new XHR();
					xhr.open('POST', url, true);
					xhr.onload = function (e) {
						fileLoad(e, file);
						fileComplete(e, file);
					};
					xhr.onerror = function (e) {
						fileError(e, file);
						fileComplete(e, file);
					};
					xhr.upload.onprogress = function (e) {
						fileProgress(e, file);
					};
					xhr.send(formData);
					fileStartLoad(file, xhr);
				} else {
					fileError({}, file);
					fileComplete({}, file);
				}
			}

			function sendDataByAtob(file) {
				try {
					if (XHR && !XHR.prototype.sendAsBinary && window.ArrayBuffer && window.Uint8Array) {
						var BlobBuilder = window.MozBlobBuilder || window.WebKitBlobBuilder || window.BlobBuilder;
						if (BlobBuilder || window.Blob) {
							XHR.prototype.sendAsBinary = function (text) {
								var data = new ArrayBuffer(text.length);
								var ui8a = new Uint8Array(data, 0);
								var i = 0;
								for (; i < text.length; i++) {
									ui8a[i] = (text.charCodeAt(i) & 0xff);
								}
								if (window.Blob) {
									var blob = new Blob([data]);
								} else {
									var bb = new BlobBuilder();
									bb.append(data);
									var blob = bb.getBlob();
								}
								this.send(blob);
							};
						}
					}
					if (file.atob) {
						sendAtobData(file, file.atob);
					} else {
						var reader = new FileReader();
						reader.onload = function () {
							sendAtobData(file, reader.result);
						};
						reader.onerror = function (e) {
							fileError(e, file);
							fileComplete(e, file);
						}
						reader.onabort = function (e) {
							fileError(e, file);
							fileComplete(e, file);
						}
						reader.readAsBinaryString(file);
					}
				} catch (err) {
					if (window.stats) {
						stats.jsErrorPush(err);
					}
					sendFormData(file);
				}
			}

			function processingFile(file) {
				if (!imagePreprocesingFlag) {// without preprocessing
					if (window.FormData) {
						sendFormData(file);
					} else {
						sendDataByAtob(file);
					}
				} else {// with preprocessing
					if (window.FormData) {
						sendFormData(file);
					} else {
						sendDataByAtob(file);
					}
				}
			}

			function imageLoadingComplete(src) {
				if (windowURLSupportInBrowserFlag) {
					window.URL.revokeObjectURL(this.src);
				}
			}

			function preprocessImage(obj, src) {
				var file = obj.file,
					processedImage = new Image();

				processedImage.onload = function (e) {
					var img = this;
					imageLoadingComplete(src);

					/* START PROCESSING */
					// recalc width/height
					var width = parseInt(img.width, 10),
						height = parseInt(img.height, 10),
						aspect = width / height,
						prep = opt.preprocessing,
						prepWidth = prep.maxWidth || 600,
						prepHeight = prep.maxHeight || 450,
						prepQuality = Number(((prep.quality || 85) / 100).toFixed(2)),
						myWidth,
						myHeight;

					if (aspect >= 1) {
						myWidth = parseInt((width <= prepWidth) ? width : prepWidth, 10);
						myHeight = parseInt(myWidth / aspect, 10);
					} else {
						myHeight = parseInt((height <= prepHeight) ? height : prepHeight, 10);
						myWidth = parseInt(myHeight * aspect, 10);
					}

					var canvas = document.createElement('canvas'),
						ctx = canvas.getContext('2d');

					var oc = document.createElement('canvas'),
						octx = oc.getContext('2d');
					oc.width = width;
					oc.height = height;
					octx.drawImage(img, 0, 0, oc.width, oc.height);

					var countDraw = 0,
						stepDraw = 0.99;

					while (oc.width / (countDraw / 3 + 1) > myWidth && oc.height / (countDraw / 3 + 1) > myHeight) {
						octx.drawImage(oc, 0, 0, oc.width * stepDraw, oc.height * stepDraw);
						countDraw++;
					}

					canvas.width = myWidth;
					canvas.height = myHeight;
					ctx.drawImage(oc, 0, 0, oc.width * Math.pow(stepDraw, countDraw), oc.height * Math.pow(stepDraw, countDraw), 0, 0, canvas.width, canvas.height);

					//function contrastImage(imageData, contrast) {
					//	var data = imageData.data;
					//	var factor = (259 * (contrast + 255)) / (255 * (259 - contrast));
					//	for(var i=0;i<data.length;i+=4)
					//	{
					//		data[i] = (factor * ((data[i] - 128) + 128));
					//		data[i+1] = (factor * ((data[i+1] - 128) + 128));
					//		data[i+2] = (factor * ((data[i+2] - 128) + 128));
					//	}
					//	return imageData;
					//}
					//
					//ctx.putImageData(contrastImage(ctx.getImageData(0,0,canvas.width,canvas.height),20),0,0);

					// canvas to atob
					var DataURL;
					try {// try to prevent bug in old FF with throw security exception 1000 (in that case compression rank isn't declarate)
						DataURL = canvas.toDataURL('image/jpeg', prepQuality);
					} catch (err) {
						DataURL = canvas.toDataURL('image/jpeg');
						if (window.stats) {
							stats.jsErrorPush(err);
						}
					}

					var ReplacedString = 'base64,',
						postProcessedDataURL = DataURL.substring((DataURL.indexOf(ReplacedString) + ReplacedString.length));
					file.atob = atob(postProcessedDataURL);

					/* END PROCESSING */
					obj.success(file);
				};

				processedImage.src = src;

				processedImage.onerror = function (e) {
					imageLoadingComplete(src);
					obj.error(file);
				};
			}

			function preprocessFile(obj) {
				var file = obj.file;
				/* READ IMAGE SRC */
				var imageSrcGivenByWindowURL;
				if (windowURLSupportInBrowserFlag) {// get window.URL, if possible
					imageSrcGivenByWindowURL = window.URL.createObjectURL(file);
				}
				if (imageSrcGivenByWindowURL && imageSrcGivenByWindowURL.length) {// process image by window.URL src
					preprocessImage(obj, imageSrcGivenByWindowURL);
				} else {// process image by FileREader src
					var oFReader = new FileReader();
					oFReader.onload = function (oFREvent) {
						try {
							var procesedDataURLPrefix = "data:" + (file.type || "image/png") + ";base64,";
							var procesedDataURL = procesedDataURLPrefix + btoa(oFREvent.target.result);
						} catch(ex) {}
						procesedDataURL ? preprocessImage(obj, procesedDataURL) : obj.error(file);
					}
					oFReader.onerror = function () {obj.error(file);}
					oFReader.onabort = function () {obj.error(file);}
					oFReader.readAsBinaryString(file);
				}
			}

			filesToUploadQueue(files, $trigger);// send files to upload queue
		}

		function checkFiles(files, maxFileSize, lang) {
			var fileArray = [];
			if (files && files.length) {
				var i = 0,
					len = files.length,
					rFilter = new RegExp('^(' + (opt.mimeFilter || 'image\/bmp|image\/gif|image\/jpeg|image\/jpg|image\/png') + ')$', 'i');
				for (; i < len; i++) {
					var _file = files[i];

					if (!_file.size) {// file size
						//alert(lang.error);
						continue;
					} else if (_file.size > maxFileSize) {
						//alert(lang.maxFileSize);
						continue;
					}
					fileArray.push(_file);
				}
			}
			return fileArray;
		}

		function is(obj, type) {
			return typeof obj === type;
		}

		function isEventSupported(eventName) {/* function from http://yura.thinkweb2.com/isEventSupported/ */
			var element = document.createElement('div'),
				eventName = 'on' + eventName;
			// When using `setAttribute`, IE skips "unload", WebKit skips "unload" and "resize", whereas `in` "catches" those
			var isSupported = eventName in element;
			if (!isSupported) {
				// If it has no `setAttribute` (i.e. doesn't implement Node interface), try generic element
				if (!element.setAttribute) {
					element = document.createElement('div');
				}
				if (element.setAttribute && element.removeAttribute) {
					element.setAttribute(eventName, '');
					isSupported = is(element[eventName], 'function');
					// If property was created, "remove it" (by setting value to `undefined`)
					if (!is(element[eventName], undefined)) {
						element[eventName] = undefined;
					}
					element.removeAttribute(eventName);
				}
			}
			element = null;
			return isSupported;
		}

		function bindDragAndDrop(options) {
			if (isEventSupported('dragstart') && isEventSupported('drop')) {// drag and drop is supported by browser
				options = options || {};
				var boxToDropIn = options.boxToDropIn || $(document.body),
					showBoxIfDragEnabled = options.showBoxIfDragEnabled || $('.drag-and-drop-message'),
					timer,
					dragMouseInClass = options.dragMouseInClass || 'drag-and-drop-in-region',
					dragMouseInWindowClass = options.dragMouseInWindowClass || 'drag-and-drop-in-window',
					isSupportedClass = 'drag-and-drop-is-supported',
					$html = $(document.documentElement);

				showBoxIfDragEnabled.show();

				function addHighlight() {
					$html.addClass(dragMouseInClass);
				}

				function hideHighlight() {
					$html.removeClass(dragMouseInClass);
				}

				function addWindowHighlight() {
					$html.addClass(dragMouseInWindowClass);
				}

				function hideWindowHighlight() {
					$html.removeClass(dragMouseInWindowClass);
				}

				function addIsSupportedClass() {
					$html.addClass(isSupportedClass);
				}

				function removeIsSupportedClass() {
					$html.removeClass(isSupportedClass);
				}

				var dragOver = function (e) {
					clearTimeout(timer);
					// timer = setTimeout(hideHighlight, 100);
					var dt = e.originalEvent.dataTransfer;
					if (!dt) {
						return;
					}
					if ($.browser.mozilla && dt.types.contains && !dt.types.contains("Files")) {//FF
						return;
					}
					if (navigator.userAgent.toLowerCase().indexOf('chrome') > -1 && dt.types.indexOf && dt.types.indexOf("Files") == -1) {//Chrome
						return;
					}
					if ($.browser.webkit) {//Webkit
						dt.dropEffect = 'copy';
					}

					addHighlight();
					addWindowHighlight();

					return false;
				};

				var dragDrop = function (e) {
					var dt = e.originalEvent.dataTransfer;
					if (!dt && !dt.files) {
						return;
					}

					hideHighlight();
					hideWindowHighlight();

					filesUpload(dt.files);

					return false;
				};

				$(document)
					.off('custom.addDragEndDrop').on('custom.addDragEndDrop', function () {
						addIsSupportedClass();

						boxToDropIn
							.off("dragover.photoAlbum").on("dragover.photoAlbum", dragOver)
							.off("dragenter.photoAlbum").on("dragenter.photoAlbum", function () {return false;})
							.off('dragleave.photoAlbum').on('dragleave.photoAlbum', function () {
								clearTimeout(timer);
								timer = setTimeout(hideHighlight, 100);
							})
							.off("drop").on("drop", dragDrop);

						$(document)
							.off('dragenter.photoAlbum').on('dragenter.photoAlbum', function () {
								addWindowHighlight();
								return false;
							})
							.off('dragleave.photoAlbum').on('dragleave.photoAlbum', function () {
								hideWindowHighlight();
								return false;
							})
							.off('dragover.photoAlbum').on('dragover.photoAlbum', function (e) {
								addWindowHighlight();
								var dt = e.originalEvent.dataTransfer;
								if (!dt) { return; }
								if (!($.browser.webkit && /win/.test(navigator.platform.toLowerCase()))) {
									dt.dropEffect = 'none';
								}
								return false;
							});
					})
					.off('custom.removeDragEndDrop').on('custom.removeDragEndDrop', function () {
						removeIsSupportedClass();
						$(document).add(boxToDropIn).off('dragenter.photoAlbum dragleave.photoAlbum dragover.photoAlbum');
					});
				$(document).trigger('custom.addDragEndDrop');
			}
		}

		function unbindDragAndDrop(boxToDropIn) {
			$(document).add(boxToDropIn || $(document.body)).off('dragenter.photoAlbum dragleave.photoAlbum dragover.photoAlbum');
		}

		function abortUpload() {
			$trigger.trigger('onAbortXHRFileUpload');
		}

		/*
		 * Public Methods
		 */

		obj = {
			setUrl: setUrl,
			filesUpload: filesUpload,
			bindDragAndDrop: bindDragAndDrop,
			unbindDragAndDrop: unbindDragAndDrop,
			abortUpload: abortUpload
		};

		instancesHtml5[btn] = obj;

		return obj;
	};

	uploader.html5 = function (btn, opt) {
		var up = uploader.getHtml5(btn);
		if (!up) {
			up = new uploader.Html5(btn, opt);
		}
		return up;
	};

	uploader.getHtml5 = function (btn) {
		return instancesHtml5[btn];
	};
	// }}}


	/*
	 * Upload by iframe
	 */

	uploader.Iframe = function (btn, opt) // {{{
	{
		var	uid = _uid++,
			url = opt.url,
			form,
			iframeName = 'iframeFileUpload' + uid,

			$iframe = createIframe(iframeName),
			$trigger = opt.triggerEl;

		function init() {
			$(document.body).append($iframe);

			opt.form.action = url;
			opt.form.target = iframeName;
			opt.form.enctype = 'multipart/form-data';
			opt.form.encoding = 'multipart/form-data';
		}

		function setUrl(_url) {
			url = _url || opt.url;

			opt.form.action = url;

			return url;
		}

		function fileUpload(file) {

			if (file === undefined) {
				return;
			}

			$trigger.trigger({
				type: 'onStartIframeFileUpload',
				file: file
			});

			opt.form.submit();

			bindIframe(file);

			fileStartLoad(file);

			if (opt.isFakeProgress) {
				fileForceFakeProgress(file);
			}
		}

		function fileStartLoad(file) {
			$trigger.trigger({
				type: 'onStartIframe',
				file: file
			});
		}

		function fileLoad(e, win, file) {
			$trigger.trigger({
				type: 'onLoadIframe',
				e: e,
				win: win,
				file: file
			});
		}

		function fileError(e, file) {
			$trigger.trigger({
				type: 'onErrorIframe',
				e: e,
				file: file
			});
		}

		function fileComplete(e, file) {
			$trigger.trigger({
				type: 'onCompleteIframe',
				e: e,
				file: file
			});
			$trigger.trigger({
				type: 'onFinishIframeFileUpload',
				e: e,
				file: file
			});

			fileForceFakeProgress(file, 100);
		}

		function fileProgress(file, progress) {
			$trigger.trigger({
				type: 'onProgressIframe',
				file: file,
				progress: progress
			});
		}

		function fileForceFakeProgress(file, progress, _timer) {
			var PROGRESS_MAX = 98;
			var progress = (progress && progress > PROGRESS_MAX) ? PROGRESS_MAX : (progress || 0),
				step = 1,
				timeout = 150,
				// step = mambo.utils.math.getRandomInt(1, 5),
				// timeout = mambo.utils.math.getRandomInt(100, 1000),
				timer;

			fileProgress(file, progress);

			if (!_timer && fileForceFakeProgress.timer) {
				clearTimeout(fileForceFakeProgress.timer);
			}

			if (progress < PROGRESS_MAX) {
				timer = setTimeout(function() {
					progress += step;
					fileForceFakeProgress(file, progress, timer);
				}, timeout);
				fileForceFakeProgress.timer = timer;
			} else {
				fileForceFakeProgress.timer = false;
			}
		}

		function createIframe(name) {
			return $('<iframe name="' + name + '" style="display:none"></iframe>');
		}

		function destroyIframe($iframe) {
			return $iframe.remove();
		}

		function bindIframe(file) {
			$iframe.unbind('load').bind('load', function (e) {
				fileLoad(e, frames[iframeName], file);
				fileComplete(e, file);
			});

			$iframe.unbind('error').bind('error', function (e) {
				fileError(e, file);
				fileComplete(e, file);
			});
		}

		function abortUpload() {
			$trigger.trigger('onAbortIframeFileUpload');
		}

		init();

		/*
		 * Public Methods
		 */

		obj = {
			setUrl: setUrl,
			fileUpload: fileUpload,
			abortUpload: abortUpload
		};

		instancesIframe[btn] = obj;

		return obj;
	};

	uploader.iframe = function (btn, opt)
	{
		var up = uploader.getIframe(btn);
		if (!up) {
			up = new uploader.Iframe(btn, opt);
		}
		return up;
	};

	uploader.getIframe = function (btn)
	{
		return instancesIframe[btn];
	};
	// }}}


	/*
	 * Upload by url
	 */

	uploader.UrlProvider = function (btn, opt) // {{{
	{
		var	uid = _uid++,
			url,
			albumId,
			photos = {loaded: [], failed: []},
			files = new FakeFiles(),
			obj,

			queueId, queueUrl, queueTimer,

			$trigger = opt.triggerEl;

		function init()
		{
			setAlbumId();
			setUrl();
		}

		function setUrl(_url)
		{
			url = _url || opt.url;

			return url;
		}

		// TODO
		function setAlbumId(id)
		{
			albumId = id || opt.albumId;

			return albumId;
		}

		function FakeFiles()
		{
			var files = [];

			this.addByUrl = function(urls)
			{
				for (var i=0, len=urls.length; i < len; i++)
				{
					var file = new uploader.FakeFile();
					file.origin = urls[i];
					files[i] = file;
				}
			}

			this.getByUrl = function(urls)
			{
				var arr = [];

				for (var i=0; i < urls.length; i++)
				{
					var url = urls[i];

					for (var j=0; j < files.length; j++)
					{
						var file = files[j];

						if (file.origin === url)
						{
							arr.push(file);
						}
					}
				}

				return arr;
			}

			this.getByPhotos = function(photos)
			{
				var arr = [];

				if (photos && photos.length)
				{
					for (var i=0; i < photos.length; i++)
					{
						var photo = photos[i];

						for (var j=0; j < files.length; j++)
						{
							var file = files[j];

							if (file.origin === photo.origin)
							{
								arr.push(file);
							}
						}
					}
				}

				return arr;
			}

			this.subtract = function(_files)
			{
				var arr = [];

				for (var i=0; i < files.length; i++)
				{
					var file = files[i];
					var eq = false;

					for (var j=0; j < _files.length; j++)
					{
						var _file = _files[j];

						if (file === _file)
						{
							eq = true;
							break;
						}
					}

					if (!eq)
					{
						arr.push(file);
					}
				}

				return arr;
			}

			this.get = function()
			{
				return files;
			}
		}

		function subtractByProp(arr, arr2, prop)
		{
			var diff = [];

			for (var i = 0, arr_max = arr.length; i < arr_max; i++)
			{
				var item = arr[i],
					uniq = true;

				for (var j = 0, arr2_max = arr2.length; j < arr2_max; j++)
				{
					var item2 = arr2[j];

					if (item[prop] === item2[prop])
					{
						uniq = false;
						break;
					}
				}

				if (uniq)
				{
					diff.push(item);
				}
			}

			return diff;
		}

		function uploadByUrl(urls)
		{
			var xhr = new XMLHttpRequest;

			files.addByUrl(urls);

			$trigger.on('abort', function()
			{
				if (queueId)
				{
					var abortXHR = new XMLHttpRequest();
					var abortUrl = opt.abortUrl.replace(/:pid/, queueId);

					abortXHR.open('GET', abortUrl);
					abortXHR.send(null);

					clearTimeout(queueTimer);
					queueId = null;
					queueUrl = null;
				}
				else
				{
					xhr.abort();
				}
			});

			xhr.onload = function (e)
			{
				if (xhr.responseText)
				{
					var response = {};

					try {
						response = JSON.parse(xhr.responseText);
					} catch (e) {
						throw e;
					}

					if (response.s)
					{
						if (response.d && response.d.queue)
						{
							queueId = String(response.d.queue);
							queueUrl = response.d.queue_url;

							processQueue(queueUrl);
						}
						else
						{
							//TODO: FOR TEST ONLY!!!
							// var _files = files.get();
							// for (var i=0; i < response.d.photo.length; i++)
							// {
							// 	var photo = response.d.photo[i];
							// 	var file = _files[i];

							// 	photo.origin = file.origin;
							// }
							//TODO

							var loadedFiles = files.getByPhotos(response.d.photo),
								failedFiles = files.subtract(loadedFiles);

							processFiles(loadedFiles, failedFiles, response.d.photo);

							finishUpload(files);
						}
					}
					else
					{
						errorUpload();
						finishUpload(files);
					}
				}
				else
				{
					errorUpload();
					finishUpload(files);
				}
			};

			xhr.onerror = function (e)
			{
				errorUpload();
				finishUpload(files);
			};

			if (xhr.upload)
			{
				xhr.upload.onprogress = function (e)
				{
					filesProgress(e, files);
				}
			}

			xhr.open('POST', url, true);

			xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');

			xhr.send(prepareDataToSend(urls));

			startUpload(files.get());
		}

		function prepareDataToSend(urls)
		{
			var arr = [],
				str = '';

			// Make array
			for (var i=0; i < urls.length; i++)
			{
				var item = ["photo[]", urls[i]];
				arr.push(item);
			}
			arr.push(["album_id", albumId]);

			// Encode and join array values to string
			for (var i=0; i < arr.length; i++)
			{
				arr[i] = encodeURIComponent(arr[i][0]) + '=' + encodeURIComponent(arr[i][1]);
			}

			// Make string from array
			str = arr.join('&');
			str = str + opt.sPost;

			return str;
		}

		function startUpload(files)
		{
			$trigger.trigger({
				type: 'start',
				files: files
			});

			for (var i=0; i < files.length; i++)
			{
				fileStart(files[i]);
			}
		}

		function finishUpload(files)
		{
			$trigger.trigger({
				type: 'finish',
				files: files.get()
			});
		}

		function errorUpload()
		{
			$trigger.trigger({
				type: 'error'
			});
		}

		function problemsUpload()
		{
			$trigger.trigger({
				type: 'problems'
			});
		}

		function abortUpload()
		{
			$trigger.trigger('abort');
		}

		function timeoutUpload()
		{
			$trigger.trigger({
				type: 'timeout'
			});
		}

		function filesProgress(e, files)
		{
			$trigger.trigger({
				type: 'progress',
				files: files.get()
			});
		}

		function fileStart(file)
		{
			$trigger.trigger({
				type: 'startFile',
				file: file
			});
		}

		function fileLoad(file, response)
		{
			$trigger.trigger({
				type: 'loadFile',
				file: file,
				_response: response
			});
		}

		function fileError(file, response)
		{
			$trigger.trigger({
				type: 'errorFile',
				file: file,
				_response: response
			});
		}

		function fileComplete(file)
		{
			$trigger.trigger({
				type: 'completeFile',
				file: file
			});
		}

		function processFiles(loadedFiles, failedFiles, loadedResponses, failedResponses)
		{
			for (var i=0; i < loadedFiles.length; i++)
			{
				var file = loadedFiles[i];
				var fileResponse = $.extend(new uploader.FakeResponse(), {
					"d": {
						"photo": loadedResponses[i]
					}
				});

				fileLoad(file, fileResponse);
				fileComplete(file);
			}

			processErrorFiles(failedFiles, failedResponses);
		}

		function processErrorFiles(list, responses)
		{
			for (var i=0; i < list.length; i++)
			{
				var file = list[i];
				var fileResponse = $.extend(new uploader.FakeResponse(), {
					"s": 0,
					"XForms": 1,
                    "error": responses[i]
				});

				fileError(file, fileResponse);
				fileComplete(file);
			}
		}

		function processQueue(url, attempt)
		{
			var MAX_ATTEMPTS = opt.maxUploadAttempts || 10,
				TIMEOUT = 2000;

			var attempt = attempt || 0;

			$.ajax({
				url: url,
				type: 'post',
				dataType: 'json',
				success: function(response)
				{
					if (response.e)
					{
						problemsUpload();

						return;
					}

					if (!response.s)
					{
						errorUpload();
						finishUpload(files);

						return;
					}

					attempt++;

					var data = response.d;

					if (data)
					{
						var allLoaded = data.loaded,
							allFailed = data.failed,

							completed = allLoaded.length + allFailed.length,
							total = data.total,

							uniqLoaded = subtractByProp(allLoaded, photos.loaded, 'origin'),
							uniqFailed = subtractByProp(allFailed, photos.failed, 'origin'),

							loadedFiles = files.getByPhotos(uniqLoaded),
							failedFiles = files.getByPhotos(uniqFailed);

						photos.loaded = allLoaded;
						photos.failed = allFailed;

						processFiles(loadedFiles, failedFiles, uniqLoaded, uniqFailed);

						if (completed < total)
						{
							if (attempt >= MAX_ATTEMPTS)
							{
								failedFiles = files.subtract(files.getByPhotos(allLoaded).concat(files.getByPhotos(allFailed)));
								processErrorFiles(failedFiles);

								finishUpload(files);
								timeoutUpload();
							}
							else
							{
								queueTimer = setTimeout(function() {
									processQueue(url, attempt);
								}.bind(this), TIMEOUT);
							}
						}
						else
						{
							finishUpload(files);
						}
					}
					else
					{
						errorUpload();
						finishUpload(files);
					}
				}
			});
		}

		init();


		/*
		 * Public Methods
		 */

		obj = {
			setUrl: setUrl,
			setAlbumId: setAlbumId,
			uploadByUrl: uploadByUrl,
			abortUpload: abortUpload
		};

		instancesUrlProvider[btn] = obj;

		return obj;
	};

	uploader.urlProvider = function (btn, opt)
	{
		var up = uploader.getUrlProvider(btn);

		if (!up) {
			up = new uploader.UrlProvider(btn, opt);
		}

		return up;
	};

	uploader.getUrlProvider = function (btn)
	{
		return instancesUrlProvider[btn];
	};
	// }}}


	/*
	 * Init module
	 */

	mambo.ui = mambo.ui || {};

	mambo.ui.Uploader_v2 = uploader;

	mambo.ready('{mambo.ui}Uploader_v2');

}, true);

// vim: fdm=marker:
