var ModalTemplate = _.template(
	"<div class='modal-overlay b-modal_layer <%= wrapperClass %>'>" +
		"<div class='modal-overlay-bg'></div>" +
		"<div class='modal origin <%= modalClass %>' style='<% if( modalWidth ) { %>width:<%- modalWidth %>px; " +
        "<% if( !mambo.langIsRTL ) { %>margin-left<% } else { %>margin-right<% } %>:-<%- modalWidth/2 %>px;<% } %>'>" +
		"<% if( buttonCloseView ) { %><div class='misc_icons modal-close'></div><% } %>" +
		"<div class='modal-bg'><i class='modal-bg-icon'></i></div>" +
		"<div class='modal-in' <% if( !content ) { %>ng-app='modal'<% } %>></div>" +
		"</div>" +
	"</div>"
);

var Modal = function (params) {
	"use strict";

	// Init params
	var paramsDefault = {
		modalClass: "modal-default",
		buttonCloseView: true,
		resize: true,
		modalWidth: 700,
		useScrolling: false,
		// Use the property "content" instead of the method "createCache"
		content: null,
		wrapperClass: ""
	};
	params = $.extend({}, paramsDefault, params);

	var obj,
		cache = null,
		win = window,
        ns = '.modal';

	var $win = $(win),
		$body = $('body'),
		$sectionPage = $(".section-page"),
		$overlay = $(ModalTemplate(params)),
		$overlayBg = $overlay.find(".modal-overlay-bg"),
		$modalBg = $overlay.find(".modal-bg >i"),
		$modal = $overlay.find('.modal'),
		$modalClose = $overlay.find('.modal-close'),
		$emitter = $({}),
        $lastFocusedElement = $();

    if (params.modalClass) {
        if (~params.modalClass.indexOf('m-photoline')) {
            $modalBg.addClass('icn icn-photoline-huge icn-huge');
        } else if (~params.modalClass.indexOf('modal-leader')) {
            $modalBg.addClass('icn icn-leader-huge icn-huge');
        } else if (~params.modalClass.indexOf('modal-vip')) {
            $modalBg.addClass('icn icn-vip-huge icn-huge');
        } else if (~params.modalClass.indexOf('modal-maketop')) {
            $modalBg.addClass('icn icn-profileUp-huge icn-huge');
        } else if (~params.modalClass.indexOf('modal-gift')) {
            $modalBg.addClass('icn icn-gift-huge icn-huge');
		} else if (~params.modalClass.indexOf('modal-ny-gifts')) {
            $modalBg.addClass('icn icn-gift-huge icn-huge');
		} else if (~params.modalClass.indexOf('modal-candy_box')) {
			$modalBg.addClass('icn icn-gift-huge icn-huge');
		} else if (~params.modalClass.indexOf('js-modal-top100')) {
			$modalBg.addClass('icn icn-rating-huge icn-huge');
        } else {
            $modalBg.remove();
        }
    }

	if (params.resize) {
		$win.on('resize', function () {
			if ($overlay.is(':visible')) {
				_resize();
			}
		});
	}

	if (params.useScrolling) {
		$overlay.css('overflow', 'auto');
	}

	// Move the content(html element) to the modal
	if (params.content) {
		_createCache(params.content);
	}
	
	$overlay.find(".js-modal-close").on('click', function () {
		_destroy();
        _userActionDestroy();
		$emitter.trigger('modalClose');
		return false;
	});

	$modalClose.on('click', function () {
		_destroy();
        _userActionDestroy();
		$emitter.trigger('modalClose');
		return false;
	});

	$overlayBg.on('click', function (e) {
		e.stopPropagation();
		_destroy();
        _userActionDestroy();
		$emitter.trigger('overlayBgClick');
	});

	function _show() {
        $body.toggleClass('modal__showcases-overflow');
		if (!cache) {
			$body.append($overlay);
		} else {
			$overlay.show();
		}

        $(document).on('keyup' + ns, function(event) {
            if (event.keyCode == 27) {
                _destroy();
				$emitter.trigger('escClick');
                return false;
            }
        });

		$emitter.trigger('show');

        // Убираем фокус с елемента. Иначе, если элемент был ссылкой, нажание по Enter вызовет тригер click.
        $lastFocusedElement = $( document.activeElement ).blur();

		return false;
	};


	function _urchange() {
		var url = mambo.utils.parseStringGetParams();
		if (url.LVipPresent) {
			$(document).trigger('LOCATION_URL_CLEAR');
		}
	}

	function _hide(total) {
        if ($overlay.is(':hidden')) return false;
        $modal.css('top', '');
        $overlay.removeClass('show');
        $body.toggleClass('modal__showcases-overflow');
        $(document).off('keyup' + ns);
        setTimeout(function() {
            if ($overlay.hasClass('show')) {
                return;
            }
            if (total) {
                $overlay.remove();
            } else {
                $overlay.hide();
                $emitter.trigger('hide');
            }
        }, 300);

		$emitter.trigger('hide');

        // Возвращяем элементу фокус, на случай, если это было поле ввода...
        $lastFocusedElement.focus();

		return false;
	};

	function _resize()
	{
		var MODAL_MARGIN = 0;
		var isCallDefaultResizeFunc = true;

		if (params.resizeFunc) {
			isCallDefaultResizeFunc = params.resizeFunc({
				"$modal": $modal,
				"$overlay": $overlay,
				"$overlayBg": $overlayBg
			});
		}

		if (isCallDefaultResizeFunc) {
			var bodyHeight = $(window).height(),
				ModalHeight = $modal.height(),
				positionTop = (bodyHeight - ModalHeight) / 2;

			if (positionTop < MODAL_MARGIN) {
				$modal.css("top", MODAL_MARGIN);
				$overlayBg.height($modal[0].scrollHeight);
			} else {
				$modal.css("top", positionTop);
				$overlayBg.css('height', 'inherit');
			}
		}
		$overlay.addClass('show');

		$emitter.trigger('resize');

		return false;
	};

	function _createCache(data) {

		if (cache) return;

		if (data) {
			$overlay.find('.modal-in').append($(data).show());
		}

		$overlay.hide();
		$body.append($overlay);
		cache = $overlay;

		return false;
	};

	function _destroy() {

		// Prevent destroying if onBeforeDestroy return false
		if (typeof params.onBeforeDestroy === 'function' && params.onBeforeDestroy() === false){
            return;
        }

		if (cache) {
			_hide();

		} else {
            // makeshift for a show-window of appendices and event too
            if ($modal.hasClass('modal-applicationCharge')) {
                $("#AppContainerBox").css('marginLeft', '');
            }

			mambo.modal.appEvent = false;

			_hide(true);//$overlay.remove();
		}

        _urchange();

		if (mambo.ui.Alert().isAlertShow()) {
			mambo.ui.Alert().hide();
		}

		$emitter.trigger('modalClose');
        mambo.modal.current = null;

		return false;
	};

    function _userActionDestroy() {
        if ($modal.hasClass('modal-applicationCharge')) {
            if ($overlay.find('.modal-applicationCharge').data('paymentAction') !== true && mambo.app) {
                if (mambo.modal && !mambo.modal.appEvent) {
                    mambo.app.event('paymentCancel');
                }
            }
        }

        return false;
    };

	function on()
	{
		$emitter.on.apply($emitter, arguments);
	}

	function off()
	{
		$emitter.off.apply($emitter, arguments);
	}

	function getBox() {
		return $overlay[0];
	}

	obj = {
		show: function () {
			_show.apply(this, arguments);
			if (params.resize) {
				setTimeout(function() {
					_resize.apply(this, arguments);	
				}, 100);
			}
			return this;
		},
		hide: function () {
			_destroy.apply(this, arguments);
			return this;
		},
		resize: function () {
			_resize.apply(this, arguments);
			return this;
		},
		// createCache is deprecated now
		createCache: function () {
			_createCache.apply(this, arguments);
			return this;
		},
		destroy: function () {
			_destroy.apply(this, arguments);
			return this;
		},
		on: function () {
			on.apply(this, arguments);
			return this;
		},
		off: function () {
			off.apply(this, arguments);
			return this;
		},
		getBox: getBox
	};

	return obj;
};
(function ()
{
	"use strict";
	var getParamsToStartLayer = {
		'gifts': ['ws_gift', '/ws_gift', 'LGift', '/LGift'],
		'leader': ['LLeader'],
		'leader_edit': ['LEdit'],
		'vip': ['LVip', '/LVip'],
		'vip_present': ['LVipPresent', '/LVipPresent'],
		'maketop': ['LTop', '/LTop'],
		'photoline': ['LPhotoline', '/LPhotoline'],
        'application_charge': ['LApplication_charge', '/LApplication_charge'],
		'ny_layer': ['ny_layer'],
		'ny_give_tree': ['ny_give_tree'],
		'ny_payment': ['LNy2015_gifts'],
		'ny_top': ['ny_top'],
		'ny_search': ['ny_search'],
		'contest': ['LContest', '/LContest', 'LNewYear', '/LNewYear'],
        'top100': ['LTop100','/LTop100']
	};


	mambo.modal = {
		current: undefined,
		unblockLayerRedirect: undefined,
		serviceGiftTo: false,
		_extraParams: {},
		_extraParamsOnce: {},
		setExtraParams: function setExtraParams(params)
		{
			this._extraParams = params || {};
		},
		getExtraParams: function getExtraParams()
		{
			return this._extraParams || {};
		},
        setExtraParam: function setExtraParam(paramName, val)
        {
            this._extraParams[paramName] = val;
        },
        setOnceExtraParam: function setOnceExtraParam(paramName, val)
        {
            this._extraParamsOnce[paramName] = val;
        },
        getExtraParam: function getExtraParam(paramName)
        {
            if (this._extraParamsOnce[paramName] !== undefined)
            {
                var t = this._extraParamsOnce[paramName];
                this._extraParamsOnce[paramName] = undefined;
                return t;
            }

            return this._extraParams[paramName];
        },
		show: function show()
		{
			mambo.modal.current && mambo.modal.current.show();
		},
		resize: function resize()
		{
			mambo.modal.current && mambo.modal.current.resize();
		},
		destroy: function destroy()
		{
			mambo.modal.current && mambo.modal.current.destroy();
		},
		triggerMixpanel: function(responseObjects) {
			var mixpanel = window.mixpanel;
			if (!mixpanel) {
				return;
			}
			responseObjects = responseObjects || [];
			for (var i = 0, responseObject, l = responseObjects.length; i < l; i++) {
				responseObject = responseObjects[i];
				responseObject = responseObject || {};
				responseObject = responseObject.mixpanel || {};

				if ('events' in responseObject) {
					mixpanel.identify(mambo.aAnketaId);
					for (var j = 0, event, ll = responseObject.events.length; j < ll; j++) {
						event = responseObject.events[j];
						if ('name' in event && 'type' in event) {
							mixpanel.track(event.name, {'type': event.type});
						} else if ('name' in event) {
							mixpanel.track(event.name);
						}
					}
				}
			}
		},
		/**
		 * Returns template path by it's name on dev machine or in production
		 */
		getTemplatePath: function getTemplatePath(tplName)
		{
			return  'jsCore/mamba-app/view/' + (mambo.debugMode ? 'debug-ru' : mambo.aUserLang) + '/' + tplName;
		},
		/**
		 * Returns template path by it's name on dev machine or in production
		 */
		getTemplatePathOrName: function getTemplatePathOrName(tplName)
		{
			return mambo.debugMode ? (mambo.aliases.jsCore + 'mamba-app/view/debug-ru/' + tplName) : tplName;
		},
		/**
		 * Returns all get params for starting particular payment layer
		 * or params for all layers if argument is omitted
		 */
		paymentLayerGetParams: function paymentLayerGetParams(layerType)
		{
			if (! layerType)
			{
				var t = [];
				for (var i in getParamsToStartLayer) {
					t = t.concat(getParamsToStartLayer[i]);
				}
				return t;
			}
			else if (layerType in getParamsToStartLayer) {
				return getParamsToStartLayer[layerType];
			}

			return [];
		},
		/**
		 * Checks if we need to open current layer by get params (or hash)
		 * @param {String} layerType
		 * @param {Object} urlParams
		 * @returns {boolean}
		 */
		mustOpenLayerByUrl: function mustOpenLayerByUrl(layerType, urlParams)
		{
			var openByUrl = false,
				validParams = this.paymentLayerGetParams(layerType);

			for (var i = 0, l = validParams.length, p; i < l, p = validParams[i]; i ++)
			{
				if (urlParams[p] == 1) {
					openByUrl = true;
				}
			}

			return openByUrl;
		}
	};
}());

mambo.ready('jsCore/mamba-app/Modal');
