define('photolayer/config.js',['require'],function (require) {

	var photolayer = {};

	photolayer.metrics = mambo.project.photolayerStatistics;

	window.onbeforeunload = function () {
		// Send statistics data to the server;
		if (photolayer.metrics.photoHits) {
			$.ajax('/stat/photo_viewed.phtml?data=' + [
					mambo.vUserId,
					'web',
					photolayer.metrics.photoHits
				].join('|'), {
				method: 'POST'
			});
		}
		photolayer.metrics.sendData().clearData();
	};

	if (window.opera) {
		var photolayerStatisticsTimer = function () {
			setTimeout(function () {
				photolayer.metrics.sendData().clearData();
				photolayerStatisticsTimer();
			}, 1000 * 60 * 5);
		};
		photolayerStatisticsTimer();
	}


	photolayer.utils = photolayer.utils || {};

	photolayer.utils.inject = function () {
		var self = this;
		self = self || {};
		var dependencies = arguments[arguments.length - 1] || {};
		for (var i in dependencies) {
			if (Object.prototype.hasOwnProperty.call(dependencies, i)) {
				self[i] = dependencies[i];
			}
		}
		return self;
	};

	photolayer.utils.isTouchDevice = function () {
		return navigator.msMaxTouchPoints || 'ontouchstart' in window;
	};

	// Use utils/Common.js trim function;
	if (typeof String.prototype.trim !== 'function') {
		String.prototype.trim = function () {
			return this.replace(/^\s+|\s+$/g, '');
		};
	}

	return photolayer;
});


define('photolayer/AdvBannerSection.js',['require'],function (require) {
	var PHOTOLAYER_WITH_ADV_CSS = 'photolayer_with-adv';

	var module = function (context) {
		
		var AdvBannerSection = function () {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof AdvBannerSection)) {
				return new AdvBannerSection(dependencies);
			}
			var isOwnerProfile = dependencies.isOwnerProfile || false;
			var cssClass = dependencies.cssClass || '';
			var cssClassWrapper = dependencies.cssClassWrapper || '';

			if (mambo.isPhotoLayerBanner === 1 && !isOwnerProfile && mambo.partnerId !== 800000003) {
				self.$ = $('<div></div>').attr('class', cssClassWrapper);
				self.$advBannerSection = $('<iframe></iframe>').attr({
					'scrolling': 'no',
					'seamless': 'seamless',
					'class': cssClass
				});
				self.$.append(self.$advBannerSection);
				self.exists = true;
			} else {
				self.$ = $(document.createDocumentFragment());
				self.$advBannerSection = $({});
				self.exists = false;
			}

			self.windowMinHeightForAdv = dependencies.windowMinHeightForAdv || 768;
			self.photoLayer = dependencies.photoLayer;
			var advBannerUrlInited = false;
			var $clone = null;
			self.isAdvBannerUrlInited = function () {
				return !advBannerUrlInited ? (advBannerUrlInited = true, false) : advBannerUrlInited;
			};
			self.getAdvBannerSectionClone = function () {
				return $clone || ($clone = self.$.clone());
			};
		};
		AdvBannerSection.fn = AdvBannerSection.prototype;
		AdvBannerSection.fn.initAdvBannerUrl = function (aAnketaId, partnerId, bannerWidth, windowHeight) {
			var self = this;
			if (!self.isAdvBannerUrlInited()) {
				self.changeAdvBannerUrl(aAnketaId, partnerId, bannerWidth);
			}
            self.$.css('margin-top', windowHeight < 800 ? '-110px' : '');
			return self;
		};
		AdvBannerSection.fn.toggle = function (value) {
			var self = this;
			var show = value ? true : false;
			self.$.toggle(show);
			self.photoLayer.toggleClass(PHOTOLAYER_WITH_ADV_CSS, show);
			return self;
		};
		AdvBannerSection.fn.shouldHideAdvSection = function () {
			return $(window).height() < this.windowMinHeightForAdv;
		};
		AdvBannerSection.fn.showIfHidden = function () {
			if (!this.shouldHideAdvSection() && !this.photoLayer.hasClass(PHOTOLAYER_WITH_ADV_CSS)) {
				// console.log('show bottom banner, was hidden');
				this.toggle(true);
			}
		};
		AdvBannerSection.fn.changeAdvBannerUrl = function (aAnketaId, partnerId, bannerWidth) {
			var self = this;
			bannerWidth = bannerWidth || 728;
			var src = '//loveadvert.ru/be.phtml?pid=50&aid=' + aAnketaId + '&partner=' + partnerId + '&nohead=1&width=' + bannerWidth + '&ilang=' + mambo.aUserLangId;
			self.$advBannerSection.attr('src', src);
			return self;
		};
		context.AdvBannerSection = AdvBannerSection;
	};

	return module;
});
define('photolayer/LikeCtrl.js',['require'],function (require) {
    var CSS_ANIMATION_LIKE = 'icn-top100-huge-liked icn-top100-huge-animation',
        CSS_ANIMATION_FADE = 'icn-top100-huge-fade',
        CSS_ANIMATION_PREPARE = 'icn-top100-huge_animation',
        
        CSS_DISABLED = 'like-button-container_disabled',
        CSS_VOTED = 'like-button-container_voted';

    return function (context) {

        function LikeCtrl() {
            var dependencies = arguments[arguments.length - 1] || {};
            
            this.photoratingService = dependencies.photoratingService;
            
            this.$ = $('#photolayer-LikeButtonTmpl').tmpl();
            
            this.$likeButton = this.$.find('.js-like-button');
            this.$votedTooltip = this.$.find('.js-tooltip-photo-voted');
            this.$notInRatingTooltip = this.$.find('.js-tooltip-photo-not-in-rating');
            
            this.$likeAnimationIcon = dependencies.$likeAnimationIcon;
            
            this.hideTooltipTimeout = null;
            this.likeData = {};
            
            this.preventPhotoFrameDestroy = false;
        }
        
        LikeCtrl.fn = LikeCtrl.prototype;
        
		LikeCtrl.fn.init = function (dependencies) {
            this.photoFrameCtrl = dependencies.photoFrameCtrl;
            
            var self = this;
            
            this.$likeButton.on('click', function () {
                if (self.hasNotActiveState()) {
                    return;
                }
                
                self.$likeAnimationIcon.on('transitionend', function () {
                    self.$likeAnimationIcon.off('transitionend');
                    self.$likeAnimationIcon.addClass(CSS_ANIMATION_FADE);
                    
                    self.likeAction();
                });
                self.$likeAnimationIcon.css('visibility', 'visible');
                self.$likeAnimationIcon.addClass(CSS_ANIMATION_PREPARE);
                
                setTimeout(function() {
                    self.$likeAnimationIcon.addClass(CSS_ANIMATION_LIKE);
                }, 50);
            });

            this.$
                .on('mouseenter', function () {
                    if (self.hasNotActiveState()) {
                        self.renderTips();
                    }
                })
                .on('mouseleave', function () {
                    self.hideTooltipTimeout = setTimeout(function () {
                        self.hideTips();
                    }, 1000);
                });
        };

        LikeCtrl.fn.hasNotActiveState = function () {
            return this.$.hasClass(CSS_DISABLED) || this.$.hasClass(CSS_VOTED);
        };
        
        LikeCtrl.fn.likeAction = function () {
            var self = this;
            
            if (!this.likeData.voted) {
                this.$.addClass(CSS_VOTED);
                this.photoratingService.voteForRatingPhoto({ratingId: this.likeData.ratingId, photoId: this.likeData.photoId}, {vote: 1, sourceId: 2})
                    .then(function (res) {
                        self.likeData.voted = true;
                        self.likeData.recentlyVoted = true;
                        
                        if (res.mutualLike && res.mutualLike.profiles.length) {
                            self.showMutualVoting(res.mutualLike);
                        }
                    }, function () {
                        if (window.stats) {
                            stats.jsErrorPush(new Error('LikeCtrl.js: vote for rating photo error'));
                        }
                    });
                self.likeData.voted = true;
            }
        };
        
        LikeCtrl.fn.showMutualVoting = function (mutualLike) {
            var self = this,
                friend = mutualLike.profiles[1];
            
            self.preventPhotoFrameDestroy = true;
            var mutualVoting = Mb.ComponentManager.getComponentInstance({
		        url: 'Web/Modal',
                localId: 'MutualVoting',
                
		        params: {
                    content: {
                        componentUrl: 'Web/Photorating/MutualVoting',
                        componentData: {
                            selfPhotoUrl: mutualLike.photoUrl,
                            friendPhotoUrl: mutualLike.likePhotoUrl,
                            friendId: mutualLike.likeAnketaId,
                            friendLogin: friend.login,
                            friendGender: friend.gender
                        }
                    },
                    innerCssClass: 'mutual-voting-paddings'
		        }
		    }, {
		        parent: null,
		        parentElement: $('body')
		    });
            
            
            mutualVoting.then(function (mutualVotingModal) {
                self.photoFrameCtrl.hideLayer();
                
                mutualVotingModal.disposed.once(function () {
                    self.preventPhotoFrameDestroy = false;
                    self.photoFrameCtrl.showLayer();
                    
                    $('body').css({
                        'overflow': 'hidden',
                        'min-width': '1000px'
                    }).scrollTop(0);                    
                });
            });
        };
        
        LikeCtrl.fn.renderTips = function () {
            var $currentTip;
            
            this.hideTips();
            
            if (this.$.hasClass(CSS_VOTED) && !this.likeData.recentlyVoted) {
                $currentTip = this.$votedTooltip;
            } else if (this.$.hasClass(CSS_DISABLED)) {
                $currentTip = this.$notInRatingTooltip;
            }
            
            if ($currentTip) {
                $currentTip.css({ left: -($currentTip.outerWidth() / 2) + (this.$.outerWidth() / 2) }).show();
            }
        };
        
        LikeCtrl.fn.hideTips = function () {
            clearTimeout(this.hideTooltipTimeout);

            this.$votedTooltip.hide();
            this.$notInRatingTooltip.hide();
        };
        
        LikeCtrl.fn.renderLikeButton = function () {
            if (mambo.aUserId && this.likeData.ownerId != mambo.aUserId && this.likeData.photoId && this.likeData.default) {
                if (this.likeData.ratingId) {
                    if (this.likeData.voted) {
                        this.$.addClass(CSS_VOTED);
                    }
                } else {
                    this.$.addClass(CSS_DISABLED);
                }
                this.$likeButton.show();
                return this;
            }
            this.$likeButton.hide();
            return this;
        };
        
        LikeCtrl.fn.clearLikeLayer = function () {
            this.hideTips();

            this.$.removeClass(CSS_DISABLED)
                  .removeClass(CSS_VOTED);
            this.$likeAnimationIcon
                .removeClass(CSS_ANIMATION_LIKE)
                .removeClass(CSS_ANIMATION_FADE);
            
            this.preventPhotoFrameDestroy = false;
            this.likeData = {};
            this.$likeAnimationIcon.css('visibility', 'hidden');
            this.$likeAnimationIcon.removeClass(CSS_ANIMATION_PREPARE);
        };
        
        LikeCtrl.fn.updateLikeButtonStatus = function (params) {
            this.clearLikeLayer();
            this.likeData = {
                voted: params.voted.isAlreadyVoted,
                ratingId: params.selectedPhoto.ratingId,
                ownerId: params.selectedPhoto.ownerId,
                photoId: params.selectedPhoto.id,
                default: params.selectedPhoto.album.default
            };
            this.renderLikeButton();
        };

        LikeCtrl.fn.dispose = function () {
            this.$.empty();
        };
        
        context.LikeCtrl = LikeCtrl;
    };
});

// CommentButton

define('photolayer/Button.js',['require'],function (require) {
	var module = function (context) {
        // CloseButton;
		function CloseButton(cssClass) {
			return $('<div></div>').attr('class', cssClass || '');
		}

		context.CloseButton = CloseButton;

	};
	return module
});
define('photolayer/PhotolayerNewFaces.js',['require'],function (require) {
	var module = function (context) {
		var PhotolayerNewFaces = function () {
			var self = this;
			if (!(this instanceof context.PhotolayerNewFaces)) {
				return new context.PhotolayerNewFaces();
			}			

			this.$ = $('#photolayer-NewFacesTmpl').tmpl({
				oid: mambo.vUserId,
				uid: mambo.uid
			});
			
			this.$.find(".close-button").click(function (e) {
				self.$.trigger('close');
			});
			
			this.$.find("#restart").click(function (e) {
				self.$.trigger('restart');
			});
		}; 
		PhotolayerNewFaces.fn = PhotolayerNewFaces.prototype;
		PhotolayerNewFaces.fn.show = function() {
			this.$.show();
			
			if (!this.isLoaded) this.load();
		}
		
		PhotolayerNewFaces.fn.hide = function() {			
			this.$.hide();
		}
		
		PhotolayerNewFaces.fn.load = function() {
			var self = this;

			Mb.ComponentManager.appendComponent({
				url: 'Web/NewFaces',
				params: {
					mode: 'three-lane',
					rootCssClass: 'shadow-3 dark-border'
				}
			}, {
				parentElement: self.$.find('.new-faces-container')[0],
				cleanParent: true
			}).then(function (instance) {
				self.isLoaded = true;
			});
		}
		
		PhotolayerNewFaces.fn.liftUp = function() {
			this.$.addClass('lift-up');
		}
		
		PhotolayerNewFaces.fn.center = function() {
			this.$.removeClass('lift-up');
		}
		
		context.PhotolayerNewFaces = PhotolayerNewFaces;
	}
	return module;
});
define('photolayer/AlbumsPreviewSection.js',['require'],function (require) {
	var DISABLED_PREVIEW_CSS = 'albums-preview-section_disabled';

	var module = function (context) {
		// AlbumsPreviewSectionWrapper;

		function AlbumsPreviewSectionWrapper(cssClass) {
			return $('<div></div>').attr('class', cssClass || '');
		}

		context.AlbumsPreviewSectionWrapper = AlbumsPreviewSectionWrapper;


		// AlbumsPreviewSection;

		/*********************** AlbumImageSection **********************/
		var AlbumImageSection = function (photo, cssClass, albumImageCssClass, photoNotFoundCssClass, usePixelization) {
			var self = this;
			if (!(self instanceof AlbumImageSection)) {
				return new AlbumImageSection(photo, cssClass, albumImageCssClass, photoNotFoundCssClass, usePixelization);
			}
			var PREVIEW_WIDTH = 36;
			var PREVIEW_HEIGHT = 50;

			if (!photo.removed) {
				// photo is found;
				self.$ = $('<div></div>')
					.attr('class', cssClass || '')
					.append(
					$('<img/>')
						.attr('class', albumImageCssClass || '')
						.hide()
				);
				if (usePixelization && photo.erotic) {
					/// TODO: refactor! Use setTimeout to keep pixelized and default-clicked image selected;
					setTimeout(function () {
						mambo.project.Pixelization(self.$.find('.' + (albumImageCssClass || '')), PREVIEW_WIDTH, PREVIEW_HEIGHT, true);
					}, 10);
				} else {
					mambo.project.Pixelization(self.$.find('.' + (albumImageCssClass || '')), PREVIEW_WIDTH, PREVIEW_HEIGHT, false);
				}
			} else {
				self.$ = $('<div></div>')
					.attr('class', cssClass + ' ' + (photoNotFoundCssClass || ''))
					.append(
					$('<span></span>')
						.attr('class', albumImageCssClass || '')
				)
			}
			self.albumId = photo.albumId;
		};
		/*********************** **************** **********************/

		/*********************** AlbumCaption **********************/
		var AlbumCaption = function (text, albumCaptionCssClass, albumCaptionTextCssClass) {
			var self = this;
			if (!(self instanceof AlbumCaption)) {
				return new AlbumCaption(text, albumCaptionCssClass, albumCaptionTextCssClass);
			}
			self.$ = $('<div></div>')
				.append(
				$('<span></span>')
					.attr('class', albumCaptionTextCssClass || '')
					.text(text ? text : '')
			)
				.attr('class', albumCaptionCssClass || '');
		};
		/*********************** ************ **********************/

		/*********************** AlbumSection **********************/
		var AlbumSection = function (album) {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof AlbumSection)) {
				return new AlbumSection(album, dependencies);
			}

			self.albumSectionCssClass = dependencies.albumSectionCssClass || '';
			self.albumImageSectionCssClass = dependencies.albumImageSectionCssClass || '';
			var albumSectionLastCssClass = dependencies.albumSectionLastCssClass || '';
			var lastAlbum = dependencies.lastAlbum;
			self.albumCaption = AlbumCaption(album.name, 'album-caption', 'album-caption-text');

			self.$ = $('<div></div>')
				.attr('class', lastAlbum ? self.albumSectionCssClass + ' ' + albumSectionLastCssClass : self.albumSectionCssClass)
				.append(self.albumCaption.$);
		};
		AlbumSection.fn = AlbumSection.prototype;
		AlbumSection.fn.getAlbumImageSectionsCount = function () {
			var self = this;
			return self.$.find('.' + self.albumImageSectionCssClass).length;
		};
		/*********************** ************ **********************/

		/*********************** AlbumsPreviewSection **********************/
		var AlbumsPreviewSection = function () {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof AlbumsPreviewSection)) {
				return new AlbumsPreviewSection(dependencies);
			}
			self.photos = [];
			self.albums = [];
			self.usePixelization = dependencies.usePixelization || false;
			self.albumsPreviewSectionCssClass = dependencies.albumsPreviewSectionCssClass || '';
			self.albumImageSectionCssClass = dependencies.albumImageSectionCssClass || '';

			self.$ = $('<div></div>').attr('class', self.albumsPreviewSectionCssClass);
		};
		AlbumsPreviewSection.fn = AlbumsPreviewSection.prototype;
		AlbumsPreviewSection.fn.toggle = function (value) {
			var self = this;
			if (value) {
				self.$.css('visibility', 'visible');
			} else {
				self.$.css('visibility', 'hidden');
			}
			return self;
		};
		AlbumsPreviewSection.fn.loadVisibleAlbumImages = function () {
			var self = this;
			// Load only visible images of albumsPreviewSection;
			$.each(self.photos, function (index, photo) {
				if (photo.view.$.offset().left < $(window).width()) {
					var img = new Image();
					img.onload = function () {
						var width = this.width;
						var height = this.height;
						var $img = photo.view.$.find('.album-image');
						$img.attr('src', photo.smallSrc).show();

						if (photo.erotic && self.usePixelization) {
							mambo.project.Pixelization($img, width, height, true);
						} else {
							mambo.project.Pixelization($img, width, height, false);
						}
					};
					img.src = photo.smallSrc;
				}
			});
			return self;
		};
		AlbumsPreviewSection.fn.getPhotoInAlbum = function (photoId, album) {
			var self = this;
			var selectedPhoto = {id: 0, name: '', album: {id: 0}};
			album = album || {};
			for (var i = 0, l = self.photos.length; i < l; i++) {
				if (photoId === self.photos[i].id && album.id === self.photos[i].album.id) {
					selectedPhoto = self.photos[i];
					selectedPhoto.index = i;
					break;
				}
			}
			return selectedPhoto;
		};
		AlbumsPreviewSection.fn.putPhotoInAlbum = function (photo, album) {
			var self = this;
			photo = photo || {album: {id: 0}};
			album = album || {
				id: 0,
				view: {
					$: $(document.createDocumentFragment())
				}
			};
			var selectedPhoto = self.getPhotoInAlbum(photo.id, {id: photo.album.id});
			var lastPhotoInAlbum = self.getLastPhotoInAlbum(album);

			if (selectedPhoto.id) {
				selectedPhoto.album = album;
				photo.album = album;
				selectedPhoto.view.$.detach();
				self.photos.splice(selectedPhoto.index, 1);
				if (lastPhotoInAlbum.index === -1) {
					// TODO: refactor! Hell's hardcode!
					var emptyAlbum = self.getAlbumById(album.id);
					if (emptyAlbum.index === 0) {
						// the only album in the gallery;
						self.photos.push(selectedPhoto);
					} else {
						lastPhotoInAlbum = self.getLastPhotoInAlbum(self.albums[emptyAlbum.index - 1]);
						if (selectedPhoto.index < lastPhotoInAlbum.index) {
							self.photos.splice(lastPhotoInAlbum.index, 0, selectedPhoto);
						} else {
							self.photos.splice(lastPhotoInAlbum.index + 1, 0, selectedPhoto);
						}
					}
				} else if (selectedPhoto.index < lastPhotoInAlbum.index) {
					self.photos.splice(lastPhotoInAlbum.index, 0, selectedPhoto);
				} else {
					self.photos.splice(lastPhotoInAlbum.index + 1, 0, selectedPhoto);
				}
				album.view.$.append(selectedPhoto.view.$);
			}
			return self;
		};
		AlbumsPreviewSection.fn.getFirstPhoto = function () {
			var self = this;
			var selectedPhoto = {id: 0, name: ''};
			if (self.photos[0]) {
				selectedPhoto = self.photos[0];
			}
			return selectedPhoto;
		};
		AlbumsPreviewSection.fn.getLastPhoto = function () {
			var self = this;
			var selectedPhoto = {id: 0, name: ''};
			if (self.photos[self.photos.length - 1]) {
				selectedPhoto = self.photos[self.photos.length - 1];
			}
			return selectedPhoto;
		};
		AlbumsPreviewSection.fn.getLastPhotoInAlbum = function (album) {
			var self = this;
			album = album || {id: 0};
			var result = {index: -1, id: 0};
			for (var l = self.photos.length, i = l - 1; i >= 0; i--) {
				if (self.photos[i].album.id === album.id) {
					result = self.photos[i];
					result.index = i;
					break;
				}
			}
			return result;
		};
		AlbumsPreviewSection.fn.getAdjacentPhotos = function (selectedPhoto) {
			var self = this;
			var result = {previous: {}, next: {}};

			var photoAlbumId = selectedPhoto.album ? selectedPhoto.album.id : selectedPhoto.albumId;

			for (var i = 0, l = self.photos.length; i < l; i++) {
				if (selectedPhoto.id === self.photos[i].id && photoAlbumId * 1 === self.photos[i].album.id) {
					result.previous = self.photos[i - 1] || {};
					result.next = self.photos[i + 1] || {};
					break;
				}
			}
			return result;
		};
		AlbumsPreviewSection.fn.appendAlbum = function (album, defaultAlbum, albumOwner, lastAlbum) {
			var self = this;
			album = album && album.photos ? album : {photos: []};
			defaultAlbum = defaultAlbum || {};
			var albumSection = AlbumSection(album, {
				albumSectionCssClass: 'album-section',
				albumSectionLastCssClass: 'album-section-last',
				albumImageSectionCssClass: 'album-image-section',
				lastAlbum: lastAlbum
			});
			if (!album.photos.length) {
				albumSection.$.hide();
			}

			var photos = album.photos || [];
			album = {
				id: album.id * 1,
				commentState: album.commentState,
				name: album.name,
				view: albumSection,
				'default': album['default'] /// Special for IE8 we use 'default' in quotes;
			};
			var windowScreenWidth = window.screen.width;
			for (var i = 0, photo, l = photos.length; i < l; i++) {
				photo = photos[i];
				photo = {
					album: album,
					ownerId: albumOwner.id,
					ownerGender: albumOwner.gender,
					status: photo.status === 'Moderating' ? 'MODERATING' : 'APPROVED',
					defaultAlbum: defaultAlbum,
					commentsCount: photo.commentsCount,
					isRejected: photo.isReject === true ? true : false,
					id: photo.id,
					width: photo.width * 1,
					height: photo.height * 1,
					src: windowScreenWidth <= 960 ? photo.hugePhotoUrl : (photo.largePhotoUrl || photo.hugePhotoUrl),
					srcMasked: windowScreenWidth <= 960 ? photo.hugePhotoUrlMasked : (photo.largePhotoUrlMasked || photo.hugePhotoUrlMasked),
					smallSrc: photo.squareSmallPhotoUrl,
					description: photo.name,
					selected: false,
					current: photo['default'] ? true : false,
					ratingId: photo.ratingId * 1,
					erotic: photo.erotic,
					removed: photo.removed,
					commentsListOpened: false,
					verified: photo.verified,
					view: AlbumImageSection(
						photo,
						'album-image-section photo-shadow',
						'album-image',
						'album-image-not-found',
						self.usePixelization
					)
				};
				self.photos.push(photo);
				albumSection.$.append(photo.view.$);
			}
			self.$.append(albumSection.$);
			self.albums.push(album);
			return self;
		};
		AlbumsPreviewSection.fn.getAlbumById = function (albumId) {
			var self = this;
			var album = {
				id: 0, view: {
					$: $(document.createDocumentFragment())
				}
			};
			albumId = albumId * 1;
			for (var i = 0, l = self.albums.length; i < l; i++) {
				if (self.albums[i].id === albumId) {
					album = self.albums[i];
					album.index = i;
					break;
				}
			}
			return album;
		};
		AlbumsPreviewSection.fn.deletePhoto = function (photo) {
			var self = this;
			photo = photo || {};
			var deletedPhoto = {id: 0};
			for (var i = 0, l = self.photos.length; i < l; i++) {
				if (self.photos[i].id === photo.id) {
					deletedPhoto = self.photos[i];
					self.photos.splice(i, 1);
					deletedPhoto.view.$.remove();
					break;
				}
			}
			return deletedPhoto;
		};
		AlbumsPreviewSection.fn.disablePreviewSection = function (disabled) {
			this.$.toggleClass(DISABLED_PREVIEW_CSS, disabled);
		};
		AlbumsPreviewSection.fn.isDisabled = function () {
			return this.$.hasClass(DISABLED_PREVIEW_CSS); 
		};
		/*********************** ************************ **********************/
		context.AlbumsPreviewSection = AlbumsPreviewSection;

	};

	return module;
});
define('photolayer/ApplicationCtrl.js',['require'],function (require) {

	// ApplicationCtrl
	var module = function (context) {
		function ApplicationCtrl(clickedPhotoId, clickedPhotoAlbumId) {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};

			self.clickedPhotoAlbumId = clickedPhotoAlbumId;
			self.clickedPhotoId = clickedPhotoId;
			self.PhotoFrameWrapper = context.PhotoFrameWrapper;
			self.PhotoFrame = context.PhotoFrame;
			self.Photo = context.Photo;
			self.PhotoNotFound = context.PhotoNotFound;
			self.EroticWarning = context.EroticWarning;
			self.AlbumsPreviewSection = context.AlbumsPreviewSection;
			self.SmallAlbumsPreviewSection = context.SmallAlbumsPreviewSection;
			self.PreviousPhoto = context.PreviousPhoto;
			self.NextPhoto = context.NextPhoto;
			self.AlbumsPreviewSectionWrapper = context.AlbumsPreviewSectionWrapper;
			self.PhotoSlider = context.PhotoSlider;
			self.CloseButton = context.CloseButton;
			self.Photolayer = context.Photolayer;
			self.mambaAPI = context.mambaAPI;
			self.PreviewSectionCtrl = context.PreviewSectionCtrl;
			self.PhotoFrameCtrl = context.PhotoFrameCtrl;
			self.SmallAlbumSection = context.SmallAlbumSection;
			self.PhotoCaptionSection = context.PhotoCaptionSection;
			self.AdvBannerSection = context.AdvBannerSection;
			self.PhotoCaption = context.PhotoCaption;
			self.PhotoDescription = context.PhotoDescription;
            self.PhotolayerNewFaces = context.PhotolayerNewFaces;
            self.LikeCtrl = context.LikeCtrl;
			self.CommentCreateSection = context.CommentCreateSection;
			self.ConfirmationDialog = context.ConfirmationDialog;
			self.PhotoSettingsSection = context.PhotoSettingsSection;
			self.ComplaintDialog = context.ComplaintDialog;
			self.ComplaintDescription = context.ComplaintDescription;
			self.EditPhotoDialog = context.EditPhotoDialog;
			self.Tooltip = context.Tooltip;
			self.VKLikeButton = context.VKLikeButton;
			self.FBLikeButton = context.FBLikeButton;
			self.photolayerStatistics = context.metrics;

			self.WINDOW_MIN_WIDTH = dependencies.WINDOW_MIN_WIDTH || 960;
			self.WINDOW_MIN_HEIGHT = dependencies.WINDOW_MIN_HEIGHT || 700;
			self.SHOW_EROTIC_WARNING = dependencies.SHOW_EROTIC_WARNING;
			self.VK_LIKE_BUTTON = dependencies.VK_LIKE_BUTTON;
			self.FB_LIKE_BUTTON = dependencies.FB_LIKE_BUTTON;
			self.incognitoMode = dependencies.incognitoMode || false;
			self.likeButtonAvailable = dependencies.likeButtonAvailable || false;
		}

		ApplicationCtrl.fn = ApplicationCtrl.prototype;
		ApplicationCtrl.fn.createOverlay = function () {
			var self = this;
			self.photoOverlay = context.PhotoOverlay('photolayer-photo-overlay photolayer-photo-overlay-loading');
			return self.photoOverlay;
		};
		ApplicationCtrl.fn.init = function () {
			var self = this;
			var mambaAPI = self.mambaAPI;

			var dependencies = arguments[arguments.length - 1] || {};

			self.photolayerLoader = dependencies.photolayerLoader;
			self.IS_OWNER_PROFILE = dependencies.isOwnerProfile;
			self.PHOTO_FRAME_MIN_WIDTH = dependencies.photoFrameMinWidth || 600;
			self.PHOTO_FRAME_MIN_HEIGHT = dependencies.photoFrameMinHeight || 450;

			self.openedAlbumPreviousPhotoId = dependencies.previousPhotoId;
			self.openedAlbumNextPhotoId = dependencies.nextPhotoId;
			var userProfile = dependencies.userProfile || {};
			var defaultAlbum = dependencies.defaultAlbum || {};
			var albums = dependencies.albums || [];
			var editableAlbums = dependencies.editableAlbums || [];
			var albumOwner = dependencies.albumOwner;
			var incognitoService = dependencies.incognitoService;
			var photoratingService = dependencies.photoratingService;
			self.incognitoStatus = dependencies.incognitoStatus;
			self.incognitoRelationStatus = dependencies.incognitoRelationStatus;
			self.incognitoRequestStatus = dependencies.incognitoRequestStatus;

			var defaultRatings = dependencies.ratings;
			var defaultComments = dependencies.commentsData;
			var defaultComplaints = dependencies.complaintsObject;
			var defaultVote = dependencies.votedObject;

			var usePixelization = !self.IS_OWNER_PROFILE && (!self.incognitoStatus.isOn || self.incognitoStatus.isOn && self.incognitoRelationStatus.canView);

			var albumsPreviewSection = self.AlbumsPreviewSection({
				albumsPreviewSectionCssClass: 'albums-preview-section',
				usePixelization: usePixelization,
				albumImageSectionCssClass: 'album-image-section'
			});
			var smallAlbumsPreviewSection = self.SmallAlbumsPreviewSection('small-albums-preview-section', 'small-album-section', 'small-album-section-selected');
			var albumsPreviewSectionWrapper = self.AlbumsPreviewSectionWrapper('albums-preview-section-wrapper');
			var photoSlider = self.PhotoSlider('#photolayer-PhotoSliderTmpl', 'photo-slider-ie8');
			var closeButton = self.CloseButton('close-button');
			var photoLayer = self.Photolayer('photolayer');
			var advBannerSection = new self.AdvBannerSection({
				cssClass: 'adv-banner-section photo-shadow',
				cssClassWrapper: 'adv-banner-section-wrapper',
				isOwnerProfile: self.IS_OWNER_PROFILE,
				windowMinHeightForAdv: 768,
				photoLayer: photoLayer
			});

			// TODO: refactor!
			var currentAlbumId = self.clickedPhotoAlbumId ? self.clickedPhotoAlbumId : 0;

			var findPhotoItem = function (photoId, albums) {
				var result = {photo: 0, album: 0, photoIndex: 0, albumIndex: 0};
				albums = albums || [];
				for (var i = 0, l = albums.length; i < l; i++) {
					if (!albums[i] || !albums[i].photos) {
						continue;
					}
					for (var j = 0, ll = albums[i].photos.length; j < ll; j++) {
						if (albums[i].photos[j].id === photoId) {
							result.photo = albums[i].photos[j];
							result.album = albums[i];
							result.photoIndex = j;
							result.albumIndex = i;
							return result;
						}
					}
				}
				return result;
			};

			var findAlbum = function (albumId, albums) {
				for (var i = 0, l = albums.length; i < l; i++) {
					if (albums[i].id === albumId) {
						return albums[i];
					}
				}
				return null;
			};

			var userHasPhotos = function (albums) {
				albums = albums || [];
				for (var i = 0, l = albums.length; i < l; i++) {
					albums[i].photos = albums[i].photos || [];
					for (var j = 0, ll = albums[i].photos.length; j < ll; j++) {
						return true;
					}
				}
				return false;
			};

			/*
			 Here we can have three different cases:
			 1) User photo is completely deleted;
			 2) User photo is moved to another album;
			 3) User has got no photos at all;
			 */

			var currentAlbum = findAlbum(currentAlbumId, albums);
			if (!userHasPhotos(albums)) {
				// has got no photos at all;
				if (currentAlbum) {
					currentAlbum.photos = [{id: self.clickedPhotoId, removed: true, album: currentAlbum}];
				} else {
					currentAlbumId = 0;
					albumsPreviewSection.appendAlbum({
						id: 0,
						photos: [
							{id: self.clickedPhotoId, removed: true, album: {id: 0}}
						],
						name: context.t.albumsDelete
					}, defaultAlbum, albumOwner);
					smallAlbumsPreviewSection.appendPreviewSection(
						self.SmallAlbumSection({name: context.t.albumsDelete}, 'small-album-section')
					);
				}
			} else {
				/*
				 User has got some photos;
				 Clicked photo can be moved into another album,
				 can be deleted completely or just exist;
				 */
				var foundPhotoItem = findPhotoItem(self.clickedPhotoId, albums);
				if (!foundPhotoItem.photo) {
					// A photo is not found;
					// It's probably deleted;
					// We need to find current album and paste a dummy photo frame there;
					if (currentAlbum) {
						var previousPhotoPaste = findPhotoItem(self.openedAlbumPreviousPhotoId, [currentAlbum]);
						var nextPhotoPaste = findPhotoItem(self.openedAlbumNextPhotoId, [currentAlbum]);

						if (previousPhotoPaste.photo) {
							previousPhotoPaste.album.photos.splice(previousPhotoPaste.photoIndex + 1, 0, {
								id: self.clickedPhotoId,
								removed: true,
								album: previousPhotoPaste.album
							});
						} else if (nextPhotoPaste.photo) {
							nextPhotoPaste.album.photos.splice(nextPhotoPaste.photoIndex - 1, 0, {
								id: self.clickedPhotoId,
								removed: true,
								album: nextPhotoPaste.album
							});
						} else {
							// album is empty;
							currentAlbum.photos = [{id: self.clickedPhotoId, removed: true, album: currentAlbum}];
						}
					} else {
						currentAlbumId = 0;
						albumsPreviewSection.appendAlbum({
							id: 0,
							photos: [
								{id: self.clickedPhotoId, removed: true, album: {id: 0}}
							],
							name: context.t.albumsDelete
						}, defaultAlbum, albumOwner);
						smallAlbumsPreviewSection.appendPreviewSection(
							self.SmallAlbumSection({name: context.t.albumsDelete}, 'small-album-section')
						);
					}
				} else {
					// Photo exists and is found;
					// Here we need to try searching for photo in a current album;
					if (currentAlbum) {
						var foundPhotoItem = findPhotoItem(self.clickedPhotoId, [currentAlbum]);
						if (!foundPhotoItem.photo) {
							// Photo is not found in a current album;
							// It's moved;
							var previousPhotoPaste = findPhotoItem(self.openedAlbumPreviousPhotoId, [currentAlbum]);
							var nextPhotoPaste = findPhotoItem(self.openedAlbumNextPhotoId, [currentAlbum]);
							if (previousPhotoPaste.photo) {
								previousPhotoPaste.album.photos.splice(previousPhotoPaste.photoIndex + 1, 0, {
									id: self.clickedPhotoId,
									removed: true,
									album: currentAlbum
								});
							} else if (nextPhotoPaste.photo) {
								nextPhotoPaste.album.photos.splice(nextPhotoPaste.photoIndex - 1, 0, {
									id: self.clickedPhotoId,
									removed: true,
									album: currentAlbum
								});
							} else {
								// album is empty;
								currentAlbum.photos = [{id: self.clickedPhotoId, removed: true, album: currentAlbum}];
							}
						}
					}
				}
			}

			for (var i = 0, l = albums.length; i < l; i++) {
				if (albums[i]) {
					albums[i].photos = albums[i].photos || [];
					albumsPreviewSection.appendAlbum(albums[i], defaultAlbum, albumOwner, i === l - 1);
					smallAlbumsPreviewSection.appendPreviewSection(
						self.SmallAlbumSection(albums[i], 'small-album-section')
					);
				}
			}

			var currentPhoto = albumsPreviewSection.getPhotoInAlbum(self.clickedPhotoId, {id: currentAlbumId});
			var adjacentPhotos = albumsPreviewSection.getAdjacentPhotos(currentPhoto);
			var previousPhoto = adjacentPhotos.previous;
			var nextPhoto = adjacentPhotos.next;
			var $likeAnimationIcon = $('<span class="js-like-icn icn icn-top100-huge white icn-huge"></span>').css('visibility', 'hidden');

			var photoFrame = self.PhotoFrame('photo-frame');
			var photo = self.Photo(null, {
				incognitoService: incognitoService,
				incognitoStatus: self.incognitoStatus,
				incognitoRequestStatus: self.incognitoRequestStatus,
				incognitoRelationStatus: self.incognitoRelationStatus,
				userProfile: userProfile,
				$likeAnimationIcon: $likeAnimationIcon,
				photoCssClass: 'photo',
				photoContainerCssClass: 'photo-container',
				photoImageCssClass: 'photo-image',
				incognitoAnimationCssClass: 'animated',
				blackRadialGradientLayerCssClass: 'black-radial-gradient-layer black-radial-gradient',
				incognitoCssClass: 'photo-image-incognito-medium white',
				incognitoMediumIconCssClass: 'icn icn-incognitoMan-medium icn-medium',
				incognitoHugeIconCssClass: 'icn icn-incognitoMan-huge icn-huge',
				photoFrameCtrl: {
					destroy: function () {
					}
				},
				mediumAnimatedClockCssClass: 'animated-clock medium'
			});

			var previousPhoto = self.PreviousPhoto(
				previousPhoto.src,
				'previous-photo',
				'previous-photo-wrap photo-shadow',
				'previous-photo-image',
				!self.IS_OWNER_PROFILE && previousPhoto.erotic
			);
			var nextPhoto = self.NextPhoto(
				nextPhoto.src,
				'next-photo',
				'next-photo-wrap photo-shadow',
				'next-photo-image',
				!self.IS_OWNER_PROFILE && nextPhoto.erotic
			);
			var photoSettingsSection = self.PhotoSettingsSection(
				'photo-settings-section',
				self.IS_OWNER_PROFILE,
				!self.IS_OWNER_PROFILE && userProfile.isReal && mambo.aUserId &&
				(!self.incognitoStatus.isOn || self.incognitoStatus.isOn && self.incognitoRelationStatus.canView)
			);
			var photoCaptionSection = $("<div class='photo-comments-wrap'></div>");//self.PhotoCaptionSection('photo-caption-section');
			var photoCaption = self.PhotoCaption('photo-caption');
			var photoDescription = self.PhotoDescription(currentPhoto, 'photo-description js-photo-description');

			var editPhotoDialog = self.EditPhotoDialog({
				mainPhotoSectionActiveText: context.t.albumsMainPhoto,
				mainPhotoSectionActiveCssClass: 'main-photo-section-active',
				mainPhotoSectionCssClass: 'main-photo-section',
				mainPhotoSectionText: context.t.albumsMakeMainPhoto
			}); // Later on we will call init() to build this dialogs' DOM;
			
			var newFaces = self.PhotolayerNewFaces(); 

			var vkLikeButton = self.VKLikeButton(self.VK_LIKE_BUTTON);
			var fbLikeButton = self.FBLikeButton(self.FB_LIKE_BUTTON);

			// Controllers;
			var likeCtrl = new self.LikeCtrl({
				photoratingService: photoratingService,
				$likeAnimationIcon: $likeAnimationIcon
			});
			var previewSectionCtrl = new self.PreviewSectionCtrl({
				photoLayer: photoLayer,
				advBannerSection: advBannerSection,
				albumsPreviewSection: albumsPreviewSection,
				smallAlbumsPreviewSection: smallAlbumsPreviewSection,
				albumsPreviewSectionWrapper: albumsPreviewSectionWrapper,
				photoSlider: photoSlider,
				photoCaptionSection: photoCaptionSection,
				clickedPhotoId: self.clickedPhotoId,
				currentAlbumId: currentAlbumId,
				openedAlbumPreviousPhotoId: self.openedAlbumPreviousPhotoId,
				openedAlbumNextPhotoId: self.openedAlbumNextPhotoId,
				applicationCtrl: self,
				photoFrame: photoFrame,
				newFaces: newFaces
			});
			var photoFrameCtrl = new self.PhotoFrameCtrl({
				incognitoMode: self.incognitoMode,
				incognitoStatus: self.incognitoStatus,
				incognitoRequestStatus: self.incognitoRequestStatus,
				incognitoRelationStatus: self.incognitoRelationStatus,
				incognitoService: incognitoService,
				photolayerLoader: self.photolayerLoader,
				mambaAPI: mambaAPI,
				advBannerSection: advBannerSection,
				defaultRatings: defaultRatings,
				defaultComments: defaultComments,
				defaultComplaints: defaultComplaints,
				defaultVote: defaultVote,
				vkLikeButton: vkLikeButton,
				fbLikeButton: fbLikeButton,
				userProfile: userProfile,
				albumOwner: albumOwner,
				albumsPreviewSection: albumsPreviewSection,
				albumsPreviewSectionWrapper: albumsPreviewSectionWrapper,
				smallAlbumsPreviewSection: smallAlbumsPreviewSection,
				previousPhoto: previousPhoto,
				nextPhoto: nextPhoto,
				photoFrame: photoFrame,
				Photo: self.Photo,
				photoCaptionSection: photoCaptionSection,
				PhotoNotFound: self.PhotoNotFound,
				EroticWarning: self.EroticWarning,
				openedAlbumPreviousPhotoId: self.openedAlbumPreviousPhotoId,
				openedAlbumNextPhotoId: self.openedAlbumNextPhotoId,
				photo: photo,
				defaultNextPhoto: adjacentPhotos.next,
				defaultPreviousPhoto: adjacentPhotos.previous,
				defaultPhoto: currentPhoto,
				photoSlider: photoSlider,
				photoOverlay: self.photoOverlay,
				photoLayer: photoLayer,
				likeCtrl: likeCtrl,
				photoSettingsSection: photoSettingsSection,
				photoCaption: photoCaption,
				newFaces: newFaces,
				ComplaintDialog: self.ComplaintDialog,
				ComplaintDescription: self.ComplaintDescription,
				ConfirmationDialog: self.ConfirmationDialog,
				editPhotoDialog: editPhotoDialog,
				photoDescription: photoDescription,
				closeButton: closeButton,
				editableAlbums: editableAlbums,
				applicationCtrl: self,
				usePixelization: usePixelization,
				isOwnerProfile: self.IS_OWNER_PROFILE
			});

            var $commentsComponent = $('.comments-component-wrap').children();
			
			var $fragment = $(document.createDocumentFragment());
			$fragment
				.append(photoLayer
					.append(
					$('<div></div>')
						.attr('class', 'photolayer-container js-allowed-to-close')
						.append(
						$('<div></div>')
							.attr('class', 'albums-preview-section-super-wrapper js-albums-preview-section-super-wrapper')
							.append(albumsPreviewSectionWrapper
								.append(albumsPreviewSection.$)
						)
							.append(smallAlbumsPreviewSection.$)
							.append(context.utils.isTouchDevice() ? $(document.createDocumentFragment()) : photoSlider.$)
					)
						.append(
						$('<div></div>')
							.attr({
								'class': 'photo-super-wrapper',
								'id': 'SuperWrapper'
							})
							.append(previousPhoto)
							.append(nextPhoto)
							.append(
							$('<div></div>')
								.attr('class', 'nice-vertical-scrollbar-wrapper')
								.append(
								$('<div></div>')
									.attr('class', 'nice-vertical-scrollbar')
									.append(
									$('<div></div>')
										.attr('class', 'nice-vertical-scrollbar-runner')
								)
							)
						)
							.append(
							$('<div></div>')
								.attr({
									'class': 'photo-wrapper',
									'id': 'Wrapper'
								})
								.append(photo.$
									.append(photoCaptionSection
										.append($commentsComponent)
									)
							)
						)
					)
					.append(newFaces.$)
				)					
					.append(advBannerSection.$.show())
			);
            $commentsComponent.find('.header')
                .append(self.likeButtonAvailable ? likeCtrl.$ : '')
                .find('.photo-ctrls-wrap').html('')
                    .append(photoSettingsSection.$)
                    .append(
                        (!self.incognitoStatus.isOn || self.incognitoStatus.isOn && self.incognitoRelationStatus.canView) &&
                            !self.likeButtonAvailable ? vkLikeButton.$ : ''
                    )
                    .append(
                        (!self.incognitoStatus.isOn || self.incognitoStatus.isOn && self.incognitoRelationStatus.canView) &&
                            !self.likeButtonAvailable? fbLikeButton.$.show() : ''
                    );

			$('body').append($fragment).css({
				'overflow': 'hidden',
				'min-width': '1000px'
			})
				.addClass('photolayer-opened')
				.scrollTop(0);

			$('body').append(
				$('<div></div>')
					.attr('id', 'PhotolayerLoader')
					.css({
						position: 'absolute',
						left: 0,
						top: 0,
						width: '100%',
						height: '100%',
						zIndex: 2147483647
					})
					.on('click', function (event) {
						event.stopPropagation();
					})
			).scrollTop(0);
			$('html').scrollTop(0); // for Firefox;
			self.photoOverlay.removeClass('photolayer-photo-overlay-loading');

			if (!context.utils.isTouchDevice()) {
				$('#Wrapper').nicescrollbar({
					niceScrollType: 'VERTICAL',
					niceScrollStep: 150,
					niceScrollbarCSSSelector: '.nice-vertical-scrollbar-wrapper',
					niceScrollbarIE8CSSClass: 'nice-vertical-scrollbar-ie8',
					niceScrollbarRunnerCSSClass: 'nice-vertical-scrollbar-runner'
				}).triggerHandler('setnicescrollbar', [0]);
			} else {
				albumsPreviewSection.$.css('overflow-x', 'auto');
				$('#Wrapper').css('overflow-y', 'auto');
			}

			previewSectionCtrl.init({
				photoFrameCtrl: photoFrameCtrl
			});

			likeCtrl.init({
				photoFrameCtrl: photoFrameCtrl
			});
			photoFrameCtrl.init({
				previewSectionCtrl: previewSectionCtrl
			});

			self.photolayerStatistics.addData(mambo.vUserId, 1, 2, 'isOpened');
		};
		context.ApplicationCtrl = ApplicationCtrl;
	}


	return module;
});
define('photolayer/ComplaintDescription.js',['require'],function (require) {
	var module = function (context) {
		var months = context.t.months;

		var ComplaintDescription = function () {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof context.ComplaintDescription)) {
				return new context.ComplaintDescription(dependencies);
			}

			var complaintDescriptionCssClass = dependencies.complaintDescriptionCssClass || '';
			var cancelCssClass = dependencies.cancelCssClass || '';

			var cancelText = dependencies.cancelText || context.t.OK;
			var reasonText = dependencies.reasonText || $('#ReasonText').text();
			var reasonDescriptionText = dependencies.reasonDescriptionText || '';
			var publishedDescriptionDate = dependencies.publishedDescriptionDate || '';
			var publishedDescriptionTime = dependencies.publishedDescriptionTime || '';

			self.publishedText = dependencies.publishedText || context.t.complaint;
			self.i_n = dependencies.i_n || context.t.into;
			self.publishedBanText = dependencies.publishedBanText || context.t.photoBlocked;

			var template = '<div class="' + complaintDescriptionCssClass + '">'
				+ '<p class="js-published-description-text">' + self.publishedText + ' ' + publishedDescriptionDate + ' ' + self.i_n + ' ' + publishedDescriptionTime + '</p>'
				+ '<p style="margin-bottom: 5px;" class="js-reason-description">' + reasonText + ': <span class="js-reason-description-text" style="color: #f00;">' + reasonDescriptionText + '</span></p>'
				+ '</div>';

			self.$ = $(template);

			self.$reasonDescription = self.$.find('.js-reason-description');
			self.$reasonDescriptionText = self.$.find('.js-reason-description-text');
			self.$publishedDescriptionText = self.$.find('.js-published-description-text');
		};
		ComplaintDescription.fn = ComplaintDescription.prototype;
		ComplaintDescription.fn.update = function (data) {
			var self = this;
			data = data || {};
			self.$reasonDescriptionText.text(data.reasonDescriptionText || '');

			if (data.publishedDescriptionDate && data.publishedDescriptionTime) {
				self.$publishedDescriptionText.text(self.publishedText + ' ' + data.publishedDescriptionDate + ' ' + self.i_n + ' ' + data.publishedDescriptionTime);
				self.$reasonDescription.show();
			} else if (data.publishedBanText) {
				self.$publishedDescriptionText.text(data.publishedBanText);
				self.$reasonDescription.hide();
			}
			return self;
		};
		ComplaintDescription.fn.toggle = function (value) {
			var self = this;
			if (value) {
				self.$.show();
			} else {
				self.$.hide();
			}
			return self;
		};
		ComplaintDescription.fn.formatPublishedDate = function (dateObject) {
			return dateObject.getDate() + ' ' + months[dateObject.getMonth()] + ' ' + dateObject.getFullYear();
		};
		ComplaintDescription.fn.formatPublishedTime = function (dateObject) {
			return dateObject.getHours() + ':' + dateObject.getMinutes() + ':' + dateObject.getSeconds();
		};

		context.ComplaintDescription = ComplaintDescription;
	}

	return module;
});
define('photolayer/ComplaintDialog.js',['require'],function (require) {
	var module = function (context) {
		var ComplaintDialog = function () {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof context.ComplaintDialog)) {
				return new context.ComplaintDialog(dependencies);
			}

			var complaintDialogCssClass = dependencies.complaintDialogCssClass || '';
			var submitCssClass = dependencies.submitCssClass || '';
			var cancelCssClass = dependencies.cancelCssClass || '';
			var buttonsWrapClass = dependencies.buttonsWrapClass || '';
			var buttonWrapClass = dependencies.buttonWrapClass || '';
			var submitText = dependencies.submitText || context.t.complaintText;
			var cancelText = dependencies.cancelText || context.t.cancel;
			self.reasonText = dependencies.reasonText || context.t.reasonComplaint;
			self.reasonBanText = dependencies.reasonBanText || context.t.reasonBanText;

			var violenceSceneText = dependencies.violenceSceneText || context.t.violenceSceneText;
			var advertisementText = dependencies.advertisementText || context.t.advertisementText;
			var strangePhotoText = dependencies.strangePhotoText || context.t.strangePhotoText;
			var pornoPhotoText = dependencies.pornoPhotoText || $('#PornographyText').text();

			var template = '<div class="' + complaintDialogCssClass + '">'
				+ '<p class="js-complaint-reason-text" style="margin-bottom: 10px;">' + self.reasonText + ':</p>'
				+ '<ul style="margin-bottom: 10px;" class="js-complaint-reasons-list">'
				+ '<li><label><input class="js-radio-button" type="radio" value="1" name="cause"/>' + advertisementText + '</label></li>'
				+ '<li><label><input class="js-radio-button" type="radio" value="2" name="cause"/>' + strangePhotoText + '</label></li>'
				+ '<li><label><input class="js-radio-button" type="radio" value="3" name="cause"/>' + pornoPhotoText + '</label></li>'
				+ '<li><label><input class="js-radio-button" type="radio" value="4" name="cause"/>' + violenceSceneText + '</label></li>'
				+ '</ul>'
				+ '<div class="' + buttonsWrapClass + '">'
				+ '<span class="' + buttonWrapClass + '"><button type="button" class="js-submit-button ' + submitCssClass + '">' + submitText + '</button></span>'
				+ '<span class="' + buttonWrapClass + '"><button type="reset" class="js-cancel-button ' + cancelCssClass + '">' + cancelText + '</button></span>'
				+ '</div>'
				+ '</div>';

			self.$ = $(template);
			self.$submitButton = $('.js-submit-button', self.$);
			self.$cancelButton = $('.js-cancel-button', self.$);
			self.$radioButtons = $('.js-radio-button', self.$);
			self.$complaintReasonText = $('.js-complaint-reason-text', self.$);
		};

		ComplaintDialog.fn = ComplaintDialog.prototype;
		ComplaintDialog.fn.resetValues = function () {
			var self = this;
			self.$radioButtons.prop('checked', false).parent().removeClass('red-color'); // TODO: refactor! red-color hardcode!
			self.$submitButton.prop('disabled', true);
			return self;
		};

		ComplaintDialog.fn.setComplaintReasons = function (reasons) {
			var self = this;
			reasons = reasons || {};
			var $complaintReasonsList = self.$.find('.js-complaint-reasons-list');
			$complaintReasonsList.html('');
			for (var i in reasons) {
				$complaintReasonsList.append('<li><label>' +
				'<input class="js-radio-button" type="radio" value="' + reasons[i].causeId + '" name="cause"/>' + reasons[i].text +
				'</label></li>');
			}
			self.$radioButtons = $('.js-radio-button', self.$);
			return self;
		};

		ComplaintDialog.fn.updateComplaintReasonText = function (newComplaintReasonText) {
			var self = this;
			self.$complaintReasonText.text(newComplaintReasonText);
			return self;
		};

		context.ComplaintDialog = ComplaintDialog;
	}

	return module;

});
define('photolayer/ConfirmationDialog.js',['require'],function (require) {
	var module = function (context) {
		function Button(text, cssClass) {
			return $('<button></button>')
				.attr({
					'class': cssClass || '',
					'type': 'button'
				})
				.text(text || '');
		}

		context.ConfirmationDialog = function (cssClass, submitText, submitCssClass, cancelText, cancelCssClass) {
			var self = this;
			if (!(self instanceof context.ConfirmationDialog)) {
				return new context.ConfirmationDialog(cssClass, submitText, submitCssClass, cancelText, cancelCssClass);
			}
			self.$submitButton = Button(submitText, submitCssClass);
			self.$cancelButton = Button(cancelText, cancelCssClass);
			self.$ = $('<div></div>')
				.attr('class', cssClass || '')
				.append(self.$submitButton)
				.append(self.$cancelButton)
		};
	}

	return module;
});
define('photolayer/EditPhotoDialog.js',['require'],function (require) {
	var module = function (context) {
		var EditPhotoDialog = function () {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof EditPhotoDialog)) {
				return new EditPhotoDialog(dependencies);
			}

			self.mainPhotoSectionActiveCssClass = dependencies.mainPhotoSectionActiveCssClass || 'main-photo-section-active';
			self.mainPhotoSectionCssClass = dependencies.mainPhotoSectionCssClass || 'main-photo-section';
			self.mainPhotoSectionActiveText = dependencies.mainPhotoSectionActiveText || context.t.albumsMainPhoto;
			self.mainPhotoSectionText = dependencies.mainPhotoSectionText || context.t.albumsMakeMainPhoto;

			self.reCropPhotoActive = true;
			self.reCropPhotoActiveAll = true;
			self.reCropPhotoBlockedArray = [];
		};
		EditPhotoDialog.fn = EditPhotoDialog.prototype;
		EditPhotoDialog.fn.init = function (templateId, userAlbums, userRatings) {
			var self = this;
			userAlbums = userAlbums || [];
			userRatings = userRatings || [];

			templateId = templateId || '#photolayer-EditPhotoDialogTmpl';

			self.$ = $(templateId).tmpl({
				albums: userAlbums, ratings: userRatings
			});

			self.$photoDescription = self.$.find('.js-edit-photo-dialog-description');
			self.$ratingsMenu = self.$.find('.js-edit-photo-dialog-user-ratings');
			self.$albumsMenu = self.$.find('.js-edit-photo-dialog-user-albums');
			self.$cancelButton = self.$.find('.js-cancel-button');
			self.$submitButton = self.$.find('.js-submit-button');
			self.$symbolsLimit = self.$.find('.js-edit-photo-dialog-symbols-limit');
			self.$ratingsMenuSection = self.$.find('.js-ratings-menu');
			self.$mainPhotoSection = self.$.find('.js-main-photo-section');
			self.$mainPhotoSectionText = self.$.find('.js-main-photo-section-text');
			self.$reCropPhoto = self.$.find('.js-re-crop-photo');
			self.$reCropPhotoText = self.$reCropPhoto.find('.js-re-crop-photo-text');

			self.lastSelectedRatingId = 0;
			self.lastSelectedAlbumId = 0;
			
			if ( navigator.userAgent.search(/AppleWebKit/) > -1 ) {
				self.$photoDescription.removeAttr('maxlength');
			}
			self.photoDescriptionOldValue = self.$photoDescription.val();
			
			return self;
		};
		EditPhotoDialog.fn.updateRatings = function (newUserRatings) {
			var self = this;
			if (!self.$ratingsMenu) {
				return self;
			}
			self.$ratingsMenu.html('');
			newUserRatings = newUserRatings || [];
			var selectedRating;
			for (var i = 0, l = newUserRatings.length; i < l; i++) {
				self.$ratingsMenu.append(
					$('<option value="' + newUserRatings[i].value + '">' + newUserRatings[i].text + '</option>')
				);
				if (newUserRatings[i].selected) {
					selectedRating = newUserRatings[i];
				}
			}
			if (selectedRating) {
				self.selectRating(selectedRating.id);
			}
			return self;
		};
		EditPhotoDialog.fn.updateAlbums = function(newUserAlbums) {
			var self = this,
				selectedAlbum,
				l, i;
			
			if (!self.$albumsMenu) {
				return;
			}
			
			self.$albumsMenu.html('');
			newUserAlbums = newUserAlbums || [];
			for (i = 0, l = newUserAlbums.length; i < l; i++) {
				self.$albumsMenu.append(
					$('<option value="' + newUserAlbums[i].value + '">' + newUserAlbums[i].text + '</option>')
				);
				if (newUserAlbums[i].selected) {
					selectedAlbum = newUserAlbums[i];
				}
			}
			
			if (selectedAlbum) {
				self.selectAlbum(selectedAlbum.key);
			}
			
			return self;
		};
		EditPhotoDialog.fn.toggleRatingsMenu = function (value) {
			var self = this;
			if (!self.$ratingsMenuSection) {
				return self;
			}
			if (value) {
				self.$ratingsMenuSection.show();
			} else {
				self.$ratingsMenuSection.hide();
			}
			return self;
		};
		EditPhotoDialog.fn.toggleAlbum = function (value, album) {
			var self = this;
			album = album || {};
			if (!self.$albumsMenu) {
				return self;
			}
			var $albumOption = self.$albumsMenu.find('option[value="' + album.id + '"]');
			if (value) {
				if (!$albumOption.length) {
					self.$albumsMenu.prepend(
						$('<option></option>').val(album.id).text(album.name || '')
					);
				}
			} else {
				$albumOption.remove();
			}
			return self;
		};
		EditPhotoDialog.fn.selectAlbum = function (albumId) {
			var self = this;
			var albums = self.$albumsMenu.get(0);
			var options = albums.getElementsByTagName('option') || [];
			for (var i = 0, l = options.length; i < l; i++) {
				if (options[i].value * 1 === albumId * 1) {
					options[i].selected = true;
					self.lastSelectedAlbumId = albumId * 1;
				}
			}
			return self;
		};
		EditPhotoDialog.fn.selectRating = function (ratingId) {
			var self = this;
			var $ratings = self.$ratingsMenu.find('option[value="' + ratingId + '"]');
			if ($ratings.length) {
				$ratings.prop('selected', true);
				self.lastSelectedRatingId = ratingId * 1;
			}
			return self;
		};
		EditPhotoDialog.fn.toggleMainPhotoSection = function (value) {
			var self = this;
			if (!self.$mainPhotoSection) {
				return self;
			}
			if (value) {
				self.$mainPhotoSection.show();
			} else {
				self.$mainPhotoSection.hide();
			}
			return self;
		};
		EditPhotoDialog.fn.showReCropSection = function (value) {
			var self = this;
			this.$reCropPhoto.toggle(value);
			return self;
		};
		EditPhotoDialog.fn.blockedReCropSectionAddBlockedPhotoById = function( id ) {
			var self = this;
			self.reCropPhotoBlockedArray.push(id);
			return self;
		};

		EditPhotoDialog.fn.blockedReCropSectionAllPhoto = function () {
			var self = this;
			this.reCropPhotoActiveAll = false;
			return self;
		};
		EditPhotoDialog.fn.blockedReCropSection = function (value) {
			var self = this;
			if (!self.$reCropPhotoText) {
				return self;
			}
			var isModeration;
			if (!self.reCropPhotoActiveAll) {
				isModeration = true;
			} else {
				isModeration = (typeof value === 'object' ?
									(!!(self.reCropPhotoBlockedArray.indexOf( value.id ) + 1) ? true : value.status === 'MODERATING') : value);
			}
			self.reCropPhotoActive = !isModeration;
			self.$reCropPhotoText.toggleClass('inactive', isModeration);

			return self;
		};
		EditPhotoDialog.fn.updateMainPhotoSection = function (photo) {
			var self = this;
			if (!self.$mainPhotoSection) {
				return self;
			}
			photo = photo || {};
			self.$mainPhotoSection
				.toggleClass(self.mainPhotoSectionActiveCssClass, photo.current)
				.toggleClass(self.mainPhotoSectionCssClass, !photo.current);
			self.$mainPhotoSectionText.text(photo.current ? self.mainPhotoSectionActiveText : self.mainPhotoSectionText);
			self.mainPhotoSectionActive = photo.current ? false : true;
			return self;
		};
		EditPhotoDialog.fn.toggleSubmitButtonState = function () {
			var self = this;
			if (!self.$photoDescription || !self.$albumsMenu || !self.$ratingsMenu || !self.$submitButton) {
				return self;
			}
			if (self.$photoDescription.val() === undefined) {
                self.$submitButton.prop('disabled', false);

                return self;
			}
            var enabledState = self.$photoDescription.val().trim() !== self.$photoDescription.data('photoDescription') ||
				self.$albumsMenu.find('option:selected').val() != self.lastSelectedAlbumId ||
				self.$ratingsMenu.find('option:selected').val() != self.lastSelectedRatingId;
			self.$submitButton.prop('disabled', !enabledState);

			return self;
		};
		EditPhotoDialog.fn.updateSymbolsLimit = function (newSymbolsCount) {
			var self = this;
			if (!self.$symbolsLimit || !self.$photoDescription) {
				return self;
			}
			var MAX_LIMIT = 50;
			if (typeof newSymbolsCount !== 'number') {
				newSymbolsCount = self.$photoDescription.val().length;
			}
			var txt = self.$photoDescription.val();
			var charsLeft = MAX_LIMIT - newSymbolsCount;
			if (charsLeft < 0) {
				charsLeft = 0;
				var newVal = self.photoDescriptionOldValue.length === MAX_LIMIT ? self.photoDescriptionOldValue: txt.substr(0, MAX_LIMIT);
				self.$photoDescription.val(newVal);
				newSymbolsCount = newVal.length;
			}
			self.photoDescriptionOldValue = newVal || txt;
			self.$symbolsLimit.text(MAX_LIMIT - newSymbolsCount <= 0 ? 0 : MAX_LIMIT - newSymbolsCount);
			return self;
		};
		context.EditPhotoDialog = EditPhotoDialog;
	}


	return module;
});

define('photolayer/EroticWarning.js',['require'],function (require) {
	var module = function (context) {
		var EroticWarning = function (photo) {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof EroticWarning)) {
				return new EroticWarning(photo, dependencies);
			}
			photo = photo || {};
			self.id = photo.id;
			self.src = photo.src;
			self.album = photo.album;
			self.description = photo.description;
			self.type = 'EROTIC';

			var photoCssClass = dependencies.photoCssClass || '';
			var eroticWarningCssClass = dependencies.eroticWarningCssClass + ' js-photo-container' || '';
			var proceedButtonCssClass = dependencies.proceedButtonCssClass || '';
			var insetCssClass = dependencies.insetCssClass || '';
			var insetTextCssClass = dependencies.insetTextCssClass;
			var headerText = dependencies.headerText || context.t.photoErotic;
			var proceedButtonText = dependencies.proceedButtonText || context.t.photoContinueView;
			var settingsLink = dependencies.settingsLink || '/my/settings_erotic.phtml';
			var settingsLinkText = dependencies.settingsLinkText || context.t.settings;
			var settingsLinkDescription = dependencies.settingsLinkDescription || context.t.description.settingsLinkDescription;
			var warningCheckboxText = dependencies.warningCheckboxText || context.t.description.warningCheckboxText;

			self.$proceedButton = $('<button></button>')
				.attr({
					'type': 'button',
					'class': proceedButtonCssClass
				})
				.text(proceedButtonText);
			self.$warningCheckbox = $('<input/>').attr('type', 'checkbox');

			var insetHtmlBuffer = [
				$('<h1></h1>').text(headerText),
				$('<p></p>')
					.attr('class', insetTextCssClass)
					.append($('<span></span>').text(settingsLinkDescription))
					.append(' &laquo;')
					.append($('<a></a>').attr('href', settingsLink).text(settingsLinkText))
					.append('&raquo;.'),
				$('<p></p>')
					.attr('class', insetTextCssClass)
					.append(
					$('<label></label>')
						.append(self.$warningCheckbox)
						.append($('<span></span>').text(warningCheckboxText))
				),
				self.$proceedButton
			];

			self.$photoContainer = $('<div></div>')
				.attr('class', eroticWarningCssClass)
				.append(
				$('<div></div>').attr('class', insetCssClass).append(insetHtmlBuffer)
			);
			self.$ = $('<div></div>')
				.attr('class', 'js-photo ' + photoCssClass)
				.append(self.$photoContainer);
		};
		context.EroticWarning = EroticWarning;
	}

	return module;
});

define('photolayer/HttpErrorModal.js',['require'],function (require) {
	var module = function (context) {
		var HttpErrorModal = function () {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof HttpErrorModal)) {
				return new HttpErrorModal(dependencies);
			}
			var templateId = dependencies.templateId || '#photolayer-HttpErrorModalTmpl';
			self.$ = $(templateId).tmpl()
			self.$submitButton = self.$.find('.js-submit-button');
		};
		context.httpErrorModal = new HttpErrorModal();
	}

	return module;
});

define('photolayer/MobileMambaAPI.js',['require'],function (require) {

	// Mobile Mamba API
	var module = function (context, mambo) {
		context.mambaAPI = context.mambaAPI || {};
		context.mambaAPI.genericApiGet = function (type, params, callback) {
			$.getJSON(urlFactory(params, type), function (res) {
				(typeof callback === 'function' ? callback : function () {
				})(res);
			});
		};
		context.mambaAPI.genericApiPost = function (type, params, callback) {
			$.post(urlFactory(params, type), JSON.stringify(params), function (res) {
				(typeof callback === 'function' ? callback : function () {
				})(res);
			});
		};
		context.mambaAPI.genericApiPut = function (type, params, callback) {
			callback = typeof callback === 'function' ? callback : function () {
			};
			$.ajax({
				type: 'PUT',
				contentType: 'application/json;charset=utf-8',
				url: urlFactory(params, type),
				data: JSON.stringify(params)
			}).done(function (data) {
				callback(null, data);
			}).fail(function (error) {
				callback(error);
			});
		};
		context.mambaAPI.genericApiDelete = function (type, params, callback) {
			callback = typeof callback === 'function' ? callback : function () {
			};
			$.ajax({
				type: 'DELETE',
				contentType: 'application/json;charset=utf-8',
				url: urlFactory(params, type),
				data: JSON.stringify(params)
			}).done(function (data) {
				callback(null, data);
			}).fail(function (error) {
				callback(error);
			});
		};

		context.mambaAPI.containerRequest = function (params, sysRequestsContainer, callback) {
			params = params || {};
			sysRequestsContainer = sysRequestsContainer || [];
			var langId = params.langId || 'en';
			var dateType = params.dateType || 'timestamp';
			var limit = params.limit || 10000;

			var url = '/mobile/api/v5.17.0.0/?reqType=json';
			var data = {
				langId: langId,
				dateType: dateType,
				limit: limit,
				sysRequestsContainer: sysRequestsContainer
			};
			callback = typeof callback === 'function' ? callback : function () {
			};
			$.post(url, JSON.stringify(data)).done(function (data) {
				callback(!!data.errorCode || !data.sysResponsesContainer, data);
			}).fail(function () {
				callback({});
			});
		};

		function urlFactory(params, type) {
			params = params || {};
			var applicationId = params.applicationId;
			var url = ['/mobile/api/v5.17.0.0'];
			var langId = params.langId || 'en';
			var complaintType = params.complaintType || '';
			var entityId = params.entityId || '';
			var albumId = params.albumId;
			var photoId = params.photoId;
			var complaint = params.complaint || {};
			var commentId = params.commentId;
			var limit = params.limit || 10000;
			var offset = params.offset || 0;
			var dateType = params.dateType || 'timestamp';
			var commentText = params.commentText || '';

			if (type !== 'ratings' &&
				type !== 'putPhoto' &&
				type !== 'deletePhoto' &&
				type !== 'postComplaint' &&
				type !== 'mainPhoto' &&
				type !== 'checkComplaint' &&
				type !== 'complaintsList' &&
				type !== 'cropPhoto'
			) {
				url.push('/users/' + applicationId);
			}

			switch (type) {
				case 'albums':
					url.push('/albums/?');
					url.push('langId=' + langId + '&dateType=' + dateType);
					break;
				case 'photos':
					url.push('/albums/' + albumId + '/photos/?');
					url.push('limit=' + limit + '&offset=' + offset + '&langId=' + langId + '&dateType=' + dateType);
					break;
				case 'comments':
					url.push('/photos/' + photoId + '/comments/?');
					url.push('limit=' + limit + '&langId=' + langId + '&dateType=' + dateType);
					break;
				case 'ratings':
					url.push('/photos/' + photoId + '/edit/?');
					url.push('langId=' + langId + '&dateType=' + dateType);
					break;
				case 'postComment':
					url.push('/photos/' + photoId + '/comments/');
					break;
				case 'deletePhoto':
					url.push('/photos/' + photoId + '/?');
					url.push('langId=' + langId + '&dateType=' + dateType);
					break;
				case 'deleteComment':
					url.push('/photos/' + photoId + '/comments/' + commentId + '/');
					break;
				case 'putPhoto':
					url.push('/photos/' + photoId + '/?');
					url.push('lang_id=' + langId + '&reqType=json')
					break;
				case 'postComplaint':
					url.push('/complaint/add/' + applicationId + '/');
					break;
				case 'checkComplaint':
					url.push('/complaint/photo/' + applicationId + '/' + photoId + '/?');
					url.push('lang_id=' + langId + '&reqType=json');
					break;
				case 'cropPhoto':
					url.push('/photos/recrop/');
					break;
				case 'complaintsList':
					url.push('/complaint/causes/' + applicationId + '/' + complaintType + '/' + entityId + '/?');
					url.push('lang_id=' + langId + '&reqType=json');
				case 'mainPhoto':
					url.push('/profile/main_photo/');
				default:
					url.push('');
			}
			return url.join('');
		}

		//})(photolayer, mambo);


		var Tooltip = function (cssClass, text) {
			var self = this;
			if (!(self instanceof Tooltip)) {
				return new Tooltip(cssClass, text);
			}
			self.$ = $('<span></span>')
				.attr('class', cssClass || '')
				.append(
				self.$restrictReasonText = $('<p>' + text + '</p>')
			)
				.append(
				self.$activateVipStatusButton = $('<button  data-from-photo="true" type="button" class="js-storefront-vip activate-vip-button button button-small button-ghost-blue">' + context.t.vipActiveState + '</button>')
					.on('mousedown', function () {
						$(this).css('background', 'rgba(0, 102, 204, 0.2)')
					})
					.on('mouseenter', function () {
						$(this).css('background', 'rgba(0, 102, 204, 0.1)')
					})
					.on('mouseleave', function () {
						$(this).css('background', 'transparent')
					})
			)
				.append(
				self.$activateVipButton = $('<button  data-from-photo="true" type="button" class="js-storefront-vip activate-vip-button button button-small button-blue" style="margin-right: 10px;">' + context.t.vipActive + '</button>')
			)
				.append(
				self.$loadPhotoButton = $('<button type="button" class="activate-vip-button js-open-photo-uploader button button-small button-ghost-blue">' + context.t.uploadPhoto + '</button>')
					.on('mousedown', function () {
						$(this).css('background', 'rgba(0, 102, 204, 0.2)')
					})
					.on('mouseenter', function () {
						$(this).css('background', 'rgba(0, 102, 204, 0.1)')
					})
					.on('mouseleave', function () {
						$(this).css('background', 'transparent')
					})
			);
			$(document).on('mouseup', function () {
				self.$loadPhotoButton.css('background', 'transparent');
				self.$activateVipStatusButton.css('background', 'transparent');
			});
		};
		Tooltip.fn = Tooltip.prototype;
		Tooltip.fn.toggle = function (value) {
			var self = this;
			self.$.toggle(value);
			return self;
		};
		Tooltip.fn.setPosition = function (left, top) {
			var self = this;
			if (left == null && top != null) {
				self.$.css('top', top + 'px');
			} else if (top == null && left != null) {
				self.$.css('left', left + 'px');
			} else {
				self.$.css({
					left: left + 'px',
					top: top + 'px'
				});
			}
			return self;
		};
		context.Tooltip = Tooltip;


		var VKLikeButton = function (showVKLikeButton) {
			var self = this;
			if (!(self instanceof VKLikeButton)) {
				return new VKLikeButton(showVKLikeButton);
			}
			if (!showVKLikeButton) {
				self.$ = $(document.createDocumentFragment());
			} else {
				self.$ = $('<div>' + context.t.share + '</div>').attr('class', 'vk-button');
			}
		};
		context.VKLikeButton = VKLikeButton;


		var FBLikeButton = function (showFBLikeButton) {
			var self = this;
			if (!(self instanceof FBLikeButton)) {
				return new FBLikeButton(showFBLikeButton);
			}
			if (!showFBLikeButton) {
				self.$ = $(document.createDocumentFragment());
			} else {
				self.$ = $('<div>' + context.t.share + '</div>').attr('class', 'fb-button');
			}
		};
		context.FBLikeButton = FBLikeButton;

	}
	return module;
});
define('photolayer/Photo.js',['require'],function (require) {
	// Photo;
	var module = function (context) {
    var ko = context.knockout;
    
    ko.components.loaders.unshift(ko.components.defaultLoader); // DO NOT REMOVE!!! SPECIAL Components HACK!!!
		ko.components.register('photo-incognito-layer', {
			viewModel: {
				createViewModel: function (params, componentInfo) {
					var model = {};
					var componentElement = componentInfo.element;
					model.requesting = ko.observable(false);
					model.requested = ko.observable(false);

					model.isPhotoOwner = params.isPhotoOwner;
					model.incognitoIsRequested = params.incognitoIsRequested;
					model.albumOwner = params.albumOwner;
					model.isDeclined = ko.observable(false);
					model.isSent = ko.observable(false);
					model.incognitoRequestForUnAuth = params.incognitoRequestForUnAuth;

					var $incognitoIcon = params.$incognitoIcon;
					var $blackRadialGradientLayer = params.$blackRadialGradientLayer;
					var incognitoHugeIconCssClass = params.incognitoHugeIconCssClass;
					var incognitoMediumIconCssClass = params.incognitoMediumIconCssClass;
					var incognitoAnimationCssClass = params.incognitoAnimationCssClass;
					var incognitoService = params.incognitoService;

					var photoFrameCtrl = params.photoFrameCtrl;

					$(componentElement).click(function (event) {
						event.stopPropagation();
					});

					$(params.incognitoRequestStatus).on('incognitoRequestStatusChange', function (event, data) {
						model.requesting(data && data.isPending || false);
						model.requested(data && data.isSent || false);
						model.isDeclined(data && data.isDeclined || false);
						model.isSent(data && data.isSent || false);
					});

					model.hideLinkClickHandler = function (model, event) {
						$incognitoIcon.css({
							top: '85%',
							marginLeft: '-25px',
							marginRight: ''
						}).removeClass(incognitoHugeIconCssClass)
							.addClass(incognitoMediumIconCssClass);

						$(componentElement).css('top', '100%');
						$blackRadialGradientLayer.fadeOut(700);

						if (model.isPhotoOwner) {
							photoFrameCtrl.photo.$photoImage.attr('src', photoFrameCtrl.photo.src);
						}

						if (photoFrameCtrl.photo.$photoImage.height() <= photoFrameCtrl.applicationCtrl.PHOTO_FRAME_MIN_HEIGHT) {
							setTimeout(function () {
								$incognitoIcon.show();
							}, 1000);
						}
					};

					model.requestIncognitoClickHandler = function (model, event) {
						if (model.requested() || model.requesting()) {
							return;
						}

						model.requesting(true);
						incognitoService.requestIncognitoStatus({aid: model.albumOwner.id}).then(function (res) {
							model.requesting(false);
							model.requested(true);

							// incognito is requested! So we don't need to do that anymore;
							incognitoIsRequested = true;
						}, function () {
							photoFrameCtrl.destroy();
							setTimeout(function () {
								var height = screen.availHeight < 800 ? screen.availHeight - 150 : 620;
								var chatWin = window.open('/my/message.phtml?uid=' + model.albumOwner.id, 'Chateg', 'width=750,height=' + height + ',resizable=1,scrollbars=1');
								if (chatWin) {
									chatWin.focus();
								}
							}, 1000);
						});
					};

					model.incognitoUnAuthUserRedirect = function () {

						var redirectUrl = '/' + mambo.aUserLang + '/tips/?tip=login&reason=incognito&goto=anketa_' + model.albumOwner.id;

						Mb.Storage.setLocalStorage('incognitoUserCurrentPhotoUrl', window.location.href);
						window.location = redirectUrl;
					};

					// to understand why we need this use console.log(ko.components.loaders); 
					ko.components.loaders.shift(); // DO NOT REMOVE!!! SPECIAL Components HACK!!!
					return model;
				}
			},
			template: {element: 'photolayer-PhotoIncognitoLayerTmpl'}
		});

		// fucking global variable;
		// Relax, man! It is available within Photo module only;)
		// But I can make it super-puper-duper global!
		var incognitoIsRequested = false;

		var Photo = function (photo) {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof Photo)) {
				return new Photo(photo, dependencies);
			}

			var incognitoRelationStatus = dependencies.incognitoRelationStatus || {};
			var incognitoStatus = dependencies.incognitoStatus || {};
			var incognitoService = dependencies.incognitoService || {};
			var incognitoRequestStatus = dependencies.incognitoRequestStatus || {};
			var userProfile = dependencies.userProfile || {};
			var photoCssClass = dependencies.photoCssClass || '';
			var photoContainerCssClass = dependencies.photoContainerCssClass + ' js-photo-container' || '';
			var photoImageCssClass = dependencies.photoImageCssClass || '';
			var blackRadialGradientLayerCssClass = dependencies.blackRadialGradientLayerCssClass || '';
			var incognitoCssClass = dependencies.incognitoCssClass || '';
			var photoFrameCtrl = dependencies.photoFrameCtrl;
			var isPhotoOwner = photo && userProfile && photo.ownerId === userProfile.id;

			self.mediumAnimatedClockCssClass = dependencies.mediumAnimatedClockCssClass || '';
			self.incognitoAnimationCssClass = dependencies.stopIncognitoAnimation ? '' : dependencies.incognitoAnimationCssClass;
			self.incognitoMediumIconCssClass = dependencies.incognitoMediumIconCssClass || '';
			self.incognitoHugeIconCssClass = dependencies.incognitoHugeIconCssClass || '';

			if (photo) {
				self.id = photo.id;
				self.album = photo.album;
				self.src = photo.src;
				self.srcMasked = photo.srcMasked;
				self.description = photo.description;
				self.type = 'REGULAR';
				self.verified = photo.verified;
			}

			self.$photoContainer = $('<div></div>').attr('class', photoContainerCssClass);
			self.$photoImage = $('<img/>').attr('class', photoImageCssClass);

			if (incognitoStatus.isOn && !incognitoRelationStatus.canView || incognitoStatus.isOn && isPhotoOwner || incognitoStatus.isOn && !mambo.aUserId) {
				self.$incognitoIcon = $('<div></div>').attr('class', incognitoCssClass + ' ' + self.incognitoAnimationCssClass + ' ' + self.incognitoMediumIconCssClass);
			} else {
				self.$incognitoIcon = $(document.createDocumentFragment());
			}
			self.$photoImageCover = $('<div></div>').attr('class', 'photo-image-cover js-photo-image-cover');
			self.$blackRadialGradientLayer = $('<div></div>').attr('class', blackRadialGradientLayerCssClass);
			if(self.verified){
				self.$photoVerified = $('<div></div>').attr('class', 'photo-image-verified');
			}


			self.$photoIncognitoLayer = $('<photo-incognito-layer/>')
				.attr('class', 'photo-incognito-layer')
				.attr('params', '$incognitoIcon: $incognitoIcon,\
                                         $blackRadialGradientLayer: $blackRadialGradientLayer,\
                                         incognitoHugeIconCssClass: incognitoHugeIconCssClass,\
                                         incognitoMediumIconCssClass: incognitoMediumIconCssClass,\
                                         incognitoAnimationCssClass: incognitoAnimationCssClass,\
                                         albumOwner: albumOwner,\
                                         isPhotoOwner: isPhotoOwner,\
                                         incognitoIsRequested: incognitoIsRequested,\
                                         incognitoService: incognitoService,\
                                         incognitoRequestStatus: incognitoRequestStatus,\
                                         photoFrameCtrl: photoFrameCtrl,\
                                         incognitoRequestForUnAuth: incognitoRequestForUnAuth\
                        ');
			self.$ = $('<div></div>')
				.attr('class', 'js-photo ' + photoCssClass)
				.append(
				self.$photoContainer.append(dependencies.$likeAnimationIcon).append(self.$photoVerified).append(self.$photoImage).append(
					self.$photoImageCover
						.append(incognitoStatus && incognitoStatus.isOn ? self.$blackRadialGradientLayer : $(document.createDocumentFragment()))
						.append(incognitoStatus && incognitoStatus.isOn ? self.$photoIncognitoLayer : $(document.createDocumentFragment()))
						.append(incognitoStatus && incognitoStatus.isOn ? self.$incognitoIcon : $(document.createDocumentFragment()))
				)
			);

			ko.applyBindings({
				incognitoHugeIconCssClass: self.incognitoHugeIconCssClass,
				incognitoMediumIconCssClass: self.incognitoMediumIconCssClass,
				incognitoAnimationCssClass: self.incognitoAnimationCssClass,
				albumOwner: photo ? {id: photo.ownerId, gender: photo.ownerGender} : {id: 0},
				$incognitoIcon: self.$incognitoIcon,
				$blackRadialGradientLayer: self.$blackRadialGradientLayer,
				isPhotoOwner: isPhotoOwner,
				incognitoIsRequested: incognitoIsRequested,
				incognitoService: incognitoService,
				incognitoRequestStatus: incognitoRequestStatus,
				photoFrameCtrl: photoFrameCtrl,
				incognitoRequestForUnAuth: incognitoStatus.isOn && !mambo.aUserId ? true : false
			}, self.$[0]);
		};
		context.Photo = Photo;


		// PhotoNotFound;
		var PhotoNotFound = function (photo, templateId) {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			if (!(self instanceof PhotoNotFound)) {
				return new PhotoNotFound(photo, templateId, dependencies);
			}

			var photoCssClass = dependencies.photoCssClass || '';
			self.proceedButtonDefaultText = dependencies.proceedButtonDefaultText || '';
			self.proceedButtonFromStartText = dependencies.proceedButtonFromStartText || '';
			self.proceedButtonCloseText = dependencies.proceedButtonCloseText || '';

			photo = photo || {};
			self.id = photo.id;
			self.removed = true;
			self.album = photo.album || {id: 0};

			templateId = templateId || '#photolayer-PhotoNotFoundTmpl';
			self.$photoContainer = $(templateId).tmpl();
			self.$proceedButton = self.$photoContainer.find('.js-proceed-button');

			self.$ = $('<div></div>')
				.attr('class', 'js-photo ' + photoCssClass)
				.append(self.$photoContainer);



		};
		PhotoNotFound.fn = PhotoNotFound.prototype;
		PhotoNotFound.fn.updateProceedButtonText = function (newText) {
			var self = this;
			self.$proceedButton.text(newText || self.proceedButtonDefaultText);
			return self;
		};
		context.PhotoNotFound = PhotoNotFound;


		var DeleteIcon = function () {
			return $('<div></div>')
				.attr('class', 'delete');
		};
		var SettingsIcon = function () {
			return $('<div></div>')
				.attr('class', 'settings-icon');
		};
		var ComplaintIcon = function () {
			return $('<div></div>')
				.attr('class', 'complaint-icon');
		};
		var PhotoSettingsSection = function (cssClass, photoSettings, showComplainIcon) {
			var self = this;
			if (!(self instanceof PhotoSettingsSection)) {
				return new PhotoSettingsSection(cssClass, photoSettings, showComplainIcon);
			}
			self.$deleteIcon = DeleteIcon();
			self.$settingsIcon = photoSettings ? SettingsIcon() : $(document.createDocumentFragment());
			self.$complaintIcon = showComplainIcon ? ComplaintIcon() : $(document.createDocumentFragment());
			self.$ = $('<div></div>').attr('class', cssClass || '');
			if (photoSettings) {
				self.$.append(self.$settingsIcon).append(self.$deleteIcon);
			} else {
				self.$.append(self.$complaintIcon);
			}
		};
		PhotoSettingsSection.fn = PhotoSettingsSection.prototype;
		PhotoSettingsSection.fn.toggle = function (value) {
			var self = this;
			if (value) {
				self.$.show();
			} else {
				self.$.hide();
			}
			return self;
		};
		context.PhotoSettingsSection = PhotoSettingsSection;
	}

	return module;
});

define('photolayer/PhotoCaption.js',['require'],function (require) {
	var module = function (context) {
		function PhotoCaption(cssClass) {
			return $('<div></div>').attr('class', cssClass || '');
		}

		context.PhotoCaption = PhotoCaption;


		var PhotoCaptionSection = function (cssClass) {
			var self = this;
			if (!(self instanceof PhotoCaptionSection)) {
				return new PhotoCaptionSection(cssClass);
			}

			var $original = $('<div></div>').attr('class', cssClass || '');
			var $replacer = $('<span></span>');
			self.$$getOriginal = function () {
				return {
					$: $original,
					$replacer: $replacer
				};
			};

			self.$ = self.$$getOriginal().$;
		};
		PhotoCaptionSection.fn = PhotoCaptionSection.prototype;
		PhotoCaptionSection.fn.toggle = function (value) {
			var self = this;
			if (value) {
				self.$.show();
			} else {
				self.$.hide();
			}
			return self;
		};
		context.PhotoCaptionSection = PhotoCaptionSection;
	}

	return module;
});
define('photolayer/PhotoDescription.js',['require'],function (require) {
	var module = function (context) {
		var PhotoDescription = function (photo, cssClass) {
			var self = this;
			if (!(self instanceof PhotoDescription)) {
				return new PhotoDescription(photo, cssClass);
			}
			self.$$span = $('<span></span>').text(photo ? photo.description : '');
			self.$ = $('<div></div>')
				.append(self.$$span)
				.attr('class', cssClass || '');
		};
		PhotoDescription.fn = PhotoDescription.prototype;
		PhotoDescription.fn.updateText = function (newText) {
			var self = this;
			if (newText != null) {
				self.$$span.text(newText);
			}
			return self;
		};
		context.PhotoDescription = PhotoDescription;
	}

	return module;
});
define('photolayer/PhotoFrameCtrl.js',['require'],function (require) {
	var PREVIOUS_DISABLED_CSS = 'previous-photo-wrap_disabled';
	var NEXT_DISABLED_CSS = 'next-photo-wrap_disabled';
	var NEW_FACES_MODE = 'newFaces';
	
	var module = function (context) {
		var closeConfirmationDialog = function () {
		};
		var closeEditPhotoDialog = function () {
		};
		var closeComplaintDialog = function () {
		};
		var closeComplaintDescription = function () {
		};
		var vkLikeChangeHandler;
		
		var photolayerStatistics = context.metrics;

		photolayerStatistics.photoHits = 0;

		function PhotoFrameCtrl() {
			var dependencies = arguments[arguments.length - 1] || {};
			this.mambaAPI = dependencies.mambaAPI;
			this.incognitoStatus = dependencies.incognitoStatus;
			this.incognitoRequestStatus = dependencies.incognitoRequestStatus;
			this.incognitoRelationStatus = dependencies.incognitoRelationStatus;
			this.incognitoService = dependencies.incognitoService;
			this.likeCtrl = dependencies.likeCtrl;
			this.incognitoMode = dependencies.incognitoMode;
			this.vkLikeButton = dependencies.vkLikeButton;
			this.fbLikeButton = dependencies.fbLikeButton;
			this.albumsPreviewSectionWrapper = dependencies.albumsPreviewSectionWrapper;
			this.albumsPreviewSection = dependencies.albumsPreviewSection;
			this.smallAlbumsPreviewSection = dependencies.smallAlbumsPreviewSection;
			this.previousPhoto = dependencies.previousPhoto;
			this.nextPhoto = dependencies.nextPhoto;
			this.photoFrame = dependencies.photoFrame;
			this.photoCaptionSection = dependencies.photoCaptionSection;
			this.photo = dependencies.photo; // View;
			this.defaultPhoto = dependencies.defaultPhoto; // Data;
			this.defaultNextPhoto = dependencies.defaultNextPhoto; // Data;
			this.defaultPreviousPhoto = dependencies.defaultPreviousPhoto; // Data;
			this.Photo = dependencies.Photo;
			this.advBannerSection = dependencies.advBannerSection;
			this.EroticWarning = dependencies.EroticWarning;
			this.PhotoNotFound = dependencies.PhotoNotFound;
			this.openedAlbumPreviousPhotoId = dependencies.openedAlbumPreviousPhotoId;
			this.openedAlbumNextPhotoId = dependencies.openedAlbumNextPhotoId;
			this.closeButton = dependencies.closeButton;
			this.photoLayer = dependencies.photoLayer;
			this.photoOverlay = dependencies.photoOverlay;
			this.newFaces = dependencies.newFaces;
			this.photoCaption = dependencies.photoCaption;
			this.photoSlider = dependencies.photoSlider;
			this.photoSettingsSection = dependencies.photoSettingsSection;
			this.ConfirmationDialog = dependencies.ConfirmationDialog;
			this.ComplaintDialog = dependencies.ComplaintDialog;
			this.ComplaintDescription = dependencies.ComplaintDescription;
			this.editPhotoDialog = dependencies.editPhotoDialog;
			this.photoDescription = dependencies.photoDescription;
			this.editableAlbums = dependencies.editableAlbums;
			this.applicationCtrl = dependencies.applicationCtrl;
			this.usePixelization = dependencies.usePixelization;
			this.userProfile = dependencies.userProfile;
			this.albumOwner = dependencies.albumOwner;
			this.isOwnerProfile = dependencies.isOwnerProfile;
			this.photolayerLoader = dependencies.photolayerLoader;
			this.$$currentAlbum = {id: 0};
			this.defaultRatings = dependencies.defaultRatings;
			this.defaultComments = dependencies.defaultComments;
			this.defaultComplaints = dependencies.defaultComplaints;
			this.defaultVote = dependencies.defaultVote;
            this.reCropText = dependencies.reCropText;
			this.stopIncognitoAnimation = false;
			
						var self = this;

						Mb.ComponentManager.getComponentFromHTMLElementAsync($('.js-profile-photos-banner-wrap').children()[0], 15000).then(function (bannerInstance) {
							self.bannerElement = $('.js-profile-photos-banner-wrap').children()[0];
							$('.js-profile-photos-banner-wrap script').remove();
							self.bannerInstance = bannerInstance;
							
							self.bannerInstance.videoFinished.subscribe(function () {
								// console.log('videoFinished subscribe');
								$('.photo-comments-wrap').show();
								$('.js-photo-image-cover').show();
								$(self.bannerElement).remove();
								if (self.photo.$photoImage) {
									self.photo.$photoImage.show();
								}
								if (!self.advBannerSection.shouldHideAdvSection()) {
									self.advBannerSection.toggle(true);
								}
								$('.js-photo-description').show();
								unblockPhotoNavigation(self);
							});
						});
			
			var $alarm = $('.js-alarm-all-pages');
			
			if ($alarm.length) {
				this.photoLayer.css({paddingTop: $alarm.height()});
			}
		}

		PhotoFrameCtrl.MIN_PHOTO_WIDTH = 600;
		PhotoFrameCtrl.MIN_PHOTO_HEIGHT = 450;

		PhotoFrameCtrl.fn = PhotoFrameCtrl.prototype;
		PhotoFrameCtrl.fn.init = function (options) {
			var CROP_FACTOR = 2;
			var self = this;
			this.previewSectionCtrl = options.previewSectionCtrl;
			this.previousPhoto.click(this.previousPhotoClickHandler());
			this.nextPhoto.click(this.nextPhotoClickHandler());

			var complaintDialog = self.ComplaintDialog({
				complaintDialogCssClass: 'complaint-dialog',
				submitCssClass: 'button button-left button-small button-blue',
				cancelCssClass: 'button button-right button-small button-ghost-blue',
				buttonsWrapClass: 'complaint-dialog__buttons',
				buttonWrapClass: 'complaint-dialog__button-wrap'
			});

			self.updateComplaintDialog = function (data) {
				data = data || {};
				complaintDialog.setComplaintReasons(data.causes);
				if (data.type === 'ban') {
					complaintDialog.updateComplaintReasonText(complaintDialog.reasonBanText + ':');
				}
				complaintDialog.$radioButtons.off('change').on('change', function (event) {
					complaintDialog.$radioButtons.parent().removeClass('red-color');
					$(this).parent().addClass('red-color');
					complaintDialog.$submitButton.prop('disabled', false);
				});
				return self;
			};

			self.photoLayer.mousemove(function (event) {
				var stopPropagatedClasses = [
					'js-photo-image-cover',
					'photo-caption-section',
					'photo-slider-wrapper',
					'albums-preview-section-wrapper',
					'small-albums-preview-section',
					'js-previous-photo-wrap',
					'js-next-photo-wrap',
					'new_faces__item_reload'
				];
				var $target = $(event.target);

				for (var i = 0, l = stopPropagatedClasses.length; i < l; i++) {
					if ($target.hasClass(stopPropagatedClasses[i]) || $target.parents('.' + stopPropagatedClasses[i]).length) {
						self.closeButton.removeClass('close-button-hovered');
						self.photoLayer.css('cursor', 'default');
						return;
					}
				}
				self.closeButton.addClass('close-button-hovered');
				self.photoLayer.css('cursor', 'pointer');
			});
			
			self.albumsPreviewSection.getPhotoInAlbum(self.defaultPhoto.id, self.defaultPhoto.album).viewed = true;			

			self.changePhotoFrameImage(self.defaultPhoto)
				.setPreviousPhotoFrameImage(self.defaultPreviousPhoto)
				.setNextPhotoFrameImage(self.defaultNextPhoto)
				.photoSettingsSection.toggle(!self.defaultPhoto.isRejected);

			// We use this property to track browser back and forth history;
			// If a user clicks back and forth and current photo object does not change
			// then we destroy photolayer. This photolayer version does not support
			// standard history navigation. Going back and forth between photos just
			// changes a hash value but vice versa does not work. Changing a hash value
			// does not choose another photo by hash value.
            self._previousHash = '';
			self.hashChangeHandler = function (event) {				
                if (self._previousHash === location.hash) {					
                    self.destroy();
                }
			};
			$(window).on('hashchange', self.hashChangeHandler);

			vkLikeChangeHandler = vkLikeChange(self.vkLikeButton);
			vkLikeChangeHandler(self.defaultPhoto); // init vkLikeChange function;

			self.closeButton.click(function (event) {
				event.stopPropagation();
				self.destroy();
			});
			self.newFaces.$.on('close', function (event) {				
				self.destroy();
			}).on('restart', function (event) {
				self.goToStartPhoto();
			});
			
			self.clickHandler = function (event) {
				if (event.target !== $('#Wrapper').get(0) && !event.target.classList.contains('js-allowed-to-close')) {
					return;
				}
				if (self.likeCtrl.preventPhotoFrameDestroy) {
					return;
				}
				// TODO add prevent banner close
				if (event.which !== 2 && event.which !== 3) {
					self.destroy();
				}
			};
			$(document).on('click', self.clickHandler);			

			(function () {
				var sliderShowTimer = null;
				var ARROW_RIGHT = 39;
				var ARROW_LEFT = 37;
				var SPACE = 32;
				var ESC = 27;

				var sliderHandler = function (keyCode) {					
					if (self.previousPhoto.hasClass(PREVIOUS_DISABLED_CSS) || self.nextPhoto.hasClass(NEXT_DISABLED_CSS)) {
						// console.log('keydownHandler prevented');
						clearInterval(sliderShowTimer);
						sliderShowTimer = null;
						return;
					}
					if (ARROW_LEFT === keyCode) {
						if (mambo.langIsRTL) {
							slideToNext();
						} else {
							slideToPrevious();
						}
					} else if (ARROW_RIGHT === keyCode) {
						if (mambo.langIsRTL) {
							slideToPrevious();
						} else {
							slideToNext();
						}
					} else if (SPACE === keyCode) {
						slideToNext();
					} else if (ESC === keyCode) {
                        clearInterval(sliderShowTimer);
                        sliderShowTimer = null;
                        self.destroy();
					}
					
					function slideToNext() {
						if (self.mode == 'photos') {
							self.goToNext();														
						} else if (self.mode == NEW_FACES_MODE) {
							self.goToStartPhoto();
						}
					}
					
					function slideToPrevious() {
						if (self.mode == 'photos') {
							self.goToPrevious();
						} else if (self.mode == NEW_FACES_MODE) {
							self.goToFinalPhoto();
						}
					}
				};
				
				self.keyupHandler = function (event) {
					clearInterval(sliderShowTimer);
					sliderShowTimer = null;
				};
				
				self.keydownHandler =  function (event) {
					// console.log('keydownHandler started...');
					event.stopPropagation();

					if (sliderShowTimer) {
						return;
					}                    

					var keyCode = event.keyCode;

					sliderShowTimer = setInterval(function () {
						sliderHandler(keyCode);
					}, 250);
										
					sliderHandler(keyCode);

					// For FF; Block document body scrolling down;
					if (keyCode === SPACE) {
						event.preventDefault();
					}
				};

				$(document).on('keyup', self.keyupHandler);
				$(document).on('keydown', self.keydownHandler);
			})();

			self.albumsPreviewSectionWrapper.click(function (event) {
				event.stopPropagation();
			});

			var confirmationDialog = self.ConfirmationDialog(
				'confirmation-dialog',
				context.t.remove,
				'button button-left button-small button-blue',
				context.t.cancel,
				'button button-right button-small button-ghost-blue'
			);
			confirmationDialog.isOpened = false;
			confirmationDialog.$.click(function (event) {
				event.stopPropagation();
			});

			self.photoCaption.click(function () {
				closeConfirmationDialog();
				closeEditPhotoDialog();
				closeComplaintDialog();
				closeComplaintDescription();
			});

			confirmationDialog.$cancelButton.click(function (event) {
				confirmationDialog.$.detach();
				confirmationDialog.isOpened = false;
				self.photoSettingsSection.$deleteIcon.removeClass('clicked hovered');
			});
			self.closeConfirmationDialog = closeConfirmationDialog = function () {
				confirmationDialog.$cancelButton.trigger('click');
			};
			confirmationDialog.$submitButton.click(function () {
				var adjacentPhotos = self.albumsPreviewSection.getAdjacentPhotos(self.photo);
				var deletedPhoto = self.albumsPreviewSection.deletePhoto(self.photo);

				self.mambaAPI.genericApiDelete('deletePhoto', {
					anketaId: mambo.vUserId,
					photoId: deletedPhoto.id,
					langId: mambo.aUserLang,
					s_post: mambo.utils.Cookie('s_post')
				}, function (error, res) {
					if (error) {
						// error;
						return;
					}
					// Remove albumSection if it has got no albumImages;
					if (!deletedPhoto.album.view.getAlbumImageSectionsCount()) {
						self.smallAlbumsPreviewSection.deletePreviewSectionByIndex(deletedPhoto.album.view.$.index());
						deletedPhoto.album.view.$.remove();
					}

					if (adjacentPhotos.next.view) {
						var nextPhoto = adjacentPhotos.next;
						self.changePhotoFrameImage(nextPhoto)
							.editPhotoDialog.toggleRatingsMenu(nextPhoto.album['default'] && !nextPhoto.erotic)
							.toggleAlbum(nextPhoto.album['default'], nextPhoto.defaultAlbum);

						self.photoSettingsSection.toggle(!nextPhoto.isRejected);

						vkLikeChangeHandler(adjacentPhotos.next);
						self.previewSectionCtrl.setPreviewImage(adjacentPhotos.next.view.$);

						adjacentPhotos = self.albumsPreviewSection.getAdjacentPhotos(adjacentPhotos.next);
						self.setPreviousPhotoFrameImage(adjacentPhotos.previous)
							.setNextPhotoFrameImage(adjacentPhotos.next);
						self.editPhotoDialog.toggleMainPhotoSection(
							nextPhoto.album['default'] && !nextPhoto.erotic && !nextPhoto.isRejected && nextPhoto.status !== 'MODERATING'
						).updateMainPhotoSection(nextPhoto);
					} else if (adjacentPhotos.previous.view) {
						var previousPhoto = adjacentPhotos.previous;
						self.changePhotoFrameImage(previousPhoto)
							.editPhotoDialog.toggleRatingsMenu(previousPhoto.album['default'] && !previousPhoto.erotic)
							.toggleAlbum(previousPhoto.album['default'], previousPhoto.defaultAlbum);

						self.photoSettingsSection.toggle(!adjacentPhotos.previous.isRejected);

						vkLikeChangeHandler(adjacentPhotos.previous);
						self.previewSectionCtrl.setPreviewImage(adjacentPhotos.previous.view.$);

						adjacentPhotos = self.albumsPreviewSection.getAdjacentPhotos(adjacentPhotos.previous);
						self.setPreviousPhotoFrameImage(adjacentPhotos.previous)
							.setNextPhotoFrameImage(adjacentPhotos.next);
						self.editPhotoDialog.toggleMainPhotoSection(
							previousPhoto.album['default'] && !previousPhoto.erotic && !previousPhoto.isRejected && previousPhoto.status !== 'MODERATING'
						).updateMainPhotoSection(previousPhoto);
					}
					closeConfirmationDialog();
					closeComplaintDialog();
					closeComplaintDescription();
					self.albumsPreviewSection.loadVisibleAlbumImages();

					$('#Wrapper').triggerHandler('setnicescrollbar', [0]);

					// TODO: refactor! hardcore!
					// We need to delete this photo from album photos list in a user dashboard;
					$('#photo' + deletedPhoto.id).remove();

					if (self.albumsPreviewSection.photos.length <= 1) {
						self.photoSlider.toggle(false);
						self.albumsPreviewSection.toggle(false);
						self.smallAlbumsPreviewSection.toggle(false);
						self.newFaces.liftUp();
					} else {
						self.newFaces.center();
					}
					if (!self.albumsPreviewSection.photos.length) {
						window.location.href = '/' + self.userProfile.login;
					}
				});
			});

			var complaintDescription = self.ComplaintDescription({
				complaintDescriptionCssClass: 'complaint-description',
				cancelCssClass: 'button button-small button-ghost-blue',
				cancelText: context.t.OK
			});
			complaintDescription.$.click(function (event) {
				event.stopPropagation();
			});
			self.closeComplaintDescription = closeComplaintDescription = function () {
				complaintDescription.$.detach();
				complaintDescription.isOpened = false;
				self.photoSettingsSection.$complaintIcon.removeClass('clicked');
			};

			complaintDialog.$submitButton.prop('disabled', true);
			complaintDialog.$.click(function (event) {
				event.stopPropagation();
			});
			complaintDialog.$cancelButton.click(function (event) {
				complaintDialog.$.detach();
				complaintDialog.isOpened = false;
				self.photoSettingsSection.$complaintIcon.removeClass('clicked');
				complaintDialog.resetValues();
			});
			self.closeComplaintDialog = closeComplaintDialog = function () {
				complaintDialog.$cancelButton.trigger('click');
			};
			complaintDialog.$.mousedown(function (event) {
				event.preventDefault();
			});
			complaintDialog.$submitButton.click(function (event) {
				var $this = $(this);
				$this.prop('disabled', true);
				self.mambaAPI.genericApiPost('postComplaint', {
					applicationId: mambo.vUserId,
					dateType: 'timestamp',
					langId: mambo.aUserLang,
					s_post: mambo.utils.Cookie('s_post'),
					complaint: {
						cause: complaintDialog.$radioButtons.filter(':checked').val() * 1,
						entityId: self.photo.id,
						kind: "photo"
					}
				}, function (res) {
					$this.prop('disabled', false);
					closeComplaintDialog();
					complaintDialog.resetValues();
				});
			});

			self.photoSettingsSection.$complaintIcon.click(function (event) {
				if (complaintDescription.isOpened) {
					closeComplaintDescription();
				} else if (complaintDialog.isOpened) {
					closeComplaintDialog();
				} else {
					self.mambaAPI.genericApiGet('checkComplaint', {
						applicationId: mambo.vUserId,
						photoId: self.photo.id,
						langId: mambo.aUserLang
					}, function (response) {
						var complaint = response.complaint;
						if (complaint) {
							var complaintDate = new Date(complaint.created * 1000);
							complaintDescription.update({
								publishedDescriptionDate: complaintDescription.formatPublishedDate(complaintDate),
								publishedDescriptionTime: complaintDescription.formatPublishedTime(complaintDate),
								reasonDescriptionText: complaint.text
							});

							if (complaintDescription.isOpened) {
								closeComplaintDescription();
							} else {
								self.photoSettingsSection.$complaintIcon.append(complaintDescription.$).addClass('clicked');
								complaintDescription.isOpened = true;
							}
						} else if (response.errorCode === 103) {
							complaintDescription.update({
								publishedBanText: complaintDescription.publishedBanText
							});
							if (complaintDescription.isOpened) {
								closeComplaintDescription();
							} else {
								self.photoSettingsSection.$complaintIcon.append(complaintDescription.$).addClass('clicked');
								complaintDescription.isOpened = true;
							}
						} else {
							if (complaintDialog.isOpened) {
								closeComplaintDialog();
							} else {
								self.photoSettingsSection.$complaintIcon.append(complaintDialog.$).addClass('clicked');
								complaintDialog.isOpened = true;
							}
						}
					});
				}
				event.stopPropagation();
			}).mousedown(function (event) {
				event.preventDefault();
			});
			self.photoSettingsSection.$deleteIcon.click(function (event) {
				if (confirmationDialog.isOpened) {
					closeConfirmationDialog();
				} else {
					$(this).addClass('clicked');
					closeEditPhotoDialog();
					$(this).append(confirmationDialog.$);
					confirmationDialog.isOpened = true;
				}
				event.stopPropagation();
			});

			/* Ratings */
			(function () {
				var userRatings = [];
				var editableAlbums = [];

				var userRatings = $.map(self.defaultRatings || [], function (element) {
					return {value: element.key, text: element.name, selected: element.selected ? true : false};
				});

				for (var i = 0, l = self.editableAlbums.length; i < l; i++) {
					editableAlbums.push({
						value: self.editableAlbums[i].key,
						text: self.editableAlbums[i].name
					});
				}

				var editPhotoDialog = self.editPhotoDialog; // Symlink;
				// As though we have loaded editableAlbums and userRatings we can initialize DOM;
				editPhotoDialog.init('#photolayer-EditPhotoDialogTmpl', editableAlbums, userRatings); // Initialize DOM;

				editPhotoDialog.updateRatings(userRatings);

				editPhotoDialog.toggleRatingsMenu(self.defaultPhoto.album['default'] && !self.defaultPhoto.erotic && userRatings.length)
					.toggleAlbum(self.defaultPhoto.album['default'], self.defaultPhoto.defaultAlbum);
				self.photoSettingsSection.$settingsIcon.append(editPhotoDialog.$.hide());

				// Choose selected photo album;
				editPhotoDialog.selectAlbum(self.photo.album.id).selectRating(self.defaultPhoto.ratingId);

				editPhotoDialog.$submitButton.prop('disabled', true); // disabled by default;

				editPhotoDialog.isOpened = false;
				editPhotoDialog.$.click(function (event) {
					event.stopPropagation();
				});

				editPhotoDialog.$photoDescription.on('input propertychange', function () {
					//editPhotoDialog.updateSymbolsLimit();
					editPhotoDialog.toggleSubmitButtonState();
				});

				editPhotoDialog.$photoDescription.keyup(function (event) {
					event.stopPropagation();
					var DELETE = 46;
					var BACKSPACE = 8;
					if (DELETE === event.keyCode || BACKSPACE === event.keyCode) {
						//editPhotoDialog.updateSymbolsLimit();
						editPhotoDialog.toggleSubmitButtonState();
					}
				}).keydown(function (event) {
					var symbolsCount = $(this).val().trim().length;
					var MAX_LIMIT = 50;
					var ARROW_LEFT = 37;
					var ARROW_UP = 38;
					var ARROW_RIGHT = 39;
					var ARROW_DOWN = 40;
					var BACKSPACE = 8;
					var DELETE = 46;
					if (symbolsCount > MAX_LIMIT) {
						if (ARROW_LEFT !== event.keyCode &&
							ARROW_UP !== event.keyCode &&
							ARROW_RIGHT !== event.keyCode &&
							ARROW_DOWN !== event.keyCode &&
							BACKSPACE !== event.keyCode &&
							DELETE !== event.keyCode)
							event.preventDefault();
					}
					event.stopPropagation();
				});
				editPhotoDialog.$.mousedown(function (event) {
					event.stopPropagation();
				});

				if (editPhotoDialog.$photoDescription.get(0)) {
					/* jQuery input, propertychange work some kind of weird */

					editPhotoDialog.$photoDescription.get(0).oninput = function () {
						//editPhotoDialog.updateSymbolsLimit().toggleSubmitButtonState();
					};
					editPhotoDialog.$photoDescription.get(0).onpropertychange = function () {
						//editPhotoDialog.updateSymbolsLimit().toggleSubmitButtonState();
					};
				}

				editPhotoDialog.$albumsMenu.change(function () {
					// editPhotoDialog.toggleSubmitButtonState();
				});
				editPhotoDialog.$ratingsMenu.change(function () {
					editPhotoDialog.toggleSubmitButtonState();
				});

				editPhotoDialog.$cancelButton.click(function (event) {
					editPhotoDialog.$.hide();
					editPhotoDialog.isOpened = false;
					editPhotoDialog.$symbolsLimit.text(50);
					self.photoSettingsSection.$settingsIcon.removeClass('clicked');
				});
				self.closeEditPhotoDialog = closeEditPhotoDialog = function () {
					editPhotoDialog.$cancelButton.trigger('click');
				};

				editPhotoDialog.$submitButton.click(function () {
					var sysRequestsContainer = [];
					var photoId = self.photo.id;
                    if (editPhotoDialog.$photoDescription.val()){
                        var photoDescription = editPhotoDialog.$photoDescription.val().trim();
					}

					var albumId = editPhotoDialog.$albumsMenu.find('option:selected').val() * 1;
					var ratingId = editPhotoDialog.$ratingsMenu ? editPhotoDialog.$ratingsMenu.find('option:selected').val() * 1 : 0;

					self.mambaAPI.genericApiPut('putPhoto', {
						langId: mambo.aUserLang,
						photoId: photoId,
						s_post: mambo.utils.Cookie('s_post'),
						edit: {
							name: photoDescription,
							album_id: albumId,
							rating_id: ratingId
						}
					}, function (error, data) {
						if (error || data.formBuilder) {
							// error;
							return;
						}
						self.photoDescription.updateText(photoDescription);
						$.each(self.albumsPreviewSection.photos, function (index, photo) {
							if (photo.id === self.photo.id) {
								photo.description = photoDescription;
							}
						});

						self.albumsPreviewSection.getPhotoInAlbum(self.photo.id, self.$$currentAlbum).ratingId = ratingId;

						editPhotoDialog.$photoDescription.data('photoDescription', photoDescription);
						self.photo.description = photoDescription;
						editPhotoDialog.selectAlbum(albumId);
						editPhotoDialog.selectRating(ratingId);
						closeEditPhotoDialog();

						// photo moved into another album;
						if (albumId !== self.photo.album.id) {
							var oldAlbum = self.albumsPreviewSection.getAlbumById(self.photo.album.id);
							var newAlbum = self.albumsPreviewSection.getAlbumById(albumId);

							self.albumsPreviewSection.putPhotoInAlbum(self.photo, newAlbum);

							var adjacentPhotos = self.albumsPreviewSection.getAdjacentPhotos(self.photo);
							var previousPhoto = adjacentPhotos.previous;
							var nextPhoto = adjacentPhotos.next;
							self.setPreviousPhotoFrameImage(previousPhoto).setNextPhotoFrameImage(nextPhoto);

							var albumSectionIndex = self.photo.album.view.$.index();
							self.smallAlbumsPreviewSection.deselectAllPreviewSections().selectPreviewSectionByIndex(albumSectionIndex);

							if (!oldAlbum.view.getAlbumImageSectionsCount()) {
								self.smallAlbumsPreviewSection.togglePreviewSectionByIndex(oldAlbum.view.$.index(), false);
								oldAlbum.view.$.hide();
							}
							if (newAlbum.view.getAlbumImageSectionsCount()) {
								self.smallAlbumsPreviewSection.togglePreviewSectionByIndex(newAlbum.view.$.index(), true);
								newAlbum.view.$.show();
							}
							editPhotoDialog.toggleRatingsMenu(newAlbum['default'] && !self.photo.erotic);
							editPhotoDialog.toggleMainPhotoSection(
								newAlbum['default'] && !self.photo.erotic && !self.photo.isRejected && self.photo.status !== 'MODERATING'
							);
						}
					});
				});

				if (!self.isOwnerProfile) {
					// remove mainPhotoSection for strange users;
					editPhotoDialog.$mainPhotoSection.detach();
				}
				editPhotoDialog.toggleMainPhotoSection(
					self.defaultPhoto.album['default'] && !self.defaultPhoto.erotic && !self.defaultPhoto.isRejected && self.defaultPhoto.status !== 'MODERATING'
				)
					.updateMainPhotoSection(self.defaultPhoto)
                    .blockedReCropSection(self.defaultPhoto)
                    .showReCropSection(self.defaultPhoto.album.id == self.defaultPhoto.defaultAlbum.id)
					.$mainPhotoSection
					.click(function (event) {
						event.stopPropagation();
						if (!editPhotoDialog.mainPhotoSectionActive) {
							return false;
						}
						self.mambaAPI.genericApiPost('mainPhoto', {
							dateType: 'timestamp',
							photoId: self.photo.id,
							langId: mambo.aUserLang,
							s_post: mambo.utils.Cookie('s_post')
						}, function (res) {
							if (res.errors) {
								// errors;
								return;
							}
							var allPhotos = self.albumsPreviewSection.photos || [];
							var currentPhoto;
							for (var l = allPhotos.length, i = l - 1; i >= 0; i--) {
								if (allPhotos[i].id !== self.photo.id) {
									allPhotos[i].current = false;
								} else {
									allPhotos[i].current = true;
									currentPhoto = allPhotos[i];
								}
							}
							editPhotoDialog.updateMainPhotoSection(currentPhoto);
						});
					});
                editPhotoDialog
                    .$reCropPhotoText.click(function(event) {
                        event.stopPropagation();
                        if (!editPhotoDialog.reCropPhotoActive) {
                            return false;
                        }

                        self.mambaAPI.genericApiPost('cropPhoto', {
                            dateType: 'timestamp',
                            photoId: self.photo.id,
                            langId: mambo.aUserLang,
                            s_post: mambo.utils.Cookie('s_post')
                        }, function( result ) {
                            if (result.hasOwnProperty('tips')) {

                                mambo.ui.Alert(
                                    result.tips.message, {
                                        time:9000,
                                        z_index:2147483640
                                    }
                                );

                                editPhotoDialog.blockedReCropSection( true )
                                               .blockedReCropSectionAllPhoto();

                            } else if (result.hasOwnProperty('photo')) {

                                mambo.ui.Alert(
                                    context.t.reCropText, {
                                        z_index:2147483640
                                    }
                                );

                                $('#photo' + self.photo.id).addClass('premod');
                                editPhotoDialog.blockedReCropSection( true )
                                               .blockedReCropSectionAddBlockedPhotoById(self.photo.id);
                            }
                        });
                    });

				self.photoSettingsSection.$settingsIcon.click(function (event) {
					if (editPhotoDialog.isOpened) {
						closeEditPhotoDialog();
					} else {
						$(this).addClass('clicked');
						closeConfirmationDialog();
						editPhotoDialog.$.show();
						editPhotoDialog.isOpened = true;


						editPhotoDialog.selectAlbum(self.photo.album.id)
							.selectRating(self.albumsPreviewSection.getPhotoInAlbum(self.photo.id, self.$$currentAlbum).ratingId);

						editPhotoDialog.$photoDescription
							.val(self.photo.description)
							.data('photoDescription', self.photo.description.trim());
						editPhotoDialog.photoDescriptionOldValue = self.photo.description;
						//editPhotoDialog.updateSymbolsLimit().toggleSubmitButtonState();
                        editPhotoDialog.toggleSubmitButtonState();
					}
					event.stopPropagation();
				}).mousedown(function (event) {
					event.preventDefault();
				});

			})();
		};		

		PhotoFrameCtrl.fn.nextPhotoClickHandler = function () {
			var self = this;
			return function (event) {
				if (self.nextPhoto.hasClass(NEXT_DISABLED_CSS)) {
					return;
				}
				self.goToNext();
			}
		};

        PhotoFrameCtrl.fn.previousPhotoClickHandler = function () {
            var self = this;
            return function (event) {
								if (self.previousPhoto.hasClass(PREVIOUS_DISABLED_CSS)) {
									return;
								}
                self.goToPrevious();
            }
        };

		PhotoFrameCtrl.fn.goToPrevious = function () {
			var currentPhoto = this.albumsPreviewSection.getPhotoInAlbum(this.photo.id, this.photo.album);
			if (currentPhoto === this.albumsPreviewSection.getFirstPhoto()) {
				this.changePhoto(this.albumsPreviewSection.getLastPhoto());				
			} else {
				var adjacentPhotos = this.albumsPreviewSection.getAdjacentPhotos(currentPhoto);					
				this.changePhoto(adjacentPhotos.previous);
			}
			
			var numPhotosViewedInRow = context.layerData.numPhotosViewedInRow;
			numPhotosViewedInRow = numPhotosViewedInRow > 1 ? numPhotosViewedInRow - 1 : this.albumsPreviewSection.photos.length;
			context.layerData.numPhotosViewedInRow = numPhotosViewedInRow;
		};

        PhotoFrameCtrl.fn.goToNext = function () {
            var numViewedPhotos = this.albumsPreviewSection.photos.filter(function (photo) {
                return photo.viewed;
            }).length;

            // если авторизованный пользователь просматривает свои фотографии
            // или пользователем еще не был пройден полный круг
            if (mambo.aUserId == mambo.vUserId ||
                context.layerData.numPhotosViewedInRow != this.albumsPreviewSection.photos.length ||
                numViewedPhotos != this.albumsPreviewSection.photos.length) {

                var currentPhoto = this.albumsPreviewSection.getPhotoInAlbum(this.photo.id, this.photo.album);
                if (currentPhoto === this.albumsPreviewSection.getLastPhoto()) {
                    this.changePhoto(this.albumsPreviewSection.getFirstPhoto(), true);
                } else {
                    var adjacentPhotos = this.albumsPreviewSection.getAdjacentPhotos(currentPhoto);
                    this.changePhoto(adjacentPhotos.next, true);
                }

                var numPhotosViewedInRow = context.layerData.numPhotosViewedInRow;
                numPhotosViewedInRow = numPhotosViewedInRow < this.albumsPreviewSection.photos.length ? numPhotosViewedInRow + 1 : 1;
                context.layerData.numPhotosViewedInRow = numPhotosViewedInRow;

            } else if (context.layerData.numPhotosViewedInRow == this.albumsPreviewSection.photos.length &&
                numViewedPhotos == this.albumsPreviewSection.photos.length) {

                this.changeMode(NEW_FACES_MODE);
            }
        };

        PhotoFrameCtrl.fn.goToStartPhoto = function () {
            var startPhoto = context.layerData.startPhoto;
            var photo = this.albumsPreviewSection.getPhotoInAlbum(startPhoto.id, startPhoto.album);

            this.changePhoto(photo);
            context.layerData.numPhotosViewedInRow = 1;
        };

        PhotoFrameCtrl.fn.goToFinalPhoto = function () {
            var startPhoto = context.layerData.startPhoto;
            var adjacentPhotos = this.albumsPreviewSection.getAdjacentPhotos(startPhoto);
            var finalPhoto = adjacentPhotos.previous.view && adjacentPhotos.previous || this.albumsPreviewSection.getLastPhoto();
            var photo = this.albumsPreviewSection.getPhotoInAlbum(finalPhoto.id, finalPhoto.album);

            this.changePhoto(photo);
        };

        PhotoFrameCtrl.fn.changePhoto = function (photo, markAsViewed) {
            if (this.photo.id === photo.id && this.photo.album.id === photo.album.id) {
                return;
            }

            var albumPhoto = this.albumsPreviewSection.getPhotoInAlbum(photo.id, {id: photo.album.id});
            if (markAsViewed) {
                albumPhoto.viewed = true;
            }
            this.update(photo);
            this.previewSectionCtrl.setPreviewImage(albumPhoto.view.$);

            photolayerStatistics.addData(mambo.vUserId, 0, 2, '#' + photo.id);
        };
		
		PhotoFrameCtrl.fn.update = function (photo) {
            var adjacentPhotos = this.albumsPreviewSection.getAdjacentPhotos(photo);
            var previousPhoto = adjacentPhotos.previous;
            var nextPhoto = adjacentPhotos.next;

            closeEditPhotoDialog();
            closeComplaintDialog();
            closeConfirmationDialog();
            closeComplaintDescription();
            //this.commentsCtrl.closePhotoCommentsStateTooltip();
            //this.commentsCtrl.closeCommentSectionTooltip();

            this.changePhotoFrameImage(photo)
                .setPreviousPhotoFrameImage(previousPhoto)
                .setNextPhotoFrameImage(nextPhoto)
                .editPhotoDialog
                .toggleRatingsMenu(photo.album['default'] && !photo.erotic)
                .toggleAlbum(photo.album['default'], photo.defaultAlbum);
            this.photoSettingsSection.toggle(!photo.isRejected);
            vkLikeChangeHandler(photo);
            this.editPhotoDialog
                .toggleMainPhotoSection(
                    photo.album['default'] && !photo.erotic && !photo.isRejected && photo.status !== 'MODERATING'
                )
                .updateMainPhotoSection(photo)
                .blockedReCropSection(photo)
                .showReCropSection(photo.album.id == photo.defaultAlbum.id);

            $('#Wrapper').triggerHandler('setnicescrollbar', [0]);
        };

		PhotoFrameCtrl.fn.changePhotoFrameImage = function (selectedPhoto) {
			var self = this;
			self.photo.$photoContainer.hide();
			self.$$currentAlbum = {id: selectedPhoto.album.id};
			var albumsPreviewSectionPhoto = self.albumsPreviewSection.getPhotoInAlbum(selectedPhoto.id, self.$$currentAlbum);			
			if (albumsPreviewSectionPhoto.removed && selectedPhoto.removed) {
				// Photo not found;
				displayPhotoNotFoundView(self, selectedPhoto);
			} else {
				if (selectedPhoto.erotic &&
					self.applicationCtrl.SHOW_EROTIC_WARNING && !self.applicationCtrl.IS_OWNER_PROFILE &&
					(!self.applicationCtrl.incognitoStatus.isOn || self.applicationCtrl.incognitoStatus.isOn && self.applicationCtrl.incognitoRelationStatus.canView)) {
					displayEroticPhotoView(self, selectedPhoto);
				} else {					
					displayPhotoView(self, selectedPhoto);
				}

                self._previousHash = location.hash;				
				window.location.hash = '#' + 'photo_id=' + selectedPhoto.id + '&album_id=' + selectedPhoto.album.id;
			}
			var albumSectionIndex = albumsPreviewSectionPhoto.view.$.parents('.album-section').index();
			self.smallAlbumsPreviewSection.deselectAllPreviewSections().selectPreviewSectionByIndex(albumSectionIndex);
			return self;
		};
		PhotoFrameCtrl.fn.setNextPhotoFrameImage = function (newNextPhoto) {
			var self = this;
			if (!newNextPhoto.id) {
				self.nextPhoto.hide();
			} else {
				self.nextPhoto.show().triggerHandler('changeImage', [newNextPhoto.removed ? '' : newNextPhoto.src, self.usePixelization && newNextPhoto.erotic]);
			}
			return self;
		};
		PhotoFrameCtrl.fn.setPreviousPhotoFrameImage = function (newPreviousPhoto) {
			var self = this;
			if (!newPreviousPhoto.id) {
				self.previousPhoto.hide();
			} else {
				self.previousPhoto.show().triggerHandler('changeImage', [newPreviousPhoto.removed ? '' : newPreviousPhoto.src, self.usePixelization && newPreviousPhoto.erotic]);
			}
			return self;
		};
		
		PhotoFrameCtrl.fn.changeMode = function (value) {
			if (value != this.mode) {
				if (value == 'photos') {
					this.newFaces.hide();
					$('#SuperWrapper').show();
				} else if (value == NEW_FACES_MODE) {
					$('#SuperWrapper').hide();
					this.setPreviousPhotoFrameImage({}).setNextPhotoFrameImage({});
					this.previewSectionCtrl.deselectAllPreviews();					
					this.newFaces.show();
					this.photo.id = null;										
				} else console.log('incorrect mode value');

				this.mode = value;
			}
		};

		PhotoFrameCtrl.fn.showLayer = function () {
			this.photoLayer.show();
			this.photoOverlay.show();			
		};

		PhotoFrameCtrl.fn.hideLayer = function () {
			this.photoLayer.hide();
			this.photoOverlay.hide();
		};

		PhotoFrameCtrl.fn.destroy = function () {
            var $commentsComponent = $('.photo-comments-wrap').find('.js-mb-component');
            if ($commentsComponent.length) {
                $('.comments-component-wrap').append($commentsComponent);
            }
			
						if (this.bannerElement) {
								$('.js-profile-photos-banner-wrap').append(this.bannerElement);
							  $(this.bannerElement).data(Mb.Constants.COMPONENT_VIEWMODEL_BINDING_PARAMNAME, this.bannerInstance);
								this.bannerInstance.resetBanner();
						}
			
			var closed = window.location.hash.match(/closed/);
            if(!closed) {
                var hash = angular.copy(window.location.hash);
				// временная замена на cookie согласно задаче 56801
                //localStorage.setItem('savePhotoLayerUrl', hash);
				$.cookie('savePhotoLayerUrl', hash);
            }
			
			this.newFaces.$.off();
			this.photoLayer.remove();
			this.photoOverlay.remove();
			
			$('body').css('overflow', 'visible').css('overflow', 'initial')
				.removeClass('photolayer-opened');
            window.location.hash = '#/closed';
			this._previousHash = '';
			this.previewSectionCtrl.unviewAll();
			
            $(window).off('hashchange', this.hashChangeHandler);
			$(document).off('click', this.clickHandler);
			$(document).off('keyup', this.keyupHandler)
			$(document).off('keydown', this.keydownHandler);
			
			if (this.previousPhoto) this.previousPhoto.off('click');
			if (this.nextPhoto) this.nextPhoto.off('click');
			
			this.photolayerLoader.loaded(false);
            this.likeCtrl.dispose();
            var photoCommentsComponent = Mb.ComponentManager.getComponentFromHTMLElement($commentsComponent[0]);
            photoCommentsComponent && photoCommentsComponent.destroyFrame();
		};
		
		function vkLikeChange(vkLikeButton) {
			var $vkLike = $('.js-vk-like', vkLikeButton.$);
			var title = $vkLike.data('vk_title') || '';
			var description = $vkLike.data('vk_description') || '';
			var text = $vkLike.data('vk_text') || '';
			var res = function () {
			};

			if (window.VK && $vkLike.length) {
				res = function (photo) {
					var url = location.protocol + '//' + location.host + location.pathname + (location.search ? location.search : '?');
					if (url.indexOf('&from=vk_share') == -1) {
						url = url + (url.indexOf('&', url.length - 1) == -1 ? '&' : '') + 'from=vk_share';
					}
					$vkLike.html(VK.Share.button(
						{
							url: url + '#photo_id=' + photo.id,
							title: title,
							description: description,
							image: photo.src,
							noparse: true
						},
						{
							type: "round_nocount",
							text: text
						}
					));
				};
			}
			return res;
		}

		function requestRatingsAndPhotoComments(photoFrameCtrl, photo, callback) {
			var sysRequestsContainer = [],
				commentsIndex,
				ratingsIndex,
				votedIndex,
				complaintsIndex;

			if (mambo.aAnketaId === mambo.vUserId) {
				/* Photo ratings request */
				ratingsIndex = sysRequestsContainer.push({
					uri: '/photos/' + photo.id + '/edit/',
					method: 'GET',
					params: {
						langId: mambo.aUserLang
					}
				}) - 1;
			}

			/* Photo comments request */
			commentsIndex = sysRequestsContainer.push({
				uri: '/users/' + mambo.vUserId + '/photos/' + photo.id + '/comments/',
				method: 'GET',
				params: {
					limit: 10000,
					langId: mambo.aUserLang
				}
			}) - 1;

			/* Photo complaints request */
			complaintsIndex = sysRequestsContainer.push({
				uri: '/complaint/causes/' + mambo.vUserId + '/photo/' + photo.id + '/',
				method: 'GET',
				params: {
					lang_id: mambo.aUserLang,
					reqType: 'json'
				}
			}) - 1;

			/* Photo voted info */
			votedIndex = sysRequestsContainer.push({
				uri: '/photorating/check_voted/',
				method: 'GET',
				params: {
					photoId: photo.id
				}
			}) - 1;

			photoFrameCtrl.mambaAPI.containerRequest({
				applicationId: mambo.vUserId,
				langId: mambo.aUserLang,
				photoId: photo.id
			}, sysRequestsContainer, function (error, data) {
				var ratings = [],
					editableAlbums = [];

				if (error) {
					return;
				}

				var sysResponsesContainer = data.sysResponsesContainer || [];

				///TODO: hardcore!!! Yeah , baby, yeah!!! Weehaa!
				try {
					var fields = sysResponsesContainer[ratingsIndex].formBuilder.blocks[0].fields || [];
					for (var i = 0, l = fields.length; i < l; i++) {
						switch (fields[i].field) {
							case 'rating_id':
								ratings = fields[i].variants || ratings;
								break;

							case 'album_id':
								editableAlbums = fields[i].variants || editableAlbums;
								break;

							default:
								break;
						}
					}
				} catch (e) {}

				/* Handle user photo comments; */
				var commentsData = sysResponsesContainer[commentsIndex] || {};

				/* Complaints response object */
				var complaintsObject = sysResponsesContainer[complaintsIndex] || {};

				/* Complaints response object */
				var votedObject = sysResponsesContainer[votedIndex] || {};

				(typeof callback === 'function' ? callback : function () {
				})(ratings, commentsData, complaintsObject, editableAlbums, votedObject);
			});
		}

		function updatePhotoRatings(photoFrameCtrl, photo, ratings) {
			var userRatings = [];
			userRatings = userRatings.concat($.map(ratings || [], function (element) {
					return {value: element.key, text: element.name, selected: element.selected ? true : false};
				})
			);
			photoFrameCtrl.editPhotoDialog
				.updateRatings(userRatings)
				.toggleRatingsMenu(photo.album['default'] && userRatings.length && !photo.erotic);
		}

		function updateEditableAlbums(photoFrameCtrl, albums) {
			var editableAlbums = [];

			editableAlbums = editableAlbums.concat($.map(albums || [], function (element) {
					return {value: element.key, text: element.name, selected: element.selected ? true : false};
				})
			);

			photoFrameCtrl.editPhotoDialog.updateAlbums(editableAlbums);

		}

		function displayPhotoView(photoFrameCtrl, selectedPhoto) {
			photoFrameCtrl.changeMode('photos');			
			// show photo caption section
			photoFrameCtrl.photoCaptionSection.toggle(true);
			var photoView = photoFrameCtrl.Photo(selectedPhoto, {
				incognitoService: photoFrameCtrl.incognitoService,
				incognitoRequestStatus: photoFrameCtrl.incognitoRequestStatus,
				incognitoRelationStatus: photoFrameCtrl.incognitoRelationStatus,
				incognitoStatus: photoFrameCtrl.incognitoStatus,
				userProfile: photoFrameCtrl.userProfile,
				$likeAnimationIcon: photoFrameCtrl.likeCtrl.$likeAnimationIcon,
				photoCssClass: 'photo',
				photoContainerCssClass: 'photo-container',
				photoImageCssClass: 'photo-image',
				incognitoAnimationCssClass: photoFrameCtrl.stopIncognitoAnimation ? '' : 'animated',
				blackRadialGradientLayerCssClass: 'black-radial-gradient-layer black-radial-gradient',
				incognitoCssClass: 'photo-image-incognito-medium white',
				incognitoMediumIconCssClass: 'icn icn-incognito' + (selectedPhoto.ownerGender === 'F' ? 'Woman' : 'Man') + '-medium icn-medium',
				incognitoHugeIconCssClass: 'icn icn-incognito' + (selectedPhoto.ownerGender === 'F' ? 'Woman' : 'Man') + '-huge icn-huge',
				photoFrameCtrl: photoFrameCtrl,
				mediumAnimatedClockCssClass: 'animated-clock medium'
			});
			var old$ = photoFrameCtrl.photo.$;
			photoFrameCtrl.photo.$photoContainer.find('video').trigger('profileBannerSkip');
			photoFrameCtrl.photo.$photoContainer.replaceWith(photoView.$photoContainer);
			photoFrameCtrl.photo = photoView;
			photoFrameCtrl.photo.$ = old$; // TODO: refactor!

			photoFrameCtrl.photo.$photoContainer
				.append(photoFrameCtrl.photoDescription.$)
				.append(photoFrameCtrl.closeButton.click(function (event) {
					event.stopPropagation();
					photoFrameCtrl.destroy();
				}));

			photoFrameCtrl.photoDescription.updateText(photoFrameCtrl.photo.description);
			if (photoFrameCtrl.editPhotoDialog.$photoDescription) {
				photoFrameCtrl.editPhotoDialog.$photoDescription
					.val(photoFrameCtrl.photo.description)
					.data('photoDescription', photoFrameCtrl.photo.description);
				//photoFrameCtrl.editPhotoDialog.updateSymbolsLimit().toggleSubmitButtonState();
			}


			var ratingsAndPhotoCommentsHandler;
			(function (photoFrameCtrl) {
                Mb.ComponentManager.getComponentFromHTMLElementAsync($('.photo-comments-wrap').children()[0], 5000).then(function (photoCommentsInstance)
                {
                    ratingsAndPhotoCommentsHandler = function (ratings, commentsData, complaintsData, editableAlbums, votedObject) {
                        commentsData.profile = photoFrameCtrl.userProfile;
						photoCommentsInstance.showAPI5Comments(commentsData, photoFrameCtrl.albumOwner, photoFrameCtrl.photo.id, photoFrameCtrl, selectedPhoto);
					    updatePhotoRatings(photoFrameCtrl, selectedPhoto, ratings);
                        updateEditableAlbums(photoFrameCtrl, editableAlbums);
                        photoFrameCtrl.updateComplaintDialog(complaintsData);
												
                        photoFrameCtrl.likeCtrl.updateLikeButtonStatus({ selectedPhoto: selectedPhoto, voted: votedObject });
												photolayerStatistics.photoHits += 1;
			
												clearTimeout(photoFrameCtrl.profileBannerPlayTimeout);
												if (photoFrameCtrl.mode !== NEW_FACES_MODE
													&& !photoFrameCtrl.isOwnerProfile
													&& !photoFrameCtrl.nextPhoto.hasClass(NEXT_DISABLED_CSS)
													&& photoFrameCtrl.bannerInstance 
													&& photoFrameCtrl.bannerInstance.shouldShowBanner(selectedPhoto.id)) {
													
													// console.log('show profile banner');
													clearTimeout(photoFrameCtrl.profileBannerUnblockTimeout);
													displayProfileBanner(photoFrameCtrl);
												} else {
													// console.log('check adv section');
													photoFrameCtrl.advBannerSection.showIfHidden();
													$('.js-photo-description').show();
												}
												
												
                    } || ratingsAndPhotoCommentsHandler;

                    if (photoFrameCtrl.defaultRatings || photoFrameCtrl.defaultComments || photoFrameCtrl.defaultComplaints) {
                        ratingsAndPhotoCommentsHandler(photoFrameCtrl.defaultRatings, photoFrameCtrl.defaultComments, photoFrameCtrl.defaultComplaints, photoFrameCtrl.editableAlbums, photoFrameCtrl.defaultVote);
                        // Get from cache only once and then remove!!!
                        photoFrameCtrl.defaultRatings = photoFrameCtrl.defaultComments = photoFrameCtrl.defaultComplaints = null;
                    } else {
                        requestRatingsAndPhotoComments(photoFrameCtrl, selectedPhoto, ratingsAndPhotoCommentsHandler);
                    }
                });
			})(photoFrameCtrl);

			photoFrameCtrl.photo.$photoContainer.click(function (event) {
				if (photoFrameCtrl.nextPhoto.hasClass(NEXT_DISABLED_CSS)) {
					return;
				}
				if (event.which !== 2) {
					photoFrameCtrl.goToNext();
				}
			});

			photoFrameCtrl.photo.$incognitoIcon.click(function (event) {
				event.stopPropagation();
				var self = this;
				// Current user watches own photos OR a user is not authorized;
				if (photoFrameCtrl.userProfile && photoFrameCtrl.userProfile.id === photoFrameCtrl.albumOwner.id || !mambo.aUserId) {
					$(photoFrameCtrl.incognitoRequestStatus).triggerHandler('incognitoRequestStatusChange', [{
						isSent: false,
						isApproved: false,
						isDeclined: false,
						isPending: false
					}]);

						$(self).css({
							top: photoFrameCtrl.photo.$photoImage.height() < 400 ? '10px' : '10%',
							marginLeft: mambo.langIsRTL ? '' : '-62px',
							marginRight: mambo.langIsRTL ? '-62px' : ''
						});

					$(self).removeClass(photoFrameCtrl.photo.incognitoAnimationCssClass)
						.removeClass(photoFrameCtrl.photo.incognitoMediumIconCssClass)
						.addClass(photoFrameCtrl.photo.incognitoHugeIconCssClass);

					photoFrameCtrl.photo.$photoIncognitoLayer.css('top', '40%');
					photoFrameCtrl.photo.$blackRadialGradientLayer.fadeIn(700);

					photoFrameCtrl.photo.$photoImage.attr('src', photoFrameCtrl.photo.srcMasked);
				} else {
					photoFrameCtrl.incognitoService.getOutcomingIncognitoStatus({aid: photoFrameCtrl.albumOwner.id}).then(function (res) {
						photoFrameCtrl.incognitoRequestStatus.isSent = res && res.isSent ? true : false;
						photoFrameCtrl.incognitoRequestStatus.isApproved = res && res.isApproved ? true : false;
						photoFrameCtrl.incognitoRequestStatus.isDeclined = res && res.isDeclined ? true : false;
						photoFrameCtrl.incognitoRequestStatus.isPending = res && res.isPending ? true : false;

						$(photoFrameCtrl.incognitoRequestStatus).triggerHandler('incognitoRequestStatusChange', [photoFrameCtrl.incognitoRequestStatus]);

							$(self).css({
								top: photoFrameCtrl.photo.$photoImage.height() < 400 ? '10px' : '10%',
								marginLeft: mambo.langIsRTL ? '' : '-62px',
								marginRight: mambo.langIsRTL ? '-62px' : ''
							});

						$(self).removeClass(photoFrameCtrl.photo.incognitoAnimationCssClass)
							.removeClass(photoFrameCtrl.photo.incognitoMediumIconCssClass)
							.addClass(photoFrameCtrl.photo.incognitoHugeIconCssClass);

						photoFrameCtrl.photo.$photoIncognitoLayer.css('top', '40%');
						photoFrameCtrl.photo.$blackRadialGradientLayer.fadeIn(700);
					});
				}
			});

			var img = new Image();
			img.onload = function () {
				photoFrameCtrl.photo.$.css({
					'maxWidth': this.width + 'px',
					'maxHeight': this.height + 'px'
					
				});

				photoFrameCtrl.photo.$photoContainer.height('80%');
				photoFrameCtrl.photo.$incognitoIcon.show();

				photoFrameCtrl.stopIncognitoAnimation = photoFrameCtrl.stopIncognitoAnimation || true;

				photoFrameCtrl.photo.$photoImage.attr('src', this.src);
				photoFrameCtrl.photo.$photoContainer.show();
				$('#Wrapper').triggerHandler('updatenicescrollbar'); // TODO: refactor!

				photoFrameCtrl.vkLikeButton.$.off('click').on('click', function () {
					var title = context.t.VK_LIKE_BUTTON_Looking;
					var description = context.t.VK_LIKE_BUTTON_on_site;
					var text = context.t.share;

					var vkButton = VK.Share.button(
						{
							url: encodeURIComponent(window.location.origin
							+ window.location.pathname
							+ '?album_id=' + photoFrameCtrl.photo.album.id
							+ '&photo_id=' + photoFrameCtrl.photo.id
							+ '&from=vk_share'
							+ '#album_id=' + photoFrameCtrl.photo.album.id
							+ '&photo_id=' + photoFrameCtrl.photo.id),
							title: title,
							description: description,
							image: photoFrameCtrl.photo.src,
							noparse: true
						},
						{
							type: "round_nocount",
							text: text
						}
					);

					$(vkButton).find('a:first').trigger('click');
				});
				photoFrameCtrl.fbLikeButton.$.off('click').on('click', function () {
					var windowWidth = 626;
					var windowHeight = 436;
					var url = 'https://www.facebook.com/sharer/sharer.php?u=';
					var origin = window.location.origin;
					var lang = mambo.aUserLang;
					var pathname = window.location.pathname;
					var albumId = photoFrameCtrl.photo.album.id;
					var photoId = photoFrameCtrl.photo.id;

					url += encodeURIComponent(
						origin + '/' + lang + pathname + '?' +
						'album_id=' + albumId + '&photo_id=' + photoId +
						'#album_id=' + albumId + '&photo_id=' + photoId
					);

					window.open(url, '', 'toolbar=0,status=0,width=' + windowWidth + ',height=' + windowHeight);
				});
				if (photoFrameCtrl.incognitoMode) {
					photoFrameCtrl.photo.$incognitoIcon.trigger('click');
					photoFrameCtrl.incognitoMode = false;
				}
			};
			img.src = photoFrameCtrl.photo.src;
		}

		function displayProfileBanner(photoFrameCtrl) {
			// console.log('displayProfileBanner', photoFrameCtrl.mode, 'hide comments');
			photoFrameCtrl.photo.$photoContainer.append(photoFrameCtrl.bannerElement);
			photoFrameCtrl.photo.$photoImage.hide();
			$('.photo-comments-wrap').hide();
			$('.js-photo-image-cover').hide();
			$('.js-photo-description').hide();
			
			photoFrameCtrl.advBannerSection.toggle(false);

			photoFrameCtrl.albumsPreviewSection.disablePreviewSection(true);
			photoFrameCtrl.nextPhoto.addClass(NEXT_DISABLED_CSS);
			photoFrameCtrl.previousPhoto.addClass(PREVIOUS_DISABLED_CSS);
			
			photoFrameCtrl.bannerInstance.startBanner();

			photoFrameCtrl.profileBannerPlayTimeout = setTimeout(function () {
				// console.log('start after timeout');
				// дополнительный старт, т.к. если быстро на клаве прощелкивать - может не стартануть
				photoFrameCtrl.bannerInstance.startBanner();
			}, 100);
			photoFrameCtrl.profileBannerUnblockTimeout = setTimeout(function () {

				unblockPhotoNavigation(photoFrameCtrl);
			}, 1000 * 4);
		}
		
		function unblockPhotoNavigation(photoFrameCtrl) {
			photoFrameCtrl.albumsPreviewSection.disablePreviewSection(false);
			photoFrameCtrl.nextPhoto.removeClass(NEXT_DISABLED_CSS);
			photoFrameCtrl.previousPhoto.removeClass(PREVIOUS_DISABLED_CSS);
		}

		function displayEroticPhotoView(photoFrameCtrl, selectedPhoto) {
			photoFrameCtrl.changeMode('photos');
			// hide photo caption section
			photoFrameCtrl.photoCaptionSection.toggle(false);

			var photoView = photoFrameCtrl.EroticWarning(selectedPhoto, {
				photoCssClass: 'photo',
				eroticWarningCssClass: 'erotic-warning',
				proceedButtonCssClass: 'button button-blue',
				insetCssClass: 'inset',
				insetTextCssClass: 'inset-text'
			});
			var old$ = photoFrameCtrl.photo.$;
			photoFrameCtrl.photo.$photoContainer.replaceWith(photoView.$photoContainer);
			photoFrameCtrl.photo = photoView;
			photoFrameCtrl.photo.$ = old$; // TODO: refactor!

			photoFrameCtrl.photo.$photoContainer
				.append(photoFrameCtrl.photoDescription.$)
				.append(photoFrameCtrl.closeButton.click(function (event) {
					event.stopPropagation();
					photoFrameCtrl.destroy();
				}));

			photoFrameCtrl.photoDescription.updateText('');
			photoFrameCtrl.photo.$.show();

			photoFrameCtrl.photo.$photoContainer.click(function (event) {
				event.stopPropagation();
			});

			photoFrameCtrl.photo.$proceedButton.click(function (event) {
				// show photo caption section;
				photoFrameCtrl.photoCaptionSection.toggle(true);

				if (photoFrameCtrl.photo.$warningCheckbox.prop('checked')) {
					$.get('/my/settings_erotic.phtml?action=enable&s_post=' + mambo.utils.Cookie('s_post'));
					photoFrameCtrl.applicationCtrl.SHOW_EROTIC_WARNING = false;
				}
				photoFrameCtrl.photo.$.css({
					'maxWidth': PhotoFrameCtrl.MIN_PHOTO_WIDTH + 'px',
					'maxHeight': PhotoFrameCtrl.MIN_PHOTO_HEIGHT + 'px'
					
				});
				displayPhotoView(photoFrameCtrl, selectedPhoto);
				event.stopPropagation();
			});
		}

		function displayPhotoNotFoundView(photoFrameCtrl, notFoundPhoto) {
			photoFrameCtrl.changeMode('photos');
			// hide photo caption section
			photoFrameCtrl.photoCaptionSection.toggle(false);

			var photoView = photoFrameCtrl.PhotoNotFound(notFoundPhoto, '#photolayer-PhotoNotFoundTmpl', {
				photoCssClass: 'photo',
				proceedButtonDefaultText: context.t.next,
				proceedButtonFromStartText: context.t.start,
				proceedButtonCloseText: context.t.itsСlear
			});

			photoFrameCtrl.photo.$.css({
				'maxWidth': PhotoFrameCtrl.MIN_PHOTO_WIDTH + 'px',
				'maxHeight': PhotoFrameCtrl.MIN_PHOTO_HEIGHT + 'px'
				
			});

			/* Hack! */
			if (photoFrameCtrl.albumsPreviewSection.photos.length <= 1) {
				photoView.updateProceedButtonText(photoView.proceedButtonCloseText);
				photoView.$proceedButton.click(function (event) {
					photoFrameCtrl.destroy();
				});
			} else if (photoFrameCtrl.albumsPreviewSection.getLastPhoto() === notFoundPhoto) {
				photoView.updateProceedButtonText(photoView.proceedButtonFromStartText);
			} else {
				photoView.updateProceedButtonText(photoView.proceedButtonDefaultText);
			}

			var old$ = photoFrameCtrl.photo.$;
			photoFrameCtrl.photo.$photoContainer.replaceWith(photoView.$photoContainer);
			photoFrameCtrl.photo = photoView;
			photoFrameCtrl.photo.$ = old$;

			photoFrameCtrl.photo.$photoContainer
				.append(photoFrameCtrl.photoDescription.$)
				.append(photoFrameCtrl.closeButton.click(function (event) {
					event.stopPropagation();
					photoFrameCtrl.destroy();
				}));

			photoFrameCtrl.photoDescription.updateText('');

			photoFrameCtrl.photo.$.show();

			photoFrameCtrl.photo.$photoContainer.click(function (event) {
				event.stopPropagation();
			});

			photoView.$proceedButton.click(function (event) {
				photoFrameCtrl.goToNext();
				event.stopPropagation();
			});
		}

		context.PhotoFrameCtrl = PhotoFrameCtrl;
	}

	return module;
});

define('photolayer/Photolayer.js',['require'],function (require) {
	var module = function (context) {
		function Photolayer(cssClass) {
			return $('<div></div>').attr('class', cssClass || '');
		}

		context.Photolayer = Photolayer;


		// PhotoDescription;
		var PhotoDescription = function (photo, cssClass) {
			var self = this;
			if (!(self instanceof PhotoDescription)) {
				return new PhotoDescription(photo, cssClass);
			}
			self.$$span = $('<span></span>').text('');
			self.$ = $('<div></div>')
				.append(self.$$span)
				.attr('class', cssClass || '');
		};
		PhotoDescription.fn = PhotoDescription.prototype;
		PhotoDescription.fn.updateText = function (newText) {
			var self = this;
			if (newText != null) {
                return self;
				self.$$span.text(newText);
			}
			return self;
		};
		context.PhotoDescription = PhotoDescription;


		// PhotoOverlay;
		function PhotoOverlay(cssClass) {
			return $('<div></div>').attr('class', cssClass || '');
		}

		context.PhotoOverlay = PhotoOverlay;


		// PhotoFrame;
		var PhotoFrame = function (cssClass) {
			var self = this;
			if (!(self instanceof PhotoFrame)) {
				return new PhotoFrame(cssClass);
			}
			self.$ = $('<div></div>').attr('class', cssClass || '');
		};
		PhotoFrame.fn = PhotoFrame.prototype;
		PhotoFrame.fn.append = function ($td) {
			var self = this;
			//self.$$tr.append($td);
			self.$.append($td);
			return self;
		};
		context.PhotoFrame = PhotoFrame;
	}

	return module;
});

define('photolayer/PhotoNextPrevious.js',['require'],function (require) {
	
	function calcMarginTop(height) {
		return -(height / 2) + 'px';
	}

	// PreviousPhoto;

	var module = function (context) {
		var PreviousPhoto = function (src, previousPhotoCssClass, previousPhotoWrapCssClass, previousPhotoImageCssClass, usePixelization) {
			var MAX_HEIGHT = 350;
			var MIN_HEIGHT = 250;
			var PREVIEW_SECTION_HEIGHT = 100;
			var $img = $('<img/>');
			previousPhotoCssClass = previousPhotoCssClass || '';
			previousPhotoWrapCssClass = previousPhotoWrapCssClass || '';
			previousPhotoImageCssClass = previousPhotoImageCssClass || '';

			var $result = $('<div></div>')
				.attr('class', 'js-previous-photo-wrap ' + previousPhotoWrapCssClass)
				.css({
					display: !src ? 'none' : 'block'
				})
				.append(
				$img.attr({
					'class': previousPhotoImageCssClass + ' js-previous-photo-image',
					src: src
				})
			);

			$result.on('changeImage', function (event, src, usePixelization) {
				var img = new Image();
				img.onload = function () {
					var height = this.height;
					var width = this.width;
					var height = height < MAX_HEIGHT ? MIN_HEIGHT : MAX_HEIGHT;

					$img.css('height', height + 'px')
						.attr('src', src)
						.parent()
						.removeClass('previous-photo-not-found-wrap')
						.css({
							height: height + 'px',
							marginTop: calcMarginTop(height)
						});

					if (usePixelization) {
						if (!$result.find('.js-previous-photo-image').is($img)) {
							$result.find('.js-previous-photo-image').replaceWith($img);
						}
						mambo.project.Pixelization($img, width, height, true);
					} else {
						if (!$result.find('.js-previous-photo-image').is($img)) {
							$result.find('.js-previous-photo-image').replaceWith($img);
						}
						mambo.project.Pixelization($img, width, height, false);
					}
					$result.find('.js-previous-photo-image').show();
				};

				if (src) {
					img.src = src;
				} else {
					var height = 350;
					var width = 250;

					$img.css('height', height + 'px')
						.parent()
						.addClass('previous-photo-not-found-wrap')
						.css({
							height: height + 'px',
							marginTop: calcMarginTop(height)
						});
					$result.find('.js-previous-photo-image').hide();
				}
			});

			$result.triggerHandler('changeImage', [src, usePixelization]);

			return $result;
		};
		context.PreviousPhoto = PreviousPhoto;


		// NextPhoto;

		var NextPhoto = function (src, nextPhotoCssClass, nextPhotoWrapCssClass, nextPhotoImageCssClass, usePixelization) {
			var MAX_HEIGHT = 350;
			var MIN_HEIGHT = 250;
			var PREVIEW_SECTION_HEIGHT = 100;
			var $img = $('<img/>');
			nextPhotoCssClass = nextPhotoCssClass || '';
			nextPhotoWrapCssClass = nextPhotoWrapCssClass || '';
			nextPhotoImageCssClass = nextPhotoImageCssClass || '';

			var $result = $('<div></div>')
				.attr('class', 'js-next-photo-wrap ' + nextPhotoWrapCssClass)
				.css({
					display: !src ? 'none' : 'block'
				})
				.append(
				$img.attr({
					'class': nextPhotoImageCssClass + ' js-next-photo-image',
					src: src
				})
			);
			

			$result.on('changeImage', function (event, src, usePixelization) {
				var img = new Image();
				img.onload = function () {
					var height = this.height;
					var width = this.width;
					height = height < MAX_HEIGHT ? MIN_HEIGHT : MAX_HEIGHT;

					$img.css('height', height + 'px')
						.attr('src', src)
						.parent()
						.removeClass('next-photo-not-found-wrap')
						.css({
							height: height + 'px',
							marginTop: calcMarginTop(height)
						});

					if (usePixelization) {
						if (!$result.find('.js-next-photo-image').is($img)) {
							$result.find('.js-next-photo-image').replaceWith($img);
						}
						mambo.project.Pixelization($img, width, height, true);
					} else {
						if (!$result.find('.js-next-photo-image').is($img)) {
							$result.find('.js-next-photo-image').replaceWith($img);
						}
						mambo.project.Pixelization($img, width, height, false);
					}
					$result.find('.js-next-photo-image').show();
				};

				if (src) {
					img.src = src;
				} else {
					var height = 350;
					var width = 250;

					$img.css('height', height + 'px')
						.parent()
						.addClass('next-photo-not-found-wrap')
						.css({
							height: height + 'px',
							marginTop: calcMarginTop(height)
						});
					$result.find('.js-next-photo-image').hide();
				}
			});

			$result.triggerHandler('changeImage', [src, usePixelization]);

			return $result;
		};
		context.NextPhoto = NextPhoto;
	};

	return module;
});


define('photolayer/PhotoSlider.js',['require'],function (require) {
	var module = function (context) {
		var PhotoSlider = function (templateId, ie8CssClass) {
			var self = this;
			if (!(self instanceof PhotoSlider)) {
				return new PhotoSlider(templateId, ie8CssClass);
			}
			templateId = templateId || '#photolayer-PhotoSliderTmpl';
			self.$ = $(templateId).tmpl();
			self.$photoSlider = self.$.find('.js-photo-slider');
			self.$photoSliderRunner = self.$.find('.js-photo-slider-runner');

			if ($.browser.msie8) {
				self.$photoSlider.addClass(ie8CssClass);
			}
			self.convertCoeff = 1;
		};
		PhotoSlider.fn = PhotoSlider.prototype;
		PhotoSlider.fn.inject = function () {
			var self = this;
			context.utils.inject.call(self, arguments[arguments.length - 1] || {});
			return self;
		};
		PhotoSlider.fn.updateWidth = function () {
			var self = this;
			if (!self.albumsPreviewSection) {
				throw new Error('[this.albumsPreviewSection] is not defined');
			}
			var MIN_SCROLL_WIDTH = 1;
			var scrollWidth = self.albumsPreviewSection.$.get(0).scrollWidth;
			var windowWidth = self.albumsPreviewSection.$.innerWidth();
			var convertCoeff = scrollWidth / windowWidth;
			var deltaScroll = (scrollWidth - windowWidth) / convertCoeff; // How many pixels left to scroll;
			var photoSliderWidth = self.$photoSlider.width();
			photoSliderWidth = photoSliderWidth - deltaScroll > MIN_SCROLL_WIDTH ? photoSliderWidth - deltaScroll : MIN_SCROLL_WIDTH;
			self.$photoSliderRunner.css('width', photoSliderWidth + 'px');
			self.convertCoeff = convertCoeff;
			return self;
		};
		PhotoSlider.fn.setLeft = function (newLeft, useAnimation, callback) {
			var self = this;
			if (!self.albumsPreviewSection) {
				throw new Error('[this.albumsPreviewSection] is not defined');
			}
			if (isNaN(newLeft)) {
				throw new Error('[' + newLeft + ' is not a number]');
			}
			if (useAnimation) {
				var from = self.albumsPreviewSection.$.scrollLeft();
				var to = newLeft;
				var startTime = (new Date()).getTime();
				var duration = 450;
				var timerId = null;

				var animate = function () {
					var time = (new Date()).getTime();
					if (time - startTime >= duration) {
						self.setLeft(to);
						clearTimeout(timerId);
						(typeof callback === 'function' ? callback : function () {
						})();
						return;
					} else {
						self.setLeft(from + (to - from) * (time - startTime) / duration);
					}
					timerId = setTimeout(animate, 10);
				};
				animate();
			} else {
				self.albumsPreviewSection.$.scrollLeft(newLeft);
				self.$photoSliderRunner.css('left',
					self.albumsPreviewSection.$.scrollLeft() / self.convertCoeff + 'px'
				);
			}
			return self;
		};
		PhotoSlider.fn.getLeft = function () {
			var self = this;
			if (!self.albumsPreviewSection) {
				throw new Error('[this.albumsPreviewSection] is not defined');
			}
			return self.albumsPreviewSection.$.scrollLeft();
		};
		PhotoSlider.fn.toggle = function (value) {
			var self = this;
			if (value) {
				self.$.show();
			} else {
				self.$.hide();
			}
			return self;
		};
		context.PhotoSlider = PhotoSlider;
	}

	return module;
});

define('photolayer/PreviewSectionCtrl.js',['require'],function (require) {
	var module = function (context) {
		function PreviewSectionCtrl() {
			var dependencies = arguments[arguments.length - 1] || {};
			this.photoLayer = dependencies.photoLayer;
			this.albumsPreviewSection = dependencies.albumsPreviewSection;
			this.smallAlbumsPreviewSection = dependencies.smallAlbumsPreviewSection;
			this.albumsPreviewSectionWrapper = dependencies.albumsPreviewSectionWrapper;
			this.photoSlider = dependencies.photoSlider;
			this.clickedPhotoId = dependencies.clickedPhotoId;
			this.currentAlbumId = dependencies.currentAlbumId;
			this.openedAlbumPreviousPhotoId = dependencies.openedAlbumPreviousPhotoId;
			this.openedAlbumNextPhotoId = dependencies.openedAlbumNextPhotoId;
			this.photoFrame = dependencies.photoFrame;
			this.advBannerSection = dependencies.advBannerSection;
			this.applicationCtrl = dependencies.applicationCtrl;
			this.newFaces = dependencies.newFaces;
		}

		PreviewSectionCtrl.fn = PreviewSectionCtrl.prototype;
		PreviewSectionCtrl.fn.init = function () {
			var self = this;
			var dependencies = arguments[arguments.length - 1] || {};
			var photoFrameCtrl = dependencies.photoFrameCtrl;
			var smallAlbumsPreviewSection = self.smallAlbumsPreviewSection;
			var albumsPreviewSection = self.albumsPreviewSection;
			var albumsPreviewSectionWrapper = self.albumsPreviewSectionWrapper;
			var photoSlider = self.photoSlider;
			var photoLayer = self.photoLayer;
			var photoLayerClickHandler = function (event) {
				event.stopPropagation();
				photoLayer.off('click', photoLayerClickHandler);
			};

			photoSlider.inject({albumsPreviewSection: albumsPreviewSection}); // dependency;
			photoSlider.updateWidth();
			photoSlider.$.click(function (event) {
				event.stopPropagation();
			});

			if (albumsPreviewSection.photos.length <= 1) {
				photoSlider.toggle(false);
				smallAlbumsPreviewSection.toggle(false);
				albumsPreviewSection.toggle(false);
				this.newFaces.liftUp();
			} else {
				this.newFaces.center();
			}

			// TODO: hardcore! Refactor;
			smallAlbumsPreviewSection.selectPreviewSectionByIndex(
				self.albumsPreviewSection.$.find('.album-image-section-selected').index()
			);

			smallAlbumsPreviewSection.$.find('.small-album-section').click(function (event) {
				smallAlbumsPreviewSection.deselectAllPreviewSections().selectPreviewSectionByIndex($(this).index());
				albumsPreviewSection.$
					.find('.album-section')
					.eq($(this).index())
					.find('.album-image-section:first')
					.trigger('click');
				event.stopPropagation();
				event.preventDefault();
			});
			$.each(albumsPreviewSection.photos, function (index, photo) {
				photo.view.$
					.mouseenter(function () { 
						$(this).addClass('album-image-section-hovered');
					})
					.mouseleave(function () {
						$(this).removeClass('album-image-section-hovered');
					})
					.click(function () {						
						if (albumsPreviewSection.isDisabled()) {
							return;
						}
						
						self.unviewAll();
						context.layerData.setStartPhoto( {id: photo.id, album: {id: photo.album.id}} );
						photoFrameCtrl.changePhoto(photo, true);
					});
			});

			self.setPreviewImage(albumsPreviewSection.getPhotoInAlbum(self.clickedPhotoId, {id: self.currentAlbumId}).view.$);

			// PhotoSlider Drag 'n' drop;
			photoSlider.$.mousedown(function (event) {
				var initClientX = event.clientX;
				var currentLeft = photoSlider.getLeft();

				var mousemoveHandler = function (event) {
					var clientX = event.clientX;
					photoSlider.setLeft(currentLeft + (clientX - initClientX) * photoSlider.convertCoeff);
					self.albumsPreviewSection.loadVisibleAlbumImages();
				};
				var mouseupHandler = function (event) {
					$(document).off('mousemove', mousemoveHandler);
					$(document).off('mouseup', mouseupHandler);
				};

				$(document).on('mousemove', mousemoveHandler);
				$(document).on('mouseup', mouseupHandler);
				photoLayer.on('click', photoLayerClickHandler);
				event.stopPropagation();
				event.preventDefault();
			}).click(function (event) {
				photoLayer.off('click', photoLayerClickHandler);
				event.stopPropagation();
			});

			albumsPreviewSection.$.mousewheel(function (event) {
				var $self = $(this);
				var SCROLL_UP = 1;
				var SCROLL_DOWN = -1;
				var STEP = 150;
				var deltaY = event.deltaY;

				if (SCROLL_UP === deltaY) {
					// Scroll to the left;
					photoSlider.setLeft($self.scrollLeft() - STEP);
				} else if (SCROLL_DOWN === deltaY) {
					// Scroll to the right;
					photoSlider.setLeft($self.scrollLeft() + STEP);
					self.albumsPreviewSection.loadVisibleAlbumImages();
				}
				event.preventDefault();
				event.stopPropagation();
			});

			albumsPreviewSection.$.on('touchstart', function (event) {

				var touchendHandler = function () {
					$(this).off('touchend', touchendHandler);
					setTimeout(function () {
						albumsPreviewSection.loadVisibleAlbumImages();
					}, 500);
				};

				$(this).on('touchend', touchendHandler);

				event.stopPropagation();
			});

			if (albumsPreviewSection.$.get(0).scrollWidth > albumsPreviewSection.$.outerWidth(true)) {
				// Show photoslider;
				photoSlider.toggle(true);
			} else {
				photoSlider.$.detach();
			}

			var $Wrapper = $('#Wrapper');
			$(window).resize(function () {
				if (context.utils.isTouchDevice()) {
					return;
				}

				if ($(window).height() < self.applicationCtrl.WINDOW_MIN_HEIGHT) {
					self.albumsPreviewSectionWrapper.hide();
					self.smallAlbumsPreviewSection.toggle(true);
					self.albumsPreviewSectionWrapper.parent().css({
						'min-height': '50px'
					});
					photoSlider.toggle(false);
				} else {
					self.albumsPreviewSectionWrapper.show();
					self.smallAlbumsPreviewSection.toggle(false);
					self.albumsPreviewSectionWrapper.parent().css({
						'min-height': '90px'
					});
					photoSlider.toggle(true);
				}

				if (self.advBannerSection.exists) {
          var windowHeight = $(window).height();
					if (self.advBannerSection.shouldHideAdvSection()) {
						self.advBannerSection.toggle(false);
					} else {
						self.advBannerSection.initAdvBannerUrl(mambo.aAnketaId, mambo.partnerId, 1000, windowHeight);
						self.advBannerSection.toggle(true);
					}
				}

				photoSlider.updateWidth();
				$('#Wrapper').triggerHandler('updatenicescrollbar');
			}).trigger('resize');

			self.albumsPreviewSection.loadVisibleAlbumImages();
		};
		PreviewSectionCtrl.fn.getAlbumsPreviewSectionWidth = function () {
			var width = 0;
			this.albumsPreviewSection.$.find('.album-section').each(function () {
				width += $(this).outerWidth(true);
			});
			return width;
		};
		PreviewSectionCtrl.fn.setPreviewImage = function ($newSelectedImage) {
			var self = this;
			var albumsPreviewSection = self.albumsPreviewSection;
			var albumsPreviewSectionWrapper = self.albumsPreviewSectionWrapper;

			var $oldSelectedImage = albumsPreviewSection.$
				.find('.album-image-section.album-image-section-selected')
				.removeClass('album-image-section-selected');
			$newSelectedImage.addClass('album-image-section-selected');

			if (!$oldSelectedImage.length) {
				// No image was selected before;
				// Let the very first photo to be considered $oldSelectedImage;
				$oldSelectedImage = albumsPreviewSection.getFirstPhoto();
				// If by any chance $oldSelectedImage has got no view, return;
				if (!$oldSelectedImage.view) {
					return;
				}
				$oldSelectedImage = $oldSelectedImage.view.$;
			}

			var oldLeft = $oldSelectedImage.position().left;
			var newLeft = $newSelectedImage.position().left;
			var currentLeft = albumsPreviewSection.$.position().left;
			var sectionsWidth = self.getAlbumsPreviewSectionWidth();
			var sectionsWrapperWidth = albumsPreviewSectionWrapper.outerWidth(true);

			if (sectionsWrapperWidth > sectionsWidth) {
				return;
			}

			if (oldLeft !== newLeft) {
				newLeft = currentLeft + oldLeft - newLeft
			} else {
				newLeft = currentLeft;
			}

			newLeft = $newSelectedImage.position().left + self.photoSlider.getLeft() - 2 * $newSelectedImage.width();
			self.photoSlider.setLeft(newLeft, true, function () {
				self.albumsPreviewSection.loadVisibleAlbumImages();
			});
			// scroll photo slider runner;
		};
		
		PreviewSectionCtrl.fn.deselectAllPreviews = function () {
			this.albumsPreviewSection.$.find('.album-image-section').removeClass('album-image-section-selected');
		}
		
		PreviewSectionCtrl.fn.unviewAll = function () {			
			this.albumsPreviewSection.photos.forEach(function (photo) {
				delete photo.viewed;
			});
		}

		function getScrollingAlbumSection(previewSectionCtrl) {
			var $scrollingAlbumSection = $({});
			previewSectionCtrl.albumsPreviewSection.$.find('.album-section').each(function () {
				var offset = $(this).offset();
				var position = $(this).position();
				if (offset.left - position.left <= 0 && Math.abs(offset.left) <= $(this).outerWidth(true)) {
					$scrollingAlbumSection = $(this);
					return false;
				}
			});
			return $scrollingAlbumSection;
		}

		function resetAlbumCaptionTextLeftAfter($albumSections, $scrollingAlbumCaptionText) {
			$albumSections.find('.album-caption-text').each(function () {
				var albumCaptionTextLeft = $(this).position().left;
				var scrollingAlbumCaptionTextLeft = $scrollingAlbumCaptionText.position().left;
				if (albumCaptionTextLeft > scrollingAlbumCaptionTextLeft) {
					$(this).css({
						left: 0,
						marginLeft: 0
					});
				}
			});
		}

		function setAlbumCaptionTextLeft(albumCaptionText, left, animation) {
			var OFFSET_LEFT = 14;
			var left = left - (albumCaptionText.parent().position() ? albumCaptionText.parent().position().left : 0);
			left = left <= 0 ? 0 : left;
			if (left + albumCaptionText.width() + OFFSET_LEFT > albumCaptionText.parent().width()) {
				return;
			}
			albumCaptionText.css('margin-left', (!left ? 0 : OFFSET_LEFT + 'px'));
			if (animation) {
				albumCaptionText.stop().animate({
					left: left + 'px'
				}, 450);
			} else {
				albumCaptionText.css('left', left + 'px');
			}
		}

		context.PreviewSectionCtrl = PreviewSectionCtrl;
	}

	return module;
});
 
define('photolayer/SmallAlbumSection.js',['require'],function (require) {
	var module = function (context) {
		var SmallAlbumSection = function (album, cssClass, selected) {
			var self = this;
			if (!(self instanceof SmallAlbumSection)) {
				return new SmallAlbumSection(album, cssClass, selected);
			}
			album.photos = album.photos || [];
			self.$ = $('<a></a>')
				.attr({'class': cssClass || '', href: '#'})
				.text(album.name)
				.toggleClass('small-album-section-selected', selected ? true : false);
			if (!album.photos.length) {
				self.$.hide();
			}
		};
		SmallAlbumSection.fn = SmallAlbumSection.prototype;
		context.SmallAlbumSection = SmallAlbumSection;


// Preview

		var SmallAlbumsPreviewSection = function (cssClass, albumSectionCssClass, albumSectionSelectedCssClass) {
			var self = this;
			if (!(self instanceof SmallAlbumsPreviewSection)) {
				return new SmallAlbumsPreviewSection(cssClass, albumSectionCssClass, albumSectionSelectedCssClass);
			}
			self.$ = $('<div></div>').attr('class', cssClass || '');
			self.albumSectionCssClass = albumSectionCssClass || '';
			self.albumSectionSelectedCssClass = albumSectionSelectedCssClass || '';
		};
		SmallAlbumsPreviewSection.fn = SmallAlbumsPreviewSection.prototype;
		SmallAlbumsPreviewSection.fn.appendPreviewSection = function (smallAlbumSection) {
			var self = this;
			if (smallAlbumSection != null) {
				self.$.append(smallAlbumSection.$);
			}
			return self;
		};
		SmallAlbumsPreviewSection.fn.selectPreviewSectionByIndex = function (albumSectionIndex) {
			var self = this;
			self.$
				.find('.' + self.albumSectionCssClass + ':eq(' + albumSectionIndex + ')')
				.addClass(self.albumSectionSelectedCssClass);
			return self;
		};
		SmallAlbumsPreviewSection.fn.togglePreviewSectionByIndex = function (albumSectionIndex, value) {
			var self = this;
			var $previewSection = self.$.find('.' + self.albumSectionCssClass + ':eq(' + albumSectionIndex + ')');
			(value ? $previewSection.show() : $previewSection.hide());
			return self;
		};
		SmallAlbumsPreviewSection.fn.deletePreviewSectionByIndex = function (albumSectionIndex) {
			var self = this;
			var $previewSection = self.$.find('.' + self.albumSectionCssClass + ':eq(' + albumSectionIndex + ')');
			if ($previewSection.hasClass(self.albumSectionSelectedCssClass)) {
				self.selectPreviewSectionByIndex(albumSectionIndex - 1);
			}
			$previewSection.remove();
			return self;
		};
		SmallAlbumsPreviewSection.fn.deselectAllPreviewSections = function () {
			var self = this;
			self.$.find('.' + self.albumSectionCssClass).removeClass(self.albumSectionSelectedCssClass);
			return self;
		};
		SmallAlbumsPreviewSection.fn.toggle = function (value) {
			var self = this;
			if (value) {
				self.$.show();
			} else {
				self.$.hide();
			}
			return self;
		};
		context.SmallAlbumsPreviewSection = SmallAlbumsPreviewSection;
	}


	return module;
});

define('photolayer/Application.js',['require','photolayer/config.js','photolayer/AdvBannerSection.js','photolayer/LikeCtrl.js','photolayer/Button.js','photolayer/PhotolayerNewFaces.js','photolayer/AlbumsPreviewSection.js','photolayer/ApplicationCtrl.js','photolayer/ComplaintDescription.js','photolayer/ComplaintDialog.js','photolayer/ConfirmationDialog.js','photolayer/EditPhotoDialog.js','photolayer/EroticWarning.js','photolayer/HttpErrorModal.js','photolayer/MobileMambaAPI.js','photolayer/Photo.js','photolayer/PhotoCaption.js','photolayer/PhotoDescription.js','photolayer/PhotoFrameCtrl.js','photolayer/Photolayer.js','photolayer/PhotoNextPrevious.js','photolayer/PhotoSlider.js','photolayer/PreviewSectionCtrl.js','photolayer/SmallAlbumSection.js'],function (require) {

	var module = function (blitzTempl, incognitoCanUse, knockout) {

		var context = require('photolayer/config.js');
		context.t = blitzTempl;
		context.knockout = knockout;

		require('photolayer/AdvBannerSection.js')(context);
		require('photolayer/LikeCtrl.js')(context);
		require('photolayer/Button.js')(context);
		require('photolayer/PhotolayerNewFaces.js')(context);
		require('photolayer/AlbumsPreviewSection.js')(context);
		require('photolayer/ApplicationCtrl.js')(context);
		require('photolayer/ComplaintDescription.js')(context);
		require('photolayer/ComplaintDialog.js')(context);
		require('photolayer/ConfirmationDialog.js')(context);
		require('photolayer/EditPhotoDialog.js')(context);
		require('photolayer/EroticWarning.js')(context);
		require('photolayer/HttpErrorModal.js')(context);
		require('photolayer/MobileMambaAPI.js')(context);
		require('photolayer/Photo.js')(context);
		require('photolayer/PhotoCaption.js')(context);
		require('photolayer/PhotoDescription.js')(context);
		require('photolayer/PhotoFrameCtrl.js')(context);
		require('photolayer/Photolayer.js')(context);
		require('photolayer/PhotoNextPrevious.js')(context);
		require('photolayer/PhotoSlider.js')(context);
		require('photolayer/PreviewSectionCtrl.js')(context);
		require('photolayer/SmallAlbumSection.js')(context);

		context.incognitoCanUse = incognitoCanUse === '0' || incognitoCanUse === '' || !mambo.aUserId ? false : true;
				
		context.layerData = {
			
			startPhoto: null,
			
			numPhotosViewedInRow: 0,
			
			setStartPhoto: function (photo) {
				this.startPhoto = photo;
				this.numPhotosViewedInRow = 1;
			}
			
		};
		
		var PhotolayerLoader = function () {
			var self = this;
			var loaded = false;
			self.loaded = function (value) {
				if (typeof value === 'boolean') {
					loaded = value;
				} else if (value == null /* no arguments */) {
					return loaded;
				}
				return self;
			};
		};
		PhotolayerLoader.fn = PhotolayerLoader.prototype;
		PhotolayerLoader.fn.init = function () {
		};
		PhotolayerLoader.fn.checkInputParams = function (albumId, photoId) {
			if (!albumId || !photoId) {
				return false;
			}
			return true;
		};
		PhotolayerLoader.fn.removeView = function () {
			var self = this;
			$('#PhotolayerLoader').remove();
			return self;
		};

		PhotolayerLoader.fn.render = function (photoId, albumId, incognitoMode) {
			var self = this;
            var applicationCtrl = new context.ApplicationCtrl(photoId, albumId, {
                WINDOW_MIN_WIDTH: 960,
                WINDOW_MIN_HEIGHT: 700,
                incognitoMode: incognitoMode || false,
                SHOW_EROTIC_WARNING: context.t.SHOW_EROTIC_WARNING || '',
                VK_LIKE_BUTTON: context.t.VK_LIKE_BUTTON || '',
                FB_LIKE_BUTTON: context.t.FB_LIKE_BUTTON || '',
				likeButtonAvailable: context.t.likeButtonAvailable
            });
			
            var $photoOverlay = applicationCtrl.createOverlay();
			if ($('.photolayer-photo-overlay').length) return;
            $('body').append($photoOverlay);

			var mambaAPI = context.mambaAPI;

			//---- begin of optimization hell
			var sysRequestsContainer = [];

			var photoFrameMinWidth = 600;
			var photoFrameMinHeight = 450;

			/* Get all photos, albums and profile */
			var allPhotosRequestIndex = sysRequestsContainer.push({
					uri: '/users/' + mambo.vUserId + '/albums/photos/',
					method: 'GET',
					params: {
						langId: mambo.aUserLang,
						dateType: 'timestamp',
						limit: -1
					}
				}) - 1;

			/* Get All Photo Comments */
			var commentsIndex = sysRequestsContainer.push({
					uri: '/users/' + mambo.vUserId + '/photos/' + photoId + '/comments/',
					method: 'GET',
					params: {
						langId: mambo.aUserLang,
						dateType: 'timestamp',
						limit: 10000
					}
				}) - 1;

			if (mambo.aAnketaId === mambo.vUserId) {
				/* Get Photo Ratings */
				var ratingsIndex = sysRequestsContainer.push({
						uri: '/photos/' + photoId + '/edit/',
						method: 'GET',
						params: {
							langId: mambo.aUserLang
						}
					}) - 1;
			}

			/* Get Photo Complaints */
			var complaintsIndex = sysRequestsContainer.push({
					uri: '/complaint/causes/' + mambo.vUserId + '/photo/' + photoId + '/',
					method: 'GET',
					params: {
						lang_id: mambo.aUserLang,
						reqType: 'json'
					}
				}) - 1;
			
			/* Photo voted info */
			var votedIndex = sysRequestsContainer.push({
				uri: '/photorating/check_voted/',
				method: 'GET',
				params: {
					photoId: photoId
				}
			}) - 1;

			/* Profile Mini */
			var profileMini = sysRequestsContainer.push({
					uri: '/profile/mini/',
					method: 'GET',
					params: {
						langId: mambo.aUserLang,
						dateType: 'timestamp',
						limit: -1
					}
				}) - 1;


			mambaAPI.containerRequest({
				langId: mambo.aUserLang
			}, sysRequestsContainer, function (error, data) {

				if (error) {

					var $errorModal = $('#photolayer-HttpErrorModalTmpl').tmpl();
					var $photolayerPhotoOverlay = $('.photolayer-photo-overlay');
					$errorModal.find('.js-close-button').on('click', function (event) {
						$errorModal.remove();
						$photolayerPhotoOverlay.remove();
					});
					$photolayerPhotoOverlay.on('click', function (event) {
						$errorModal.remove();
						$photolayerPhotoOverlay.remove();
					});
					$('body').append($errorModal);

					self.loaded(false);

					Mb.ComponentManager.getComponentInstance('Web/Modal').then(function (modal) {
						var httpErrorModal = context.httpErrorModal;
						modal.show(httpErrorModal.$.get(0));

						httpErrorModal.$submitButton.on('click', function () {
							modal.hideAll();
						});

						$photoOverlay.remove();
						window.location.hash = '#/closed';
					});

					return;
				}

				var sysResponsesContainer = data.sysResponsesContainer || [];

				//-
				var albums = sysResponsesContainer[allPhotosRequestIndex].albums || [];
				var albumOwner = sysResponsesContainer[allPhotosRequestIndex].anketa || {};
				var profile = sysResponsesContainer[profileMini].anketa || {};

				var isOwnerProfile = albumOwner && profile && albumOwner.id === profile.id;

				//-
				var currentAlbum = {};
				var defaultAlbum = {};
				var editableAlbums = [];

				var commentsData = null;
				var complaintsObject = null;
				var votedObject = null;
				var ratings = null;

				// comments;
				commentsData = sysResponsesContainer[commentsIndex] || commentsData;

				// complaints;
				complaintsObject = sysResponsesContainer[complaintsIndex] || complaintsObject;

				// voted;
				votedObject = sysResponsesContainer[votedIndex] || votedObject;

                function setCurrentAlbum ( albumObj ) {
                    var albumId = albumObj.id;
				
					if (albumObj.photoCounts) {
						var photoId = albumObj.photos[0].id;
						context.layerData.setStartPhoto({id: photoId, album: {id: albumId}});
					} else if (window.stats) {
						stats.jsErrorPush(new Error('Application.js: album without photos'));
					}
                    applicationCtrl.clickedPhotoAlbumId = albumId;
                    currentAlbum = albumObj;
                }

				//searching current selected album
				for (var i = 0, l = albums.length, wasUnshifted; i < l; i++) {
					if (albums[i].id === albumId) {
						currentAlbum = albums[i];
						if (wasUnshifted) break;
					}

                    albums[i].photos = albums[i].photos || [];

					if (albums[i]['default'] === true) {

						defaultAlbum.id = albums[i].id;
						defaultAlbum.name = albums[i].name;

						for (var j = 0, defaultPhoto, l2 = albums[i].photos.length; j < l2; j++) {
                            if (albumId < 0 && albums[i].photos[j].id == photoId) {
                                //set album is current
                                setCurrentAlbum( albums[i] );
                            }
							if (albums[i].photos[j]['default'] === true) {
								defaultPhoto = albums[i].photos[j];
								albums[i].photos.splice(j, 1);

								// set default photo as the first element in array;
								albums[i].photos.unshift(defaultPhoto);
								wasUnshifted = true;

                                if (albumId >= 0) break;
							}
						}
						if (!_.isEmpty(currentAlbum)) break;
					} else if (albumId < 0) {

                        //albumId not exsit, searching photoId in this album and set this album is current
                        for (var j = 0, l2 = albums[i].photos.length; j < l2; j++) {
                            if (albums[i].photos[j].id == photoId) {
                                //set album is current
                                setCurrentAlbum( albums[i] );
                                break;
                            }
                        }

                    }
				}

     			// ratings;
				///TODO: hardcore!!! Yeah , baby, yeah!!! Weehaa!
				try {
					var fields = sysResponsesContainer[ratingsIndex].formBuilder.blocks[0].fields || [];
					for (var k = 0, ll = fields.length; k < ll; k++) {
						switch (fields[k].field) {
							case 'rating_id':
								ratings = fields[k].variants || ratings;
								break;
							
							case 'album_id':
								editableAlbums = fields[k].variants || editableAlbums;
								break;
							
							default: 
								break;
						}
					}
				} catch (e) {}

				var albumPhotos = currentAlbum.photos || [];
				var previousPhotoId = 0;
				var nextPhotoId = 0;

				for (var i = 0, l = albumPhotos.length; i < l; i++) {
					if (albumPhotos[i].id > photoId) {
						nextPhotoId = albumPhotos[i].id;
						previousPhotoId = albumPhotos[i - 1] ? albumPhotos[i - 1].id : previousPhotoId;
					} else if (albumPhotos[i].id == photoId) {
						nextPhotoId = albumPhotos[i + 1] ? albumPhotos[i + 1].id : nextPhotoId;
						previousPhotoId = albumPhotos[i - 1] ? albumPhotos[i - 1].id : previousPhotoId;
					}
				}
				if (!nextPhotoId && !previousPhotoId) {
					previousPhotoId = albumPhotos[l - 1] ? albumPhotos[l - 1].id : previousPhotoId;
				}

				// incognito status request;
				require(['Modules/Dal/IncognitoService', 'Modules/Dal/PhotoratingService'], function (incognitoService, photoratingService) {
					// Request incognito status, baby!
					var incognitoStatus = {isOn: albumOwner.incognitoIsOn ? true : false};
					var incognitoRelationStatus = {canView: albumOwner.incognitoCanView ? true : false};
					var incognitoRequestStatus = {
						isSent: false,
						isApproved: false,
						isDeclined: false,
						isPending: false
					};

					applicationCtrl.init({
						commentsData: commentsData,
						complaintsObject: complaintsObject,
						votedObject: votedObject,
						ratings: ratings,
						photoId: photoId,
						nextPhotoId: nextPhotoId * 1,
						previousPhotoId: previousPhotoId * 1,
						photolayerLoader: self,
						isOwnerProfile: isOwnerProfile,
						userProfile: profile,
						albumOwner: albumOwner,
						photoFrameMinWidth: photoFrameMinWidth,
						photoFrameMinHeight: photoFrameMinHeight,
						defaultAlbum: defaultAlbum,
						albums: albums,
						incognitoStatus: incognitoStatus,
						incognitoRelationStatus: incognitoRelationStatus,
						incognitoRequestStatus: incognitoRequestStatus,
						incognitoService: incognitoService,
						editableAlbums: editableAlbums,
						photoratingService: photoratingService
					});

					self.loaded(true).removeView();
				});
			});

			//---- end of optimization hell

			return self;
		};

		var photolayerLoader = new PhotolayerLoader();


		var hashChangeHandler = function () {
			if (photolayerLoader.loaded() || (typeof anketaMessagePhoto !== 'undefined' && anketaMessagePhoto)) {
				return;
			}
			var photoId = window.location.hash.match(/photo_id=(\d+)/);
			var albumId = window.location.hash.match(/album_id=(\d+)/);


            //if albumId not exist, set it -1
            if (!albumId) {
                albumId = [];
                albumId[1] = -1;
            }

			if (photoId && albumId) {
				context.layerData.setStartPhoto( {id: parseInt(photoId[1]), album: {id: parseInt(albumId[1])}} );

				var incognitoMode = window.location.hash.match(/incognitoMode=(\d+)/);
				photolayerLoader.init();

				if (photolayerLoader.checkInputParams(albumId, photoId)) {
					var photoId = photoId[1] * 1;
					var albumId = albumId[1] * 1;
					
					photolayerLoader.render(photoId, albumId, incognitoMode);
				}
			}
		};
		$(window).on('hashchange', hashChangeHandler);

        $(document).on('VIP_MODULE_BOUGHT_PHOTO', function () {
            var savePhotoLayerUrl = $.cookie('savePhotoLayerUrl');
            var photoId = savePhotoLayerUrl.match(/photo_id=(\d+)/);
            var albumId = savePhotoLayerUrl.match(/album_id=(\d+)/);
            if (photoId && albumId) {
                window.location.hash = '#/' + 'photo_id=' + photoId + '&album_id=' + albumId;
            }
        });
		
		if (typeof anketaMessagePhoto === 'undefined' || !anketaMessagePhoto) {
			hashChangeHandler();
		}
		
	};

	return module;
});

