/**
 * @copyright Mamba ( http://corp.mamba.ru/ )
 */

/*global mambo: false, window: false */
/*jslint sloppy: true, forin: true, plusplus: true, maxerr: 50, indent: 4 */

mambo.use('{mambo.ui}Input', '{mambo.ui}Alert', '{mambo.utils}Check', '{mambo.ui}Select', 'libs/moment',  function ($) {

	mambo.project = mambo.project || {};

	mambo.project.Registration = function (id, params) {

		params = params || {};

		var $Form = $(id);

		var $Steps = $('#RegSteps');
		var $Step1 = $('#RegStep1');
		var $Step2 = $('#RegStep2');

		var $Submit = $Form.find('input[type="submit"]');

		var step = params.step || 1;
		var version = params.version || 1;

		var inputs = {};

		var checkFields = {};
		
		/*
		 * Login Validator
		 */

		function bindLogin() {

			if (inputs.login) {
				return;
			}

			var $Field = $Form.find('.Login');
			var $Input = $Field.find('input');
			var $Error = $Field.find('.error');
			var $LoginCheck = $Field.find('.checkLogin');
			var $LoginHelp = $Field.find('.error-help');

			function loginHelper(data) {
				var html = '';
				var i;

				for (i in data) {
					if (data.hasOwnProperty(i)) {
						html += '<div><a href="#" class="dotted">' + data[i] + '</a></div>';
					}
				}

				$LoginHelp.html(html).slideDown('fast');
				$LoginHelp.find('a').bind('click', function (e) {
					e.preventDefault();
					$Input.val(this.innerHTML).attr('data-valid', 2);
					$Field.removeClass('invalid valid').addClass('okey');
					$LoginHelp.slideUp('fast');
					if (inputs.password) {
						inputs.password.update();
					}
				});
			}

			function validate() {
				var data = $Form.serialize();
				var action = $Form.attr('action');
				$Field.addClass('check');
				$.post(action + '&check=login', data, function (R) {
					$Field.removeClass('check invalid valid okey');

					R = getR(R);

					if (R && R.XForms && R.XForms.Reg && R.XForms.Reg.login) {
						$Input.attr('data-valid', 0);
						$Error.html(R.XForms.Reg.login);
						$Field.addClass('invalid');

						if (R.d && R.d.logins) {
							loginHelper(R.d.logins);
						}
					} else {
						$Input.attr('data-valid', 2);
						$Field.addClass('okey');
						$LoginHelp.slideUp('fast');
					}
				});
			}

			function value() {
				return $Input.val();
			}

			function check() {
				if ($Input.attr('data-valid') > 0) {
					return true;
				} else {
					$Input.trigger('validate');
				}
				return false;
			}

			/*
			 * Run
			 */

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.login = {
					value: value,
					check: check
				};
			} else {
				return;
			}

			$Input.bind('input validate', function (e) {
				var val = $Input.val();

				$Input.attr('data-valid', 0);
				$Field.removeClass('invalid valid okey');

				if (val.length < 1) {
					if (e.type == 'validate') {
						$Field.addClass('invalid');
						$Error.html(params.lang.login ? params.lang.login[0] : '');
					}
				} else if (!val.match(/^[a-zA-Z0-9_-]+$/)) {
					$Field.addClass('invalid');
					$Error.html(params.lang.login ? params.lang.login[1] : '');
				} else {
					$Field.addClass('valid');
					$Input.attr('data-valid', 1);
				}

				if (inputs.password) {
					inputs.password.update();
				}
			});

			$Input.bind('change', function (e) {
				if ($Input.attr('data-valid') == 1) {
					validate();
				}
			});

			if ($Input.val()) {
				$Input.trigger('validate').trigger('change');
			}
		}

		/*
		 * Password Validator
		 */

		function bindPassword() {

			if (inputs.password) {
				return;
			}

			var $Field = $Form.find('.Passwd');
			var $Input = $Field.find('input');

			function check() {
				$Input.trigger('validate');
				if ($Input.attr('data-valid') > 0) {
					return true;
				}
				return false;
			}

			function update() {
				$Input.trigger('input');
			}

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.password = {
					check: check,
					update: update
				};
			} else {
				return;
			}

			var $Indicator = $Field.find('.indicator');
			var $Txt = $Indicator.find('div');
			var txts = params.lang.password;

			$Input.bind('input validate', function (e) {
				var val = $Input.val();
				var name = inputs.login ? inputs.login.value() : '';

				$Field.removeClass('valid invalid okey');
				$Input.attr('data-valid', 0);

				if (!val.length) {
					$Indicator.removeClass();
					$Txt.html(txts[0]);
					if (e.type == 'validate') {
						$Field.addClass('invalid');
					}
				} else if (!val.match($.check.PASSWORD)) {
					$Indicator.removeClass();
					$Txt.html(txts[4]);
					if (e.type == 'validate') {
						$Field.addClass('invalid');
					}
				} else if (val.length < 6) {
					$Indicator.removeClass();
					$Txt.html(txts[6]);
					if (e.type == 'validate') {
						$Field.addClass('invalid');
					}
				} else if (name.toLowerCase() == val.toLowerCase()) {
					$Indicator.removeClass();
					$Txt.html(txts[5] || txts[4]);
					if (e.type == 'validate') {
						$Field.addClass('invalid');
					}
				} else {
					var strength = $.check.passwordStrength(val);
					$Indicator.removeClass();
					if (strength < 2) {
						if (e.type == 'validate') {
							$Field.addClass('invalid');
						}
						$Indicator.addClass('indicator passBad');
						$Txt.html(txts[1]);
					} else {
						$Field.addClass('okey');
						$Input.attr('data-valid', 1);

						if (strength < 3) {
							$Indicator.addClass('indicator passNormal');
							$Txt.html(txts[2]);
						} else {
							$Indicator.addClass('indicator passGood');
							$Txt.html(txts[3]);
						}
					}

					// generate password button
					var $generate = $Field.find('._pass-generate');
					$generate[strength < 2 ? 'show' : 'hide']();
				}
			});
		}

		/*
		 * E-Mail Validator
		 */

		function bindMail() {

			if (inputs.mail) {
				return;
			}

			var $Field = $Form.find('.Email');
			var $Input = $Field.find('input');
			var $Error = $Field.find('.error');
			var $Logos = $Field.find('.logos');

			var $Block1 = $Field.find('.e-email_exists');
			var $Block2 = $Field.find('.e-email_exists_deleted');

			var filter = /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z]{2,6})$/;

			var disabled = $Input.attr('disabled');
			var validateRequestFlag = 0;

			function check() {
				if (!$Field.hasClass('invalid')) {
					$Input.trigger('validate');
				}
				if ($Input.attr('data-valid') > 0) {
					return true;
				}
				return false;
			}
			
			$Form.on('click', '#email-details-link', function() {
				
				var data = $Form.serialize(),
					action = $Form.attr('action');
						
				$.post(action.split('?')[0] + '?action=checkNonregistrableEmail', data, function (R) {
					$Field.removeClass('invalid');
				});
			});

			function validate() {
				if (!validateRequestFlag) {
					
					validateRequestFlag = 1;
					
					var data = $Form.serialize(),
						action = $Form.attr('action');

					$Field.addClass('check');
					
					$.post(action + '&check=email', data, function (R) {
						validateRequestFlag = 0;
						$Field.removeClass('check invalid valid okey');

						R = getR(R);

						if (R && R.XForms && R.XForms.Reg) {
							var er = R.XForms.Reg;

							$Input.attr('data-valid', 0);

							if (er.email) {
								$Field.addClass('invalid');
								$Error.html(er.email);
							} else {
								if (er.email_exists) {
									$Block1.slideDown('fast');
								} else if (er.email_exists_deleted) {
									$Error.html('');
									$Field.addClass('invalid');
									$Block2.slideDown('fast');
								}
							}
						} else {
							$Input.attr('data-valid', 2);
							$Field.addClass('okey valid');
						}
					});
				}
			}

			/*
			 * Run
			 */

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.mail = {
					check: check
				};
			} else {
				return;
			}

			$Input.bind('input validate', function (e) {
				var val = $Input.val().trim();
				var match, provider = 'hz';

				$Field.removeClass('valid okey invalid');
				$Input.attr('data-valid', 0);

				if (!val.length) {
					if (e.type == 'validate') {
						$Field.addClass('invalid');
						$Error.html(params.lang.email ? params.lang.email[0] : '');
					}
				} else if (filter.test(val)) {
					if ((match = val.match(/\b[A-Z0-9._%+-]+@([A-Z0-9.-]+)\.[A-Z]{2,6}\b/i)) !== null) {
						provider = match.pop().toLowerCase();
					}
					$Logos.removeClass().addClass('logos logo-' + provider);
					$Input.attr('data-valid', 1);
					$Field.addClass('valid');
				} else {
					$Error.html(params.lang.email ? params.lang.email[1] : '');
					if (e.type == 'validate') {
						$Field.addClass('invalid');
					}
				}

				if ($Block1[0] && $Block1[0].style.display != 'none') {
					$Block1.slideUp('fast');
				}

				if ($Block2[0] && $Block2[0].style.display != 'none') {
					$Block2.slideUp('fast');
				}

				checkField('email', $Input);
			});

			$Input.bind('change blur', function (e) {
				if ($Input.attr('data-valid') > 0) {
					validate();
				} else if ($Input.val().length > 0) {
					$Field.removeClass('valid okey').addClass('invalid');
				}
			});

			if ($Input.val()) {
				if (disabled) {
					$Input.attr('disabled', false);
				}
				$Input.trigger('validate');
				if (disabled) {
					$Input.attr('disabled', true);
				}
			}
		}

		/*
		 * Captcha Validator
		 */

		function bindCaptcha() {

			if (inputs.captcha) {
				return;
			}

			var $Field = $Form.find('.Captcha');
			var $Input = $Field.find('input');

			function check() {
				$Input.trigger('validate');
				if ($Input.attr('data-valid') > 0) {
					return true;
				}
				return false;
			}

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.captcha = {
					check: check
				};
			} else {
				return;
			}

			$Input.bind('input change validate', function (e) {
				var val = $Input.val();
				$Field.removeClass('invalid valid');
				if (val.length > 5 && val.match(/^[0-9]+$/)) {
					$Input.attr('data-valid', 1);
					$Field.addClass('valid');
				} else {
					$Input.attr('data-valid', 0);
					if (e.type == 'change' || e.type == 'validate') {
						$Field.addClass('invalid');
					}
				}
			});
		}

		/*
		 * Name Validator
		 */

		function bindName() {

			if (inputs.name) {
				return;
			}

			var $Field = $Form.find('.Name');
			var $Input = $Field.find('input');
			var $Error = $Field.find('.error');

			function validate() {
				var data = $Form.serialize();
				var action = $Form.attr('action');
				$Field.addClass('check');
				$.post(action + '&check=name', data, function (R) {
					$Field.removeClass('check invalid valid okey');

					R = getR(R);

					if (R && R.XForms && R.XForms.Reg && R.XForms.Reg.name) {
						$Input.attr('data-valid', 0);
						$Error.html(R.XForms.Reg.name);
						$Field.addClass('invalid');
					} else {
						$Input.attr('data-valid', 2);
						$Field.addClass('okey');
					}
				});
			}

			function check() {
				if (!$Field.hasClass('invalid')) {
					$Input.trigger('validate');
				}
				if ($Input.attr('data-valid') > 0) {
					return true;
				}
				return false;
			}

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.name = {
					check: check
				};
			} else {
				return;
			}

			$Input.bind('input change validate', function (e) {
				var val = $Input.val();

				$Field.removeClass('invalid valid');

				if (!val.length) {
					$Input.attr('data-valid', 0);
					$Field.addClass('invalid');
					$Error.html(params.lang.name ? params.lang.name[0] : '');
				} else if (val.length < 2 || val.length > 50) {
					$Input.attr('data-valid', 0);
					$Field.addClass('invalid');
					$Error.html(params.lang.name ? params.lang.name[2] : '');
				} else if (val.match(/[0-9,!,@,#,$,%,\^,&,*,?,~,=,",',<,>,/]/)) {
					$Input.attr('data-valid', 0);
					$Field.addClass('invalid');
					$Error.html(params.lang.name ? params.lang.name[1] : '');
				} else if (!(/^\S*\s?\S*\s?\S*$/).test($.trim(val))) {
					$Input.attr('data-valid', 0);
					$Field.addClass('invalid');
					$Error.html(params.lang.name ? params.lang.name[3] : '');
				} else {
					$Input.attr('data-valid', 1);
					$Field.addClass('valid');
				}

				checkField('name', $Input);
			});

			$Input.bind('change', function (e) {
				if ($Input.attr('data-valid') == 1) {
					validate();
				}
			});

			if ($Input.val()) {
				$Input.trigger('validate');
			}
		}

		/*
		 * Gender Validator
		 */

		function bindGender() {
			
			if (inputs.gender) {
				return;
			}
			

			var $Field = $Form.find('.Gender');
			var $Inputs = $Field.find('input[name=gender],select[name=gender]');

			function check() {
				try {
					var val = version == 3 ? $Inputs.val() : $Inputs.filter(':checked').val() || $Inputs.filter('select').val();
					if (!val || !val.length) {
						$Field.addClass('invalid');
						return false;
					} else {
						$Field.removeClass('invalid');
						return true;
					}
				} catch(e) {
					console.error(e.message)
				}
			}

			if ($Field.length && !$Field.attr('disabled')) {
				inputs.gender = {
					check: check
				};
			}

			if (version == 2) {
				var $lookfor = $('.genderTarget', $Form),
					$lookforwhom = $('input', $lookfor),
					$lookforselect = $('select', $lookfor);

				$lookforselect.val($Inputs.val() == 'M' ? 'F' : 'M').trigger('change');

				$Inputs.bind('change click keyup validate', function (e) {
					$Field.removeClass('invalid');
					var lookfor = $(this).val() == 'M' ? 'F' : 'M';
					$lookfor.show();
					$lookforwhom.filter(':checked').attr({ checked: false });
					$('input[value=' + lookfor + ']', $lookfor).attr({ checked: true });
					$lookforselect.val(lookfor).trigger('change');
				});

				if ($Inputs.filter(':checked').length) {
					$Inputs.filter(':checked').trigger('validate');
				}
			}
			else if (version == 3) {
				
				var $gender = $('.gender-selector', $Field),
					$lookfor = $('.genderSearch', $Field),
					$lookwhom = $('input', $lookfor);

				$gender.on('click', function (e) {
					e.preventDefault();
					var $this = $(this),
						gender = $this.data('gender'),
						lookfor = gender == 'M' ? 'F' : 'M';

					$Field.removeClass('invalid');
					$Inputs.val(gender);

					$gender.removeClass('selected');
					$this.addClass('selected');

					$lookwhom.prop('checked', false).each(function () {
						var $that = $(this);
						$that.prop('checked', $that.val() == lookfor);
					});

					$lookfor.slideDown();
				});
			}
			else {
				$Inputs.bind('change click keyup', function (e) {
					$Field.removeClass('invalid');
				});
			}

		}

		/*
		 * WithWhom Validator
		 */

		function bindWithWhom() {

			if (inputs.withWhom) {
				return;
			}

			var $Field = $Form.find('.WithWhom');
			var $Inputs = $Field.find('input');

			function check() {
				if ($Inputs.filter(':checked').length) {
					$Field.removeClass('invalid');
					return true;
				} else {
					$Field.addClass('invalid');
					return false;
				}
			}

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.withWhom = {
					check: check
				};
			} else {
				return;
			}

			$Inputs.bind('change click keyup', function (e) {
				$Field.removeClass('invalid');
			});
		}

		/*
		 * Agreement Validator
		 */

		function bindAgreement() {

			if (inputs.agreement) {
				return;
			}

			var $Field = $Form.find('.Agreement');
			var $Input = $Field.find('input');

			function check() {
				if ($Input.attr('checked')) {
					$Field.removeClass('invalid');
					return true;
				} else {
					$Field.addClass('invalid');
					return false;
				}
			}

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.agreement = {
					check: check
				};
			} else {
				return;
			}

			$Input.bind('change click keyup', function (e) {
				$Field.removeClass('invalid');
			});
		}

		/*
		 * Location Validator
		 */

		function bindLocation() {

			if (inputs.Location) {
				return;
			}

			var $Field = $Form.find('.Location');
			var $Input = $Field.find('input[type="hidden"]');

			function check() {
				if ($Input.val()) {
					$Field.removeClass('invalid');
					return true;
				} else {
					$Field.addClass('invalid');
					return false;
				}
			}

			if ($Field.length && $Field.css('display') != 'none' && !$Field.attr('disabled')) {
				inputs.Location = {
					check: check
				};
			} else {
				return;
			}

			$Input.bind('change', function (e) {
				$Field.removeClass('invalid');
			});
		}

		/*
		 * Birth Validator
		 */

		function bindBirth() {

			if (inputs.birth) {
				return;
			}

			var $Field = $Form.find('.BirthDate');
			var $Selects = $Field.find('select');
			var $Error = $Field.find('.error');
			var valid = false;
			var isChecked = false;

			function check() {
				$Selects.trigger('validate');
				return valid;
			}

			if ($Selects.length) {
				inputs.birth = {
					check: check
				};
			} else {
				return;
			}


			params.yearsOld = params.yearsOld || 18;
			
			
			
			function changeBirth(e){
				$Selects = $Field.find('select');
				var y = $Selects.filter('[name=birthyear]').val() * 1;
				var m = $Selects.filter('[name=birthmonth]').val() * 1;
				var d = $Selects.filter('[name=birthday]').val() * 1;
				
				//console.log(d, m, y);
				if (e.type == 'validate') {
					isChecked = true;
				}

				$Field.removeClass('invalid');

				if (y && m && d) {
					if (diffYears(y, m, d) < params.yearsOld) {
						$Field.addClass('invalid');
						$Error.html(params.lang.birth ? params.lang.birth[1] : '');
						valid = false;
					} else if (!(d == new Date(y, m - 1, d).getDate())) {
						$Field.addClass('invalid');
						$Error.html(params.lang.birth ? params.lang.birth[2] : '');
						valid = false;
					} else {
						valid = true;
					}
				} else {
					if (isChecked) {
						$Field.addClass('invalid');
						$Error.html(params.lang.birth ? params.lang.birth[0] : '');
					}
					valid = false;
				}
				
			}

            $Form.find('select').on('validate', function(e) {
                changeBirth(e);
            });

            $Form.off('change validate', 'select');
            $Form.on('change', 'select', function (e) {
                changeBirth(e);
            });

			if ($Selects.eq(0).val() * 1 && $Selects.eq(1).val() * 1 && $Selects.eq(2).val() * 1) {
				$Selects.trigger('change');
			}

			function diffYears(Y, M, D) {
				var date = new Date();
				var cM = date.getMonth() + 1;
				Y = date.getFullYear() - Y;
				if ((M > cM) || (D > date.getDate() && M == cM)) {
					Y--;
				}
				return Y;
			}

			function init() {
				var y = $Selects.filter('[name=birthyear]').val() * 1;
				var m = $Selects.filter('[name=birthmonth]').val() * 1;
				var d = $Selects.filter('[name=birthday]').val() * 1;
				
				if (y && m && d) {
					$Selects.trigger('validate');
				}
			}

			init();
		}

		/*
		 * Phone Validator
		 */

		function bindPhone() {
			var $Field = $Form.find('.Phone');
			var $Inputs = $Field.find('input');

			$Inputs.bind("keydown", function (e) {
				var $this = $(this);
				var value = $this.val();

				if (!(e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105)) {
					e.preventDefault();
				}
			});
		}

		function checkForm(f) {

			$Form.find('.v-target').removeClass('invalid');

			var er = '';
			var i;

			if (f) {
				for (i in f) {
					if (f.hasOwnProperty(i)) {
						var $Input = $Form.find('[name="' + i + '"]');
						if ($Input.length) {
							$Input.parents('.v-target').removeClass('valid okey').addClass('invalid').find('.error').html(f[i]);

						} else {
							er += '<p>' + f[i] + '</p>';
						}

						switch (i) {
							case 'reg_captcha':
								goToStep(1);
								$Form.find('.Captcha img').trigger('click');
								$Input.val('').select();
								break;
							case 'birthdate':
								$Form.find('.BirthDate').removeClass('valid okey').addClass('invalid').find('.error').html(f[i]);
								break;
							default:
						}
					}
				}

				if (er.length) {
					mambo.ui.Alert(er, {
						type: 'error'
					});
				} else {
					scrollTop();
				}
			}

			var $Step;

			if (step == 1) {
				$Step = $Step1;
			}
			if (step == 2) {
				$Step = $Step2;
			}
		}

		function goToStep(s) {
			var prev = step;
			step = s;

			if (!$Step2.length) {
				step = prev = s = 0;
			}
			
			if (s == 0) {
				bindLogin();
				bindPassword();
				bindMail();
				bindCaptcha();
				bindName();
				bindGender();
				bindBirth();
				bindWithWhom();
				bindAgreement();
				bindLocation();

				if (version == 2) {
					bindPhone();
				}
			}

			if (s == 1) {
				$Steps.removeClass('RegStep2');

				if (prev == 2) {
					$Step2.slideUp('fast');
					$Step1.slideDown('fast');
				} else {
					$Step2.hide();
					$Step1.show();
				}

				bindLogin();
				bindPassword();
				bindMail();
				bindCaptcha();

				if (prev == 2) {
					scrollTop();
				}
			}

			if (s == 2) {
				$Steps.addClass('RegStep2');

				if (prev == 1) {
					$Step1.slideUp('fast');
					$Step2.slideDown('fast');
				} else {
					$Step1.hide();
					$Step2.show();
				}

				bindName();
				bindGender();
				bindBirth();
				bindLocation();
				bindWithWhom();
				bindAgreement();

				if (prev == 1) {
					scrollTop();
				}
			}
		}

		function checkField(k, o) {
			checkFields[k] = o.parents('.v-target').hasClass('valid');

			if (version == 3) {
				if (checkFields.email && checkFields.name) {
					$('#RegBlockForm').slideDown();
					$('.js-form').addClass('show_more');
				}
			}
		}

		var topSlidePanelHeight = $('.ui-userbar-cont').outerHeight() || 0;// need for scrollTo correction
		function scrollTop(top) {
			var $Frame = ($.browser.webkit) ? $('body') : $('html');
			if ($Frame.scrollTop() > (190 - topSlidePanelHeight)) {
				$Frame.animate({ scrollTop: (190 - topSlidePanelHeight) }, 'slow', 'easeOutBack');
			}
		}

		function getR(R) {
			if (R && R.responseText) {
				try {
					eval('R = ' + R.responseText);
				} catch (e) {
					if (window.stats) {
						stats.jsErrorPush(e);
					}
				}
			}
			return R;
		}

		/*
		 * Run
		 */
		function disableFormSubmit(_form, _submit) {
			_form.data('inProgess', 1);
			_submit.css('opacity', 0.5).attr('disabled', true);
		}

		function enableFormSubmit(_form, _submit) {
			_form.data('inProgess', 0);
			_submit.css('opacity', 1).attr('disabled', false);
		}

		function isSubmitDisabled(_form) {
			return _form.data('inProgess');
		}

		$Form.bind('submit', function (e) {
            if (isSubmitDisabled($Form)) return false;
			var data = $Form.serialize();
			var action = $Form.attr('action');
			var go = true;
			var i;

			for (i in inputs) {
				if (inputs.hasOwnProperty(i)) {
					var k = inputs[i];
					if (k && k.check && !k.check()) {
						go = false;
					}
				}
			}

			if (!go) {
				return false;
			}
			disableFormSubmit($Form, $Submit);
			$Form.triggerHandler('regAjaxStart');

			$.ajax({
				url: $Form.attr('action'),
				type: "POST",
				dataType: "json",
				cache: false,
				data: data,
				success: function (R) {
					enableFormSubmit($Form, $Submit);
					R = getR(R);

					if (R.r) {
						window.location.href = R.r;
						return;
					}

					if (R && R.XForms && R.XForms.Reg) {
						checkForm(R.XForms.Reg);
					} else {
						if (step == 1) {
							goToStep(2);
						}
						$Form.trigger('response.registration', {status: R.s, response: R})
					}
				},
				error: function () {
					enableFormSubmit($Form, $Submit);
				},
				complete: function () {
					$Form.triggerHandler('regAjaxComplete');
				}
			});

			e.preventDefault();
		});
		
		//$('#RegBlockForm .inputBirtday select').select2();

		(function (context) {
			"use strict";


			var inputBirtday = context.find('.inputBirtday');
			var yearSelect = inputBirtday.find('.year');
			var monthSelect = inputBirtday.find('.month');
			var daySelect = inputBirtday.find('.dayKo');
			daySelect.show();
			
			if (params.fast11 || params.fast12) {
				daySelect = inputBirtday.find('.day');
			}

			// разные формы регистрации
			if(params.fast9 || params.fast10) {
				yearSelect.select2()
					.parent()
					.find('.select2-container')
					.addClass('__custom-select year');
				monthSelect.select2()
					.parent()
					.find('.select2-container')
					.addClass('__custom-select month');
				daySelect.select2()
					.parent()
					.find('.select2-container')
					.addClass('__custom-select day')
					.find('.select2-selection')
					.addClass('__custom-selectInner')
			} else {
				daySelect.select2();
				monthSelect.select2();
				yearSelect.select2();
			}

			var birthdayObject = {
				dayLength: ko.observable(false),
				year: ko.observable(),
				monthTmp: ko.observable(),
				month: ko.observable(),
				day: ko.observable(),
				birthday: ko.observable(),
				test: ko.observable()
			};


			/**
			 * 
			 * @param year
			 * @param month
			 */
			function updateDays(year, month) {
				"use strict";
				
				var date = moment();
				date.year(year);
				date.month(month - 1);
				var countDay = date.isValid();


				if (!countDay) return;

				countDay = date.daysInMonth();

				// сохраняем выбранное значение даты
				// так как селект с датой постоянно обновляется - нужно выставлять прошлое число
				// при условии если прошлое значение не больше количества дней в месяце
				// например февраль
				var oldDate = birthdayObject.test();
				if ((!oldDate || oldDate) && (month == 2 && oldDate > countDay)) {
					setTimeout(function () {
						birthdayObject.test(0);
					}, 0)
				}


				// прячем первый select с класом .dayKo
				if (daySelect) {
					daySelect.select2('destroy');
					birthdayObject.dayLength(true);
				}


				// создаем новый
				var arr = [];
				for (var i = 0; i < countDay; i++) {
					var val = (i + 1);
					arr.push({
						name: val,
						value: val
					});
				}
				birthdayObject.birthday(arr);


				if (daySelect) {
					birthdayObject.test(birthdayObject.monthTmp());
					daySelect = null;
					
					// разные формы регистрации
					if(params.fast9 || params.fast10) {
						inputBirtday.find('select.day')
						.select2()
						.parent()
						.find('.select2-container')
						.addClass('__custom-select day')
						.find('.select2-selection')
						.addClass('__custom-selectInner');
					} else {
						inputBirtday.find('select.day').select2()
					}
					
				}
			}

			birthdayObject.year.subscribe(function (newValue) {
				if (parseInt(newValue)) {
					var year = parseInt(newValue);
					var month = parseInt(birthdayObject.month());
					if (year && month) {
						updateDays(year, month);
					}
				}
			});
			birthdayObject.month.subscribe(function (newValue) {
				if (parseInt(newValue)) {
					var year = parseInt(birthdayObject.year());
					var month = parseInt(newValue);
					if (year && month) {
						updateDays(year, month)
					}
				}
			});

			ko.applyBindings(birthdayObject, inputBirtday[0]);
		}($(id)));
		
		goToStep(step);
	};

	mambo.Loader.loaded('{mambo.project}Registration');

}, true);
